DECLARE FUNCTION LJEnergy! (R AS DOUBLE)
'-------------------------------- Function02 -----------------------------'
'                                                                         '
' This program compares the use of inline functions to function calling.  '
' An empty function is used in this demo, but 'simulates' calling a       '
' Lennard-Jones pair energy function.                                     '
'                                                                         '
'-------------------------------------------------------------------------'

' global declarations
'longs
DIM counter     AS LONG         'iteration counter
DIM NumIts      AS LONG         'number of iterations

'singles
DIM starttimer  AS SINGLE       'ticks at start of loop
DIM stoptimer   AS SINGLE       'ticks at loop exit

'doubles
DIM Rval        AS DOUBLE       'value to pass in function call
DIM E           AS DOUBLE       'return value for function call/computation

'clear the screen
CLS

'initialize the iteration counter
NumIts = 10000000

'made-up R for demo
Rval = 1.234


'-------------------------------- Called Function ------------------

'display message
PRINT "Calling a Function ..."
PRINT

'start the timer
starttimer = TIMER

FOR counter = 1 TO NumIts

	'call the function
	E = LJEnergy(Rval)

NEXT counter

'stop the timer
stoptimer = TIMER

'display the result
PRINT "Called function "; NumIts; " times in: "; stoptimer - starttimer; " seconds"
PRINT

'-------------------------------- End Called Function ----------------------

'-------------------------------- Inline Function --------------------------

'display message
PRINT "Computing with inline code ..."
PRINT

'start the timer
starttimer = TIMER

	DIM Rinline     AS DOUBLE
	DIM r6inline    AS DOUBLE
	DIM r12inline   AS DOUBLE
	DIM Einline     AS DOUBLE

FOR counter = 1 TO NumIts

	'compute r^6
	r6inline = Rval * Rval * Rval * Rval * Rval * Rval

	'compute r^12
	r12inline = r6inline * r6inline

	'compute Energy
	Einline = 1 / r12inline - 1 / r6inline

NEXT counter

'stop the timer
stoptimer = TIMER

'display the result
PRINT "Inline Code Executed "; NumIts; " times in: "; stoptimer - starttimer; " seconds"
PRINT

' ------------------------------- End Inline Function ----------------------

' Keep display active until user presses a key
' this is needed when running in the dos subsystem of windows, else
' windows closes window immediately

' display a message
PRINT : PRINT "Press any key to end..."

'check keylatch; if empty, simply loop back
LoopHere:
A$ = INKEY$
IF A$ = "" THEN GOTO LoopHere

'end when keylatch contains any key
END

FUNCTION LJEnergy (R AS DOUBLE)

	'local declarations
	'doubles
	DIM r6   AS DOUBLE
	DIM r12  AS DOUBLE

	'compute r^6
	r6 = R * R * R * R * R * R

	'compute r^12
	r12 = r6 * r6

	'compute E
	LJEnergy = 1 / r12 - 1 / r6


END FUNCTION

