'-------------------------------- Loop03 ---------------------------------'
'                                                                         '
' This program compares execution times for using a generic nxn matrix    '
' multiplication to explicit functions, such as 4x4 and 8x8.  The         '
' explicit functions can be unrolled                                      '
'                                                                         '
' NOTE:  This program differs from the Windows versions (VB and VC++)     '
'        since the old DOS compiler cannot use arrays larger than 64 kB.  '
'        Therefore, only 15,000 random numbers are used, compared to the  '
'        500,000 done in the other versions (and depicted in the book).   '
'                                                                         '
'        Also, only 50,000 iterations are done rather than 500,000.       '
'                                                                         '
'-------------------------------------------------------------------------'

'procedure declarations
DECLARE SUB nxn.Matrix.Mult (A() AS SINGLE, B() AS SINGLE, C() AS SINGLE, dimen AS LONG)
DECLARE SUB Matrix.Mult.8x8.Rolled (A() AS SINGLE, B() AS SINGLE, C() AS SINGLE)
DECLARE SUB Matrix.Mult.4x4.Rolled (A() AS SINGLE, B() AS SINGLE, C() AS SINGLE)
DECLARE SUB Matrix.Mult.8x8.UnRolled (A() AS SINGLE, B() AS SINGLE, C() AS SINGLE)
DECLARE SUB Matrix.Mult.4x4.UnRolled (A() AS SINGLE, B() AS SINGLE, C() AS SINGLE)

'global declarations
'longs
DIM NumIts              AS LONG         'number of iterations
DIM counter             AS LONG         'iteration counter
DIM MTypes(15000)       AS LONG         'holder for random 4's and 8's
DIM called8             AS LONG         'number of times 8x8 is called
DIM called4             AS LONG         'number of times 4x4 is called
DIM j                   AS LONG         'j loop index
DIM k                   AS LONG         'k loop index
DIM l                   AS LONG         'l loop index

'singles
DIM A(8, 8)             AS SINGLE       'Matrix A
DIM B(8, 8)             AS SINGLE       'Matrix B
DIM C(8, 8)             AS SINGLE       'Matrix C = A * B
DIM starttimer          AS SINGLE       'ticks at loop start
DIM stoptimer           AS SINGLE       'ticks at loop exit

'---------------------------- Initializations ------------------------------

'max 50000 iterations
NumIts = 50000

'initialize A and B matrices
'set A and B to 8x8 matrices
'for the 4x4 computations, just use
'the 4x4 portion in upper left 'quadrant'
FOR j = 1 TO 8
			
	FOR k = 1 TO 8
		
	    'assign random values to A and B to 'simulate'
	    'a real calculation
	    A(j, k) = 20000 * RND
	    B(j, k) = 20000 * RND

	NEXT k
	
NEXT j

    
'assign random 4 or 8 to MTypes, which will be used
'to select 4x4 or 8x8
FOR counter = 0 TO 14999
    
	MTypes(counter) = INT(2 * RND) + 1
	
	'assign actual dimension to array for the nxn
	'call without decision logic
	IF MTypes(counter) = 1 THEN MTypes(counter) = 8
	IF MTypes(counter) = 2 THEN MTypes(counter) = 4

NEXT counter


'------------------------------- End Initializations ---------------------

'clear the screen
CLS

'-------------------- nxn Matrix Multiplication Wrapper ------------------

'display message
PRINT "Computing Matrix Multiplication with generic, nxn function"
PRINT "With Decision Logic ..."
PRINT

'clear the call counters
called8 = 0
called4 = 0

'start the timer
starttimer = TIMER

FOR counter = 1 TO NumIts

	'use if/then to send dimension to function
	IF MTypes(counter MOD 15000) = 8 THEN
		
		CALL nxn.Matrix.Mult(A(), B(), C(), 8)
		'increment the 8x8 counter
		called8 = called8 + 1

	ELSE

		CALL nxn.Matrix.Mult(A(), B(), C(), 4)
		'increment the 4x4 counter
		called4 = called4 + 1

	END IF

NEXT counter

'stop the timer
stoptimer = TIMER

'display the result
PRINT "nxn( 8 ) called "; called8; " times"
PRINT "nxn( 4 ) called "; called4; " times"
PRINT "in "; stoptimer - starttimer; " seconds"
PRINT

'do the rolled generic calculation without decision logic...passing
'value to function directly
'the call counters are not implemented here

'display message
PRINT "Computing nxn Matrix Multiplication"
PRINT "Without decision logic ..."
PRINT

'start the timer
starttimer = TIMER

FOR counter = 1 TO NumIts

	'call generic routine, passing array value directly as dimension
	CALL nxn.Matrix.Mult(A(), B(), C(), MTypes(counter MOD 15000))

NEXT counter

'stop the timer
stoptimer = TIMER

'display result
PRINT "Generic nxn(n) matrix multiplication executed in "
PRINT stoptimer - starttimer; " seconds"
PRINT

'------------------------ End nxn Matrix Multiplication Wrapper ------------

'----------------- Explicit Rolled Matrix Multiplication Wrapper -----------

'display message
PRINT "Computing with Explicit Matrix Multiplication Functions"
PRINT "With ROLLED-UP Loops ..."
PRINT

'clear the call counters
called8 = 0
called4 = 0

'start the timer
starttimer = TIMER

FOR counter = 1 TO NumIts

	'select which function to call
	IF MTypes(counter MOD 15000) = 8 THEN

		'call the explicit rolled function
		CALL Matrix.Mult.8x8.Rolled(A(), B(), C())
		'increment the 8x8 counter
		called8 = called8 + 1

	ELSE


		'call the explicit rolled function
		CALL Matrix.Mult.4x4.Rolled(A(), B(), C())
		'increment the 4x4 counter
		called4 = called4 + 1

	END IF

NEXT counter

'stop the timer
stoptimer = TIMER

'display the result
PRINT "Rolled 8x8 Function called "; called8; " times"
PRINT "Rolled 4x4 Function called "; called4; " times"
PRINT "Total Time: "; stoptimer - starttimer; " seconds"
PRINT

'------------- End Explicit Rolled Matrix Multiplication Wrapper -----------

'--------------- Explicit UnRolled Matrix Multiplication Wrapper -----------

'display message
PRINT "Computing with Explicit Matrix Multiplication Functions"
PRINT "With UnRolled Loops ..."
PRINT

'clear the call counters
called8 = 0
called4 = 0

'start the timer
starttimer = TIMER

FOR counter = 1 TO NumIts

	'select which function to call
	IF MTypes(counter MOD 15000) = 8 THEN

		'call the explicit unrolled function
		CALL Matrix.Mult.8x8.UnRolled(A(), B(), C())
		'increment the 8x8 counter
		called8 = called8 + 1

	ELSE

		'call the explicit unrolled function
		CALL Matrix.Mult.4x4.UnRolled(A(), B(), C())
		'increment the 4x4 counter
		called4 = called4 + 1

	END IF

NEXT counter

'stop the timer
stoptimer = TIMER

'display the result
PRINT "Unrolled 8x8 Function called "; called8; " times"
PRINT "Unrolled 4x4 Function called "; called4; " times"
PRINT "Total Time: "; stoptimer - starttimer; " seconds"
PRINT

'----------- End Explicit UnRolled Matrix Multiplication Wrapper -----------

' Function Calls are Below

' Keep display active until user presses a key
' this is needed when running in the dos subsystem of windows, else
' windows closes window immediately

' display a message
PRINT : PRINT "Press any key to end..."

'check keylatch; if empty, simply loop back
LoopHere:
In$ = INKEY$
IF In$ = "" THEN GOTO LoopHere

'end when keylatch contains any key
END


' This function computes the multiplication of two 4x4 matrices
' As written, all parameters are globals


' This function computes the multiplication of two 8x8 matrices
' As written, all parameters are globals

SUB Matrix.Mult.4x4.Rolled (A() AS SINGLE, B() AS SINGLE, C() AS SINGLE)
	'loop over j
	FOR j = 1 TO 4

		'loop over k
		FOR k = 1 TO 4

			'zero C(j,k)
			C(j, k) = 0

			'loop over l
			FOR l = 1 TO 4

				C(j, k) = C(j, k) + A(j, l) * B(l, k)

			NEXT l

		NEXT k

	NEXT j

END SUB

SUB Matrix.Mult.4x4.UnRolled (A() AS SINGLE, B() AS SINGLE, C() AS SINGLE)
    'j = 1
    C(1, 1) = A(1, 1) * B(1, 1) + A(1, 2) * B(2, 1) + A(1, 3) * B(3, 1) + A(1, 4) * B(4, 1)
    C(1, 2) = A(1, 1) * B(1, 2) + A(1, 2) * B(2, 2) + A(1, 3) * B(3, 2) + A(1, 4) * B(4, 2)
    C(1, 3) = A(1, 1) * B(1, 3) + A(1, 2) * B(2, 3) + A(1, 3) * B(3, 3) + A(1, 4) * B(4, 3)
    C(1, 4) = A(1, 1) * B(1, 4) + A(1, 2) * B(2, 4) + A(1, 3) * B(3, 4) + A(1, 4) * B(4, 4)
    
    'j = 2
    C(2, 1) = A(2, 1) * B(1, 1) + A(2, 2) * B(2, 1) + A(2, 3) * B(3, 1) + A(2, 4) * B(4, 1)
    C(2, 2) = A(2, 1) * B(1, 2) + A(2, 2) * B(2, 2) + A(2, 3) * B(3, 2) + A(2, 4) * B(4, 2)
    C(2, 3) = A(2, 1) * B(1, 3) + A(2, 2) * B(2, 3) + A(2, 3) * B(3, 3) + A(2, 4) * B(4, 3)
    C(2, 4) = A(2, 1) * B(1, 4) + A(2, 2) * B(2, 4) + A(2, 3) * B(3, 4) + A(2, 4) * B(4, 4)
    
    'j=3
    C(3, 1) = A(3, 1) * B(1, 1) + A(3, 2) * B(2, 1) + A(3, 3) * B(3, 1) + A(3, 4) * B(4, 1)
    C(3, 2) = A(3, 1) * B(1, 2) + A(3, 2) * B(2, 2) + A(3, 3) * B(3, 2) + A(3, 4) * B(4, 2)
    C(3, 3) = A(3, 1) * B(1, 3) + A(3, 2) * B(2, 3) + A(3, 3) * B(3, 3) + A(3, 4) * B(4, 3)
    C(3, 4) = A(3, 1) * B(1, 4) + A(3, 2) * B(2, 4) + A(3, 3) * B(3, 4) + A(3, 4) * B(4, 4)
    
    'j=4
    C(4, 1) = A(4, 1) * B(1, 1) + A(4, 2) * B(2, 1) + A(4, 3) * B(3, 1) + A(4, 4) * B(4, 1)
    C(4, 2) = A(4, 1) * B(1, 2) + A(4, 2) * B(2, 2) + A(4, 3) * B(3, 2) + A(4, 4) * B(4, 2)
    C(4, 3) = A(4, 1) * B(1, 3) + A(4, 2) * B(2, 3) + A(4, 3) * B(3, 3) + A(4, 4) * B(4, 3)
    C(4, 4) = A(4, 1) * B(1, 4) + A(4, 2) * B(2, 4) + A(4, 3) * B(3, 4) + A(4, 4) * B(4, 4)
    
END SUB

SUB Matrix.Mult.8x8.Rolled (A() AS SINGLE, B() AS SINGLE, C() AS SINGLE)
	'loop over j
	FOR j = 1 TO 8

		'loop over k
		FOR k = 1 TO 8

			'zero C(j,k)
			C(j, k) = 0

			'loop over l
			FOR l = 1 TO 8

				C(j, k) = C(j, k) + A(j, l) * B(l, k)

			NEXT l

		NEXT k

	NEXT j

END SUB

SUB Matrix.Mult.8x8.UnRolled (A() AS SINGLE, B() AS SINGLE, C() AS SINGLE)
    'j = 1
    C(1, 1) = A(1, 1) * B(1, 1) + A(1, 2) * B(2, 1) + A(1, 3) * B(3, 1) + A(1, 4) * B(4, 1) + A(1, 5) * B(5, 1) + A(1, 6) * B(6, 1) + A(1, 7) * B(7, 1) + A(1, 8) * B(8, 1)
    C(1, 2) = A(1, 1) * B(1, 2) + A(1, 2) * B(2, 2) + A(1, 3) * B(3, 2) + A(1, 4) * B(4, 2) + A(1, 5) * B(5, 2) + A(1, 6) * B(6, 2) + A(1, 7) * B(7, 2) + A(1, 8) * B(8, 2)
    C(1, 3) = A(1, 1) * B(1, 3) + A(1, 2) * B(2, 3) + A(1, 3) * B(3, 3) + A(1, 4) * B(4, 3) + A(1, 5) * B(5, 3) + A(1, 6) * B(6, 3) + A(1, 7) * B(7, 3) + A(1, 8) * B(8, 3)
    C(1, 4) = A(1, 1) * B(1, 4) + A(1, 2) * B(2, 4) + A(1, 3) * B(3, 4) + A(1, 4) * B(4, 4) + A(1, 5) * B(5, 4) + A(1, 6) * B(6, 4) + A(1, 7) * B(7, 4) + A(1, 8) * B(8, 4)
    C(1, 5) = A(1, 1) * B(1, 5) + A(1, 2) * B(2, 5) + A(1, 3) * B(3, 5) + A(1, 4) * B(4, 5) + A(1, 5) * B(5, 5) + A(1, 6) * B(6, 5) + A(1, 7) * B(7, 5) + A(1, 8) * B(8, 5)
    C(1, 6) = A(1, 1) * B(1, 6) + A(1, 2) * B(2, 6) + A(1, 3) * B(3, 6) + A(1, 4) * B(4, 6) + A(1, 5) * B(5, 6) + A(1, 6) * B(6, 6) + A(1, 7) * B(7, 6) + A(1, 8) * B(8, 6)
    C(1, 7) = A(1, 1) * B(1, 7) + A(1, 2) * B(2, 7) + A(1, 3) * B(3, 7) + A(1, 4) * B(4, 7) + A(1, 5) * B(5, 7) + A(1, 6) * B(6, 7) + A(1, 7) * B(7, 7) + A(1, 8) * B(8, 7)
    C(1, 8) = A(1, 1) * B(1, 8) + A(1, 2) * B(2, 8) + A(1, 3) * B(3, 8) + A(1, 4) * B(4, 8) + A(1, 5) * B(5, 8) + A(1, 6) * B(6, 8) + A(1, 7) * B(7, 8) + A(1, 8) * B(8, 8)
    
    'j = 2
    C(2, 1) = A(2, 1) * B(1, 1) + A(2, 2) * B(2, 1) + A(2, 3) * B(3, 1) + A(2, 4) * B(4, 1) + A(2, 5) * B(5, 1) + A(2, 6) * B(6, 1) + A(2, 7) * B(7, 1) + A(2, 8) * B(8, 1)
    C(2, 2) = A(2, 1) * B(1, 2) + A(2, 2) * B(2, 2) + A(2, 3) * B(3, 2) + A(2, 4) * B(4, 2) + A(2, 5) * B(5, 2) + A(2, 6) * B(6, 2) + A(2, 7) * B(7, 2) + A(2, 8) * B(8, 2)
    C(2, 3) = A(2, 1) * B(1, 3) + A(2, 2) * B(2, 3) + A(2, 3) * B(3, 3) + A(2, 4) * B(4, 3) + A(2, 5) * B(5, 3) + A(2, 6) * B(6, 3) + A(2, 7) * B(7, 3) + A(2, 8) * B(8, 3)
    C(2, 4) = A(2, 1) * B(1, 4) + A(2, 2) * B(2, 4) + A(2, 3) * B(3, 4) + A(2, 4) * B(4, 4) + A(2, 5) * B(5, 4) + A(2, 6) * B(6, 4) + A(2, 7) * B(7, 4) + A(2, 8) * B(8, 4)
    C(2, 5) = A(2, 1) * B(1, 5) + A(2, 2) * B(2, 5) + A(2, 3) * B(3, 5) + A(2, 4) * B(4, 5) + A(2, 5) * B(5, 5) + A(2, 6) * B(6, 5) + A(2, 7) * B(7, 5) + A(2, 8) * B(8, 5)
    C(2, 6) = A(2, 1) * B(1, 6) + A(2, 2) * B(2, 6) + A(2, 3) * B(3, 6) + A(2, 4) * B(4, 6) + A(2, 5) * B(5, 6) + A(2, 6) * B(6, 6) + A(2, 7) * B(7, 6) + A(2, 8) * B(8, 6)
    C(2, 7) = A(2, 1) * B(1, 7) + A(2, 2) * B(2, 7) + A(2, 3) * B(3, 7) + A(2, 4) * B(4, 7) + A(2, 5) * B(5, 7) + A(2, 6) * B(6, 7) + A(2, 7) * B(7, 7) + A(2, 8) * B(8, 7)
    C(2, 8) = A(2, 1) * B(1, 8) + A(2, 2) * B(2, 8) + A(2, 3) * B(3, 8) + A(2, 4) * B(4, 8) + A(2, 5) * B(5, 8) + A(2, 6) * B(6, 8) + A(2, 7) * B(7, 8) + A(2, 8) * B(8, 8)
    
    'j=3
    C(3, 1) = A(3, 1) * B(1, 1) + A(3, 2) * B(2, 1) + A(3, 3) * B(3, 1) + A(3, 4) * B(4, 1) + A(3, 5) * B(5, 1) + A(3, 6) * B(6, 1) + A(3, 7) * B(7, 1) + A(3, 8) * B(8, 1)
    C(3, 2) = A(3, 1) * B(1, 2) + A(3, 2) * B(2, 2) + A(3, 3) * B(3, 2) + A(3, 4) * B(4, 2) + A(3, 5) * B(5, 2) + A(3, 6) * B(6, 2) + A(3, 7) * B(7, 2) + A(3, 8) * B(8, 2)
    C(3, 3) = A(3, 1) * B(1, 3) + A(3, 2) * B(2, 3) + A(3, 3) * B(3, 3) + A(3, 4) * B(4, 3) + A(3, 5) * B(5, 3) + A(3, 6) * B(6, 3) + A(3, 7) * B(7, 3) + A(3, 8) * B(8, 3)
    C(3, 4) = A(3, 1) * B(1, 4) + A(3, 2) * B(2, 4) + A(3, 3) * B(3, 4) + A(3, 4) * B(4, 4) + A(3, 5) * B(5, 4) + A(3, 6) * B(6, 4) + A(3, 7) * B(7, 4) + A(3, 8) * B(8, 4)
    C(3, 5) = A(3, 1) * B(1, 5) + A(3, 2) * B(2, 5) + A(3, 3) * B(3, 5) + A(3, 4) * B(4, 5) + A(3, 5) * B(5, 5) + A(3, 6) * B(6, 5) + A(3, 7) * B(7, 5) + A(3, 8) * B(8, 5)
    C(3, 6) = A(3, 1) * B(1, 6) + A(3, 2) * B(2, 6) + A(3, 3) * B(3, 6) + A(3, 4) * B(4, 6) + A(3, 5) * B(5, 6) + A(3, 6) * B(6, 6) + A(3, 7) * B(7, 6) + A(3, 8) * B(8, 6)
    C(3, 7) = A(3, 1) * B(1, 7) + A(3, 2) * B(2, 7) + A(3, 3) * B(3, 7) + A(3, 4) * B(4, 7) + A(3, 5) * B(5, 7) + A(3, 6) * B(6, 7) + A(3, 7) * B(7, 7) + A(3, 8) * B(8, 7)
    C(3, 8) = A(3, 1) * B(1, 8) + A(3, 2) * B(2, 8) + A(3, 3) * B(3, 8) + A(3, 4) * B(4, 8) + A(3, 5) * B(5, 8) + A(3, 6) * B(6, 8) + A(3, 7) * B(7, 8) + A(3, 8) * B(8, 8)
    
    'j=4
    C(4, 1) = A(4, 1) * B(1, 1) + A(4, 2) * B(2, 1) + A(4, 3) * B(3, 1) + A(4, 4) * B(4, 1) + A(4, 5) * B(5, 1) + A(4, 6) * B(6, 1) + A(4, 7) * B(7, 1) + A(4, 8) * B(8, 1)
    C(4, 2) = A(4, 1) * B(1, 2) + A(4, 2) * B(2, 2) + A(4, 3) * B(3, 2) + A(4, 4) * B(4, 2) + A(4, 5) * B(5, 2) + A(4, 6) * B(6, 2) + A(4, 7) * B(7, 2) + A(4, 8) * B(8, 2)
    C(4, 3) = A(4, 1) * B(1, 3) + A(4, 2) * B(2, 3) + A(4, 3) * B(3, 3) + A(4, 4) * B(4, 3) + A(4, 5) * B(5, 3) + A(4, 6) * B(6, 3) + A(4, 7) * B(7, 3) + A(4, 8) * B(8, 3)
    C(4, 4) = A(4, 1) * B(1, 4) + A(4, 2) * B(2, 4) + A(4, 3) * B(3, 4) + A(4, 4) * B(4, 4) + A(4, 5) * B(5, 4) + A(4, 6) * B(6, 4) + A(4, 7) * B(7, 4) + A(4, 8) * B(8, 4)
    C(4, 5) = A(4, 1) * B(1, 5) + A(4, 2) * B(2, 5) + A(4, 3) * B(3, 5) + A(4, 4) * B(4, 5) + A(4, 5) * B(5, 5) + A(4, 6) * B(6, 5) + A(4, 7) * B(7, 5) + A(4, 8) * B(8, 5)
    C(4, 6) = A(4, 1) * B(1, 6) + A(4, 2) * B(2, 6) + A(4, 3) * B(3, 6) + A(4, 4) * B(4, 6) + A(4, 5) * B(5, 6) + A(4, 6) * B(6, 6) + A(4, 7) * B(7, 6) + A(4, 8) * B(8, 6)
    C(4, 7) = A(4, 1) * B(1, 7) + A(4, 2) * B(2, 7) + A(4, 3) * B(3, 7) + A(4, 4) * B(4, 7) + A(4, 5) * B(5, 7) + A(4, 6) * B(6, 7) + A(4, 7) * B(7, 7) + A(4, 8) * B(8, 7)
    C(4, 8) = A(4, 1) * B(1, 8) + A(4, 2) * B(2, 8) + A(4, 3) * B(3, 8) + A(4, 4) * B(4, 8) + A(4, 5) * B(5, 8) + A(4, 6) * B(6, 8) + A(4, 7) * B(7, 8) + A(4, 8) * B(8, 8)
    
    'j=5
    C(5, 1) = A(5, 1) * B(1, 1) + A(5, 2) * B(2, 1) + A(5, 3) * B(3, 1) + A(5, 4) * B(4, 1) + A(5, 5) * B(5, 1) + A(5, 6) * B(6, 1) + A(5, 7) * B(7, 1) + A(5, 8) * B(8, 1)
    C(5, 2) = A(5, 1) * B(1, 2) + A(5, 2) * B(2, 2) + A(5, 3) * B(3, 2) + A(5, 4) * B(4, 2) + A(5, 5) * B(5, 2) + A(5, 6) * B(6, 2) + A(5, 7) * B(7, 2) + A(5, 8) * B(8, 2)
    C(5, 3) = A(5, 1) * B(1, 3) + A(5, 2) * B(2, 3) + A(5, 3) * B(3, 3) + A(5, 4) * B(4, 3) + A(5, 5) * B(5, 3) + A(5, 6) * B(6, 3) + A(5, 7) * B(7, 3) + A(5, 8) * B(8, 3)
    C(5, 4) = A(5, 1) * B(1, 4) + A(5, 2) * B(2, 4) + A(5, 3) * B(3, 4) + A(5, 4) * B(4, 4) + A(5, 5) * B(5, 4) + A(5, 6) * B(6, 4) + A(5, 7) * B(7, 4) + A(5, 8) * B(8, 4)
    C(5, 5) = A(5, 1) * B(1, 5) + A(5, 2) * B(2, 5) + A(5, 3) * B(3, 5) + A(5, 4) * B(4, 5) + A(5, 5) * B(5, 5) + A(5, 6) * B(6, 5) + A(5, 7) * B(7, 5) + A(5, 8) * B(8, 5)
    C(5, 6) = A(5, 1) * B(1, 6) + A(5, 2) * B(2, 6) + A(5, 3) * B(3, 6) + A(5, 4) * B(4, 6) + A(5, 5) * B(5, 6) + A(5, 6) * B(6, 6) + A(5, 7) * B(7, 6) + A(5, 8) * B(8, 6)
    C(5, 7) = A(5, 1) * B(1, 7) + A(5, 2) * B(2, 7) + A(5, 3) * B(3, 7) + A(5, 4) * B(4, 7) + A(5, 5) * B(5, 7) + A(5, 6) * B(6, 7) + A(5, 7) * B(7, 7) + A(5, 8) * B(8, 7)
    C(5, 8) = A(5, 1) * B(1, 8) + A(5, 2) * B(2, 8) + A(5, 3) * B(3, 8) + A(5, 4) * B(4, 8) + A(5, 5) * B(5, 8) + A(5, 6) * B(6, 8) + A(5, 7) * B(7, 8) + A(5, 8) * B(8, 8)
    
    'j = 6
    C(6, 1) = A(6, 1) * B(1, 1) + A(6, 2) * B(2, 1) + A(6, 3) * B(3, 1) + A(6, 4) * B(4, 1) + A(6, 5) * B(5, 1) + A(6, 6) * B(6, 1) + A(6, 7) * B(7, 1) + A(6, 8) * B(8, 1)
    C(6, 2) = A(6, 1) * B(1, 2) + A(6, 2) * B(2, 2) + A(6, 3) * B(3, 2) + A(6, 4) * B(4, 2) + A(6, 5) * B(5, 2) + A(6, 6) * B(6, 2) + A(6, 7) * B(7, 2) + A(6, 8) * B(8, 2)
    C(6, 3) = A(6, 1) * B(1, 3) + A(6, 2) * B(2, 3) + A(6, 3) * B(3, 3) + A(6, 4) * B(4, 3) + A(6, 5) * B(5, 3) + A(6, 6) * B(6, 3) + A(6, 7) * B(7, 3) + A(6, 8) * B(8, 3)
    C(6, 4) = A(6, 1) * B(1, 4) + A(6, 2) * B(2, 4) + A(6, 3) * B(3, 4) + A(6, 4) * B(4, 4) + A(6, 5) * B(5, 4) + A(6, 6) * B(6, 4) + A(6, 7) * B(7, 4) + A(6, 8) * B(8, 4)
    C(6, 5) = A(6, 1) * B(1, 5) + A(6, 2) * B(2, 5) + A(6, 3) * B(3, 5) + A(6, 4) * B(4, 5) + A(6, 5) * B(5, 5) + A(6, 6) * B(6, 5) + A(6, 7) * B(7, 5) + A(6, 8) * B(8, 5)
    C(6, 6) = A(6, 1) * B(1, 6) + A(6, 2) * B(2, 6) + A(6, 3) * B(3, 6) + A(6, 4) * B(4, 6) + A(6, 5) * B(5, 6) + A(6, 6) * B(6, 6) + A(6, 7) * B(7, 6) + A(6, 8) * B(8, 6)
    C(6, 7) = A(6, 1) * B(1, 7) + A(6, 2) * B(2, 7) + A(6, 3) * B(3, 7) + A(6, 4) * B(4, 7) + A(6, 5) * B(5, 7) + A(6, 6) * B(6, 7) + A(6, 7) * B(7, 7) + A(6, 8) * B(8, 7)
    C(6, 8) = A(6, 1) * B(1, 8) + A(6, 2) * B(2, 8) + A(6, 3) * B(3, 8) + A(6, 4) * B(4, 8) + A(6, 5) * B(5, 8) + A(6, 6) * B(6, 8) + A(6, 7) * B(7, 8) + A(6, 8) * B(8, 8)
    
    'j=7
    C(7, 1) = A(7, 1) * B(1, 1) + A(7, 2) * B(2, 1) + A(7, 3) * B(3, 1) + A(7, 4) * B(4, 1) + A(7, 5) * B(5, 1) + A(7, 6) * B(6, 1) + A(7, 7) * B(7, 1) + A(7, 8) * B(8, 1)
    C(7, 2) = A(7, 1) * B(1, 2) + A(7, 2) * B(2, 2) + A(7, 3) * B(3, 2) + A(7, 4) * B(4, 2) + A(7, 5) * B(5, 2) + A(7, 6) * B(6, 2) + A(7, 7) * B(7, 2) + A(7, 8) * B(8, 2)
    C(7, 3) = A(7, 1) * B(1, 3) + A(7, 2) * B(2, 3) + A(7, 3) * B(3, 3) + A(7, 4) * B(4, 3) + A(7, 5) * B(5, 3) + A(7, 6) * B(6, 3) + A(7, 7) * B(7, 3) + A(7, 8) * B(8, 3)
    C(7, 4) = A(7, 1) * B(1, 4) + A(7, 2) * B(2, 4) + A(7, 3) * B(3, 4) + A(7, 4) * B(4, 4) + A(7, 5) * B(5, 4) + A(7, 6) * B(6, 4) + A(7, 7) * B(7, 4) + A(7, 8) * B(8, 4)
    C(7, 5) = A(7, 1) * B(1, 5) + A(7, 2) * B(2, 5) + A(7, 3) * B(3, 5) + A(7, 4) * B(4, 5) + A(7, 5) * B(5, 5) + A(7, 6) * B(6, 5) + A(7, 7) * B(7, 5) + A(7, 8) * B(8, 5)
    C(7, 6) = A(7, 1) * B(1, 6) + A(7, 2) * B(2, 6) + A(7, 3) * B(3, 6) + A(7, 4) * B(4, 6) + A(7, 5) * B(5, 6) + A(7, 6) * B(6, 6) + A(7, 7) * B(7, 6) + A(7, 8) * B(8, 6)
    C(7, 7) = A(7, 1) * B(1, 7) + A(7, 2) * B(2, 7) + A(7, 3) * B(3, 7) + A(7, 4) * B(4, 7) + A(7, 5) * B(5, 7) + A(7, 6) * B(6, 7) + A(7, 7) * B(7, 7) + A(7, 8) * B(8, 7)
    C(7, 8) = A(7, 1) * B(1, 8) + A(7, 2) * B(2, 8) + A(7, 3) * B(3, 8) + A(7, 4) * B(4, 8) + A(7, 5) * B(5, 8) + A(7, 6) * B(6, 8) + A(7, 7) * B(7, 8) + A(7, 8) * B(8, 8)
    
    'j=8
    C(8, 1) = A(8, 1) * B(1, 1) + A(8, 2) * B(2, 1) + A(8, 3) * B(3, 1) + A(8, 4) * B(4, 1) + A(8, 5) * B(5, 1) + A(8, 6) * B(6, 1) + A(8, 7) * B(7, 1) + A(8, 8) * B(8, 1)
    C(8, 2) = A(8, 1) * B(1, 2) + A(8, 2) * B(2, 2) + A(8, 3) * B(3, 2) + A(8, 4) * B(4, 2) + A(8, 5) * B(5, 2) + A(8, 6) * B(6, 2) + A(8, 7) * B(7, 2) + A(8, 8) * B(8, 2)
    C(8, 3) = A(8, 1) * B(1, 3) + A(8, 2) * B(2, 3) + A(8, 3) * B(3, 3) + A(8, 4) * B(4, 3) + A(8, 5) * B(5, 3) + A(8, 6) * B(6, 3) + A(8, 7) * B(7, 3) + A(8, 8) * B(8, 3)
    C(8, 4) = A(8, 1) * B(1, 4) + A(8, 2) * B(2, 4) + A(8, 3) * B(3, 4) + A(8, 4) * B(4, 4) + A(8, 5) * B(5, 4) + A(8, 6) * B(6, 4) + A(8, 7) * B(7, 4) + A(8, 8) * B(8, 4)
    C(8, 5) = A(8, 1) * B(1, 5) + A(8, 2) * B(2, 5) + A(8, 3) * B(3, 5) + A(8, 4) * B(4, 5) + A(8, 5) * B(5, 5) + A(8, 6) * B(6, 5) + A(8, 7) * B(7, 5) + A(8, 8) * B(8, 5)
    C(8, 6) = A(8, 1) * B(1, 6) + A(8, 2) * B(2, 6) + A(8, 3) * B(3, 6) + A(8, 4) * B(4, 6) + A(8, 5) * B(5, 6) + A(8, 6) * B(6, 6) + A(8, 7) * B(7, 6) + A(8, 8) * B(8, 6)
    C(8, 7) = A(8, 1) * B(1, 7) + A(8, 2) * B(2, 7) + A(8, 3) * B(3, 7) + A(8, 4) * B(4, 7) + A(8, 5) * B(5, 7) + A(8, 6) * B(6, 7) + A(8, 7) * B(7, 7) + A(8, 8) * B(8, 7)
    C(8, 8) = A(8, 1) * B(1, 8) + A(8, 2) * B(2, 8) + A(8, 3) * B(3, 8) + A(8, 4) * B(4, 8) + A(8, 5) * B(5, 8) + A(8, 6) * B(6, 8) + A(8, 7) * B(7, 8) + A(8, 8) * B(8, 8)

END SUB

SUB nxn.Matrix.Mult (A() AS SINGLE, B() AS SINGLE, C() AS SINGLE, dimen AS LONG)
	
	'loop over j
	FOR j = 1 TO dimen

		'loop over k
		FOR k = 1 TO dimen

			'zero C(j,k)
			C(j, k) = 0

			'loop over l
			FOR l = 1 TO dimen

				C(j, k) = C(j, k) + A(j, l) * B(l, k)

			NEXT l

		NEXT k

	NEXT j

END SUB

