'-------------------------------- LookUp01 -------------------------------'
'                                                                         '
' This program compares a floating point intensive calculation to a Look- '
' Up Table storing function values.                                       '
'                                                                         '
' NOTE: Due to array storage limitations, function in this version is     '
'       much smaller than in the Windows Basic and C version (that are    '
'       discussed in the text).                                           '
'                                                                         '
'-------------------------------------------------------------------------'

'declarations
'longs
DIM NumIts      AS LONG         'number of iterations
DIM counter     AS LONG         'iteration counter
DIM r           AS LONG         'distance used in calculation
DIM rIndex      AS LONG         'array index

'singles
DIM timestart   AS SINGLE       'ticks at start of loop
DIM timestop    AS SINGLE       'ticks at loop exit

'doubles
DIM R.value     AS DOUBLE       'variable to assign energy function
DIM Energy(2830) AS DOUBLE      'The Look-Up Table for E(r)
DIM ThisEnergy  AS DOUBLE       'explicitly computed energy
DIM r6          AS DOUBLE       'r^6 cached value in RISC algorithm
DIM r12         AS DOUBLE       'r^12 cached value in RISC algorithm

'Clear the screen
CLS

'-------------------------------- Initializations -------------------------

'do 10000000 iterations of calculation
NumIts = 10000000

'just a value to use for distance
r = 1023

PRINT "Initialize Look-Up Array ..."
PRINT
    
'start the timer
timestart = TIMER
    
'need 2,830 energies for this example
FOR rIndex = 50 TO 2830
    
	'convert to double precision
	R.value = CDBL(rIndex * .1)
	
	'compute rIndex^6
	r6 = R.value * R.value * R.value * R.value * R.value * R.value
	
	'compute rIndex^12
	r12 = r6 * r6
    
	'compute and store Energy array
	Energy(rIndex) = 1 / r12 - 1 / r6
	    
    NEXT rIndex

'stop the timer
timestop = TIMER
    
'display the results
PRINT "Look-Up Array initialized in"
PRINT timestop - timestart; " seconds."
PRINT
    
'---------------------------- End Initializations -------------------------

'---------------------------- Explicit Calculation ------------------------

'display message
PRINT "Starting explicit calculation of reduced LJ Energy ..."
PRINT

'start the timer
timestart = TIMER

'compute InnerR in picometers
R.value = r * .1

'begin the explicit calculation
FOR counter = 1 TO NumIts

	'compute r^6
	'compute rIndex^6
	r6 = R.value * R.value * R.value * R.value * R.value * R.value
	
	'compute rIndex^12
	r12 = r6 * r6
    
	'compute and store Energy array
	ThisEnergy = 1 / r12 - 1 / r6

NEXT counter

'stop the timer
timestop = TIMER

'display the result
PRINT "Explicitly computed Energy "; NumIts; " times in "
PRINT timestop - timestart; " seconds"
PRINT "Energy = "; ThisEnergy
PRINT

'------------------------ End Explicit Calculation ------------------------

'---------------------------- LookUp Calculation ------------------------

'display message
PRINT "Starting LookUp of reduced LJ Energy ..."
PRINT

'start the timer
timestart = TIMER


'begin the LookUp 'calculation'
FOR counter = 1 TO NumIts

	'look-up the energy
	'r is already an index
	ThisEnergy = Energy(r)

NEXT counter

'stop the timer
timestop = TIMER

'display the result
PRINT "Looked Up Energy "; NumIts; " times in "
PRINT timestop - timestart; " seconds"
PRINT "Energy = "; ThisEnergy
PRINT

'------------------------ End LookUp Calculation ------------------------

' Keep display active until user presses a key
' this is needed when running in the dos subsystem of windows, else
' windows closes window immediately

' display a message
PRINT : PRINT "Press any key to end..."

'check keylatch; if empty, simply loop back
LoopHere:
A$ = INKEY$
IF A$ = "" THEN GOTO LoopHere

'end when keylatch contains any key
END

