'-------------------------------- LookUp02 -------------------------------'
'                                                                         '
' This program compares a floating point intensive calculation to a Look- '
' Up Table storing function values.                                       '
'                                                                         '
' NOTE: Due to array storage limitations, function in this version is     '
'       much smaller than in the Windows Basic and C version (that are    '
'       discussed in the text).                                           '
'                                                                         '
'-------------------------------------------------------------------------'

'declarations
'longs
DIM NumIts      AS LONG         'number of iterations
DIM Counter     AS LONG         'iteration counter
DIM arg         AS LONG         'look-up index

'singles
DIM timestart   AS SINGLE       'ticks at start of loop
DIM timestop    AS SINGLE       'ticks at loop exit

'doubles
DIM angle       AS DOUBLE       'angle for sine function
DIM ebase       AS DOUBLE       'y in y^x
DIM expon       AS DOUBLE       'x in y^x
DIM LogArg      AS DOUBLE       'arguement for log function
DIM sinLU(2380) AS DOUBLE       'Look-up table for sine functin
DIM logLU(2380) AS DOUBLE       'Look-up table for log function
DIM exponLU(2380) AS DOUBLE     'Look-Up Table for y^x
DIM luAngle     AS DOUBLE       'arguement for sin
DIM luExpon     AS DOUBLE       'exponent for y^x
DIM luLogArg    AS DOUBLE       'arguement for log
DIM P           AS DOUBLE       'return value


'Clear the screen
CLS

'-------------------------------- Initializations -------------------------

'do 5000000 iterations of calculation
NumIts = 5000000

'constants needed
angle = 1.31266
ebase = 6.345
expon = 4.78
LogArg = 5.32

PRINT "Initialize Look-Up Arrays ..."
PRINT
    
'start the timer
timestart = TIMER
    
'need 2,380 terms for this example
FOR Counter = 1 TO 2380
    
	luAngle = Counter / 379
	sinLU(Counter) = SIN(luAngle)
	
	luExpon = Counter / 100
	exponLU(Counter) = ebase ^ luExpon
    
	luLogArg = Counter / 100
	logLU(Counter) = LOG(luLogArg)
	    
    NEXT Counter

'stop the timer
timestop = TIMER
    
'display the results
PRINT "Look-Up Array initialized in"
PRINT timestop - timestart; " seconds."
PRINT
    
'---------------------------- End Initializations -------------------------

'---------------------------- Explicit Calculations -----------------------

'display message
PRINT "Starting explicit calculation of sine ..."
PRINT

'start the timer
timestart = TIMER

'begin the explicit calculation
FOR Counter = 1 TO NumIts

	'compute the sin
	P = SIN(angle)

NEXT Counter

'stop the timer
timestop = TIMER

'display the result
PRINT "Explicitly computed sin "; NumIts; " times in "
PRINT timestop - timestart; " seconds"
PRINT "sin = "; P
PRINT

'display message
PRINT "Starting explicit calculation of y^x ..."
PRINT

'start the timer
timestart = TIMER

'begin the explicit calculation
FOR Counter = 1 TO NumIts

	'compute the y^x
	P = ebase ^ expon

NEXT Counter

'stop the timer
timestop = TIMER

'display the result
PRINT "Explicitly computed y^x "; NumIts; " times in "
PRINT timestop - timestart; " seconds"
PRINT "y^x = "; P
PRINT

'display message
PRINT "Starting explicit calculation of log ..."
PRINT

'start the timer
timestart = TIMER

'begin the explicit calculation
FOR Counter = 1 TO NumIts

	'compute the log
	P = LOG(LogArg)

NEXT Counter

'stop the timer
timestop = TIMER

'display the result
PRINT "Explicitly computed log "; NumIts; " times in "
PRINT timestop - timestart; " seconds"
PRINT "log = "; P
PRINT

'------------------------ End Explicit Calculation ------------------------

'---------------------------- LookUp Calculations -----------------------

'display message
PRINT "Starting LookUp of sin ..."
PRINT

'start the timer
timestart = TIMER

'begin the LookUp 'calculation'
FOR Counter = 1 TO NumIts

	'compute the index
	arg = CLNG(angle * 379)
	
	'look-up the sin(angle)
	P = sinLU(arg)

NEXT Counter

'stop the timer
timestop = TIMER

'display the result
PRINT "Looked Up sin "; NumIts; " times in "
PRINT timestop - timestart; " seconds"
PRINT "sin = "; P
PRINT

'display message
PRINT "Starting LookUp of y^x ..."
PRINT

'start the timer
timestart = TIMER


'begin the LookUp 'calculation'
FOR Counter = 1 TO NumIts

	'compute the index
	arg = CLNG(expon * 100)
	
	'look-up y^arg
	P = exponLU(arg)

NEXT Counter

'stop the timer
timestop = TIMER

'display the result
PRINT "Looked Up y^x "; NumIts; " times in "
PRINT timestop - timestart; " seconds"
PRINT "y^x = "; P
PRINT

'display message
PRINT "Starting LookUp of log ..."
PRINT

'start the timer
timestart = TIMER

'begin the LookUp 'calculation'
FOR Counter = 1 TO NumIts

	'compute the index
	arg = CLNG(LogArg * 100)
	
	'look-up the lookup log(arg)
	P = logLU(arg)

NEXT Counter

'stop the timer
timestop = TIMER

'display the result
PRINT "Looked Up log "; NumIts; " times in "
PRINT timestop - timestart; " seconds"
PRINT "log = "; P
PRINT


'------------------------ End LookUp Calculation ------------------------

' Keep display active until user presses a key
' this is needed when running in the dos subsystem of windows, else
' windows closes window immediately

' display a message
PRINT : PRINT "Press any key to end..."

'check keylatch; if empty, simply loop back
LoopHere:
A$ = INKEY$
IF A$ = "" THEN GOTO LoopHere

'end when keylatch contains any key
END

