VERSION 5.00
Begin VB.Form frmOS01 
   BorderStyle     =   1  'Fixed Single
   Caption         =   "OS01 - Win32 API vs. Actual Code Comparison Example"
   ClientHeight    =   5850
   ClientLeft      =   45
   ClientTop       =   330
   ClientWidth     =   7410
   Icon            =   "frmOS01.frx":0000
   LinkTopic       =   "Form1"
   MaxButton       =   0   'False
   MinButton       =   0   'False
   ScaleHeight     =   5850
   ScaleWidth      =   7410
   StartUpPosition =   2  'CenterScreen
   Begin VB.CommandButton cmdRectAPI 
      Caption         =   "Draw with Rectangle API Function"
      Height          =   735
      Left            =   5640
      TabIndex        =   5
      Top             =   3840
      Width           =   1575
   End
   Begin VB.PictureBox Picture1 
      Height          =   5175
      Left            =   120
      ScaleHeight     =   341
      ScaleMode       =   3  'Pixel
      ScaleWidth      =   357
      TabIndex        =   2
      Top             =   240
      Width           =   5415
   End
   Begin VB.CommandButton cmdPaintCode 
      Caption         =   "Paint Rectangles with code"
      Height          =   735
      Left            =   5640
      TabIndex        =   1
      Top             =   1920
      Width           =   1575
   End
   Begin VB.CommandButton cmdAPIPaint 
      Caption         =   "Paint Rectangles using Win32 API"
      Height          =   735
      Left            =   5640
      TabIndex        =   0
      Top             =   120
      Width           =   1575
   End
   Begin VB.Label Label3 
      Height          =   855
      Left            =   5640
      TabIndex        =   6
      Top             =   4680
      Width           =   1695
   End
   Begin VB.Label Label2 
      Height          =   975
      Left            =   5640
      TabIndex        =   4
      Top             =   2760
      Width           =   1575
   End
   Begin VB.Label Label1 
      Height          =   855
      Left            =   5640
      TabIndex        =   3
      Top             =   960
      Width           =   1575
   End
End
Attribute VB_Name = "frmOS01"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = False
'This program compares using Win API calls vs. hand coding.
'The demo uses two API calls, the ExtFloodFill function for
'painting an arbitrary shape, and the Rectangle for drawing
'rectangles.  The hand-coded 'version' uses a software
'rasterizer, but a very simplistic one.  The rasterizer
'does not write directly to video memory, but itself uses
'Windows functions (behind the scenes).
'
'It is important to note that this code was NOT compiled using
'the Windows 2000/XP 'patch' for Visual Studio.  Also, if ran on
'Windows 98 or Windows ME, the results may be very different than
'reported here and in the text.  The ExtFloodFill function behaves
'very differently on the home based OS's Win98 and ME.
'
'Further, performance 'should be' somewhat video card independent,
'since the hardware is not programmed directly.
'

Option Explicit

'Global declarations
'Constants
Const cMaxVal As Long = 340         'size of drawing 'window'
Const cNumRects As Long = 10000     'number of rectangles to draw

'Singles
Dim starttimer As Single        'system ticks at start of loop
Dim stoptimer As Single         'system ticks at loop exit

'This procedure uses the ExtFloodFill api call to fill (using the
'current brush, which we set to solid) rectangle outlined on the
'screen using the Line method.  Rectangle corners are randomly selected.

Private Sub cmdAPIPaint_Click()

    'Local Declarations
    'Longs
    Dim LeftX As Long           'rectangle corners
    Dim RightX As Long
    Dim TopY As Long
    Dim BottomY As Long
    Dim Counter As Long         'iteration counter
    Dim RectColor As Long       'color to draw
    Dim MidX As Long            'x and y of midpoint
    Dim MidY As Long
    Dim hDCon As Long           'handle to picture object
                                
    'Booleans
    Dim Dummy As Boolean        'not using the API return value
    
    'Strings
    Dim msg As String           'output
    
    'set fill style to solid
    Picture1.FillStyle = vbSolid
    
    'Display starting message...this procedure is SLOW on Win2000/XP
    Label1.Caption = "Computing ..."
    
    'let windows catch up
    DoEvents
    
    'need handle to picture object
    hDCon = Picture1.hdc
    
    'start the timer
    starttimer = Timer
    
    For Counter = 1 To cNumRects
    
        'select a random color
        RectColor = Int(16000000 * Rnd) + 1
        
        'fills are done in the random color
        Picture1.FillColor = RectColor
        
        'select random top left and bottom right
        LeftX = Int(cMaxVal * Rnd) + 1
        RightX = Int((cMaxVal + 1 - LeftX) * Rnd) + LeftX
        
        TopY = Int(cMaxVal * Rnd) + 1
        BottomY = Int((cMaxVal + 1 - TopY) * Rnd) + TopY
    
        'find midpoint
        MidX = LeftX + (RightX - LeftX) / 2
        MidY = TopY + (BottomY - TopY) / 2
    
        'draw lines in random color from corner to corner
        Picture1.Line (LeftX, TopY)-(RightX, TopY), RectColor
        Picture1.Line -(RightX, BottomY), RectColor
        Picture1.Line -(LeftX, BottomY), RectColor
        Picture1.Line -(LeftX, TopY), RectColor
    
        'Fill the rectangle
        Dummy = ExtFloodFill(hDCon, MidX, MidY, RectColor, 0)
    
    Next Counter

    'stop the timer
    stoptimer = Timer
    
    'display the results
    msg = "Using Win API, painted " & Str(Counter - 1) & " rectangles in " & Str(stoptimer - starttimer) & " seconds."
    Label1.Caption = msg


End Sub

'This procedure draws the rectangles using a simplistic software
'rasterizer, so it is filled line-by-line as it is drawn.  There is
'no explicit call to an API function in this case.

Private Sub cmdPaintCode_Click()

    'Local declarations
    'Longs
    Dim LeftX As Long           'rectangle corners
    Dim RightX As Long
    Dim TopY As Long
    Dim BottomY As Long
    Dim Counter As Long         'iteration counter
    Dim YCounter As Long        'rasterizer counter
    Dim RectColor As Long       'color to draw rectangle
    
    'strings
    Dim msg As String           'display message
    
    'Display starting message
    Label2.Caption = "Computing ..."
    
    'let Windows catch up
    DoEvents
    
    'start the timer
    starttimer = Timer
    
    For Counter = 1 To cNumRects
    
        'select random color
        RectColor = Int(16000000 * Rnd) + 1
        
        'select random top left and bottom right corners
        LeftX = Int(cMaxVal * Rnd) + 1
        RightX = Int((cMaxVal + 1 - LeftX) * Rnd) + LeftX
        
        TopY = Int(cMaxVal * Rnd) + 1
        BottomY = Int((cMaxVal + 1 - TopY) * Rnd) + TopY
    
        'draw the rectangle by rasterizing
        For YCounter = TopY To BottomY
        
            Picture1.Line (LeftX, YCounter)-(RightX, YCounter), RectColor
        
        Next YCounter
    
    Next Counter

    'stop the timer
    stoptimer = Timer
    
    'display the results
    msg = "Using actual code, painted " & Str(Counter - 1) & " rectangles in " & Str(stoptimer - starttimer) & " seconds."
    Label2.Caption = msg

End Sub

'This procedure draws the rectangles using the RECTANGLE API
'call, which itself fills using the current 'brush' (which we specify
'as solid).

Private Sub cmdRectAPI_Click()

    'Local declarations
    'Longs
    Dim LeftX As Long           'corners of rectangle
    Dim RightX As Long
    Dim TopY As Long
    Dim BottomY As Long
    Dim Counter As Long         'iteration counter
    Dim RectColor As Long       'color to paint
    Dim hDCon As Long           'handle to picture object
    
    'Booleans
    Dim Dummy As Boolean        'return value of the API call
    
    'Strings
    Dim msg As String           'output
    
    'set current brush to solid
    Picture1.FillStyle = vbSolid
    
    'display message
    Label3.Caption = "Computing ..."
    
    'let Windows catch up
    DoEvents
    
    'get the handle to the picture object
    hDCon = Picture1.hdc
    
    'start the timer
    starttimer = Timer
    
    For Counter = 1 To cNumRects
    
        'select a random color
        RectColor = Int(16000000 * Rnd) + 1
        
        'set brush and pen color
        Picture1.FillColor = RectColor
        Picture1.ForeColor = RectColor
        
        'select random corners
        LeftX = Int(cMaxVal * Rnd) + 1
        RightX = Int((cMaxVal + 1 - LeftX) * Rnd) + LeftX
        
        TopY = Int(cMaxVal * Rnd) + 1
        BottomY = Int((cMaxVal + 1 - TopY) * Rnd) + TopY
    
        'draw the rectangle using current brush and pen
        Dummy = Rectangle(hDCon, LeftX, TopY, RightX, BottomY)
    
    Next Counter

    'stop the timer
    stoptimer = Timer
    
    'display the results
    msg = "Using Rectangle Win API function, painted " & Str(Counter - 1) & " rectangles in " & Str(stoptimer - starttimer) & " seconds."
    Label3.Caption = msg

End Sub

