VERSION 5.00
Begin VB.Form frmFunction02 
   BorderStyle     =   1  'Fixed Single
   Caption         =   "Function02 - LJ Energy Function Calling vs. Inline Code Demo"
   ClientHeight    =   4125
   ClientLeft      =   45
   ClientTop       =   330
   ClientWidth     =   6495
   Icon            =   "frmFunction02.frx":0000
   LinkTopic       =   "Form1"
   MaxButton       =   0   'False
   MinButton       =   0   'False
   ScaleHeight     =   4125
   ScaleWidth      =   6495
   StartUpPosition =   2  'CenterScreen
   Begin VB.CommandButton cmdClearText 
      Caption         =   "Clear Text"
      Height          =   615
      Left            =   4920
      TabIndex        =   3
      Top             =   3120
      Width           =   1335
   End
   Begin VB.CommandButton cmdByRef 
      Caption         =   "Called Functions, By Reference"
      Height          =   735
      Left            =   4920
      TabIndex        =   2
      Top             =   1200
      Width           =   1335
   End
   Begin VB.CommandButton cmdInline 
      Caption         =   "In Line Functions"
      Height          =   735
      Left            =   4920
      TabIndex        =   1
      Top             =   2160
      Width           =   1335
   End
   Begin VB.CommandButton cmdByVal 
      Caption         =   "Called Functions, By Value"
      Height          =   735
      Left            =   4920
      TabIndex        =   0
      Top             =   240
      Width           =   1335
   End
End
Attribute VB_Name = "frmFunction02"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = False
' This program demonstrates the performance cost of passing parameters
' to functions.  In Visual Basic, passing By Value passes a copy of the actual
' data to the function routine; By Reference passes a 'pointer' to the actual
' data in memory.
'
' This example uses a basic reduced Lennard Jones algorithm as developed
' in Chapter 8 (RISC01).  See that example for sample algorithms.  Note
' that interaction well depth and sigma are not included; true, these
' would slow the whole works down, but
' (1) they would effect each actual computation the same and
' (2) they would slow the function calling even further since
'     it is assumed they would be passed as parameters to a generic
'     Lennard Jones function.
'
' Inline will benefit even more if more than one parameter is passed
' to the functions, as the parameter passing is part of what chews of
' cpu cycles.

Option Explicit

'Global Declarations

'Constants
Const cNumIts As Long = 10000000        'iteration counter
Const r As Double = 1.634               'just a value to use

'Singles
Dim timestart As Single     'time at loop entrance
Dim timestop As Single      'time at loop exit

'Doubles
Dim Energy As Double        'LJ Energy

' this function computes the reduced LJ energy with
' single double precision parameter passed as value.
Private Function LJ_Energy_Byval(ByVal rVal As Double) As Double

    'local declarations
    'doubles
    Dim denom1 As Double        'cache variable for r^6
    Dim denom2 As Double        'cache variable for r^12
    
    'compute r^6
    denom1 = rVal * rVal * rVal * rVal * rVal * rVal
    
    'compute r^12
    denom2 = denom1 * denom1
    
    'compute Energy
    LJ_Energy_Byval = 1 / denom2 - 1 / denom1

End Function

' this is the actual function that computes LJ energy with a
' single double precision parameter passed by reference.
Private Function LJ_Energy_byref(ByRef rRef As Double) As Double

    'local declarations
    'doubles
    Dim denom3 As Double    'cache variable for r^6
    Dim denom4 As Double    'cache variable for r^12
    
    'compute r^6
    denom3 = rRef * rRef * rRef * rRef * rRef * rRef
    
    'compute r^12
    denom4 = denom3 * denom3
    
    'compute Energy
    LJ_Energy_byref = 1 / denom4 - 1 / denom3

End Function

' calls the lennard jones energy computation
' passing the particle distance as a parameter by value.
Private Sub cmdByVal_Click()

    'local declarations
    'longs
    Dim Counter As Long             'iteration counter
    
    ' start timer
    timestart = Timer
    
    For Counter = 1 To cNumIts
    
        'call energy computation function, passing actual data
        Energy = LJ_Energy_Byval(r)
        
    Next Counter
    
    'stop the timer
    timestop = Timer
    
    'Display the results
    Print "Called LJ Function By Value "; cNumIts; " times."
    Print "Executed In " & timestop - timestart & " seconds"
    Print "Energy = "; Energy
    Print

End Sub

' initiates the computation of reduced LJ energy
' with no function calls...the function is executed
' "INLINE."
Private Sub cmdInline_Click()

    'local declarations
    'longs
    Dim Counter As Long         'iteration counter

    Dim denom5 As Double        'cache variable for r^6
    Dim denom6 As Double        'cache variable for r^12
    
    'start the timer
    timestart = Timer
    
    For Counter = 1 To cNumIts
    
        'compute r^6
        denom5 = r * r * r * r * r * r
        
        'compute r^12
        denom6 = denom5 * denom5
        
        'Compute reduced Energy
        Energy = 1 / denom6 - 1 / denom5
    
    Next Counter
    
    'stop the timer
    timestop = Timer
    
    'Display the results
    Print "Computed LJ Energy "; cNumIts; " times using Inline Code."
    Print "Executed In " & timestop - timestart & " seconds"
    Print "Energy = "; Energy
    Print


End Sub

' initiates the computation of reduced Lennard-Jones energy
' passing the particle distance by reference
' (ie, a pointer, rather than the actual data)
Private Sub cmdByRef_Click()

    'local declarations
    Dim Counter As Long                 'iteration counter
    
    'start the timer
    timestart = Timer
    
    For Counter = 1 To cNumIts
    
        'call energy computation function, by reference
        Energy = LJ_Energy_byref(r)
        
    Next Counter
    
    'stop the timer
    timestop = Timer
    
    'display the results
    Print "Called LJ Energy Function By Reference "; cNumIts; " times."
    Print "Executed In " & timestop - timestart & " seconds"
    Print "Energy = "; Energy
    Print


End Sub

' simply clears the screen so multiple checks can be made;
' due to os processes, not every run will be the same, so it
' helps to average three or four runs of each type.

Private Sub cmdClearText_Click()

    Cls

End Sub
