VERSION 5.00
Begin VB.Form frmFunction06 
   BorderStyle     =   1  'Fixed Single
   Caption         =   "Function06 - Demo Inline Code vs. Library Call"
   ClientHeight    =   2535
   ClientLeft      =   45
   ClientTop       =   330
   ClientWidth     =   5565
   Icon            =   "frmFunction06.frx":0000
   LinkTopic       =   "Form1"
   MaxButton       =   0   'False
   MinButton       =   0   'False
   ScaleHeight     =   2535
   ScaleWidth      =   5565
   StartUpPosition =   2  'CenterScreen
   Begin VB.CommandButton cmdClear 
      Caption         =   "Clear Text"
      Height          =   495
      Left            =   4200
      TabIndex        =   2
      Top             =   1800
      Width           =   1215
   End
   Begin VB.CommandButton Command2 
      Caption         =   "Inline Computation"
      Height          =   495
      Left            =   4200
      TabIndex        =   1
      Top             =   1080
      Width           =   1215
   End
   Begin VB.CommandButton Command1 
      Caption         =   "Compute by dll call"
      Height          =   495
      Left            =   4200
      TabIndex        =   0
      Top             =   360
      Width           =   1215
   End
End
Attribute VB_Name = "frmFunction06"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = False
'This program demonstrates the performance cost of using calls to
'compiled library functions.  Such a method of code development
'favors rapid development time and code re-use, but is slower
'at run time.
'
'The demo is done using the Lennard Jones Pair Energy calculation.
'The LJ_Energy function (method) is in the NumLib1.dll library
'(in fact, is the only function in that library).
'
'The NumLib1.dll was created using the NumLib1 project, so that the
'code was 'written' in Visual Basic.  For more information on creating
'dll's in Visual Basic, see the Visual Basic Component Tools Guide.
'
'The inline function executes about 4.5 times faster in this demo.


Option Explicit

'global declarations
'constants
Const cNumIts As Long = 10000000        'number of iterations

'external functions
Private Declare Function vbLJEnergy Lib "NumDemo.dll" _
    (ByVal r As Double, ByVal sigma6 As Double) As Double


'objects
'Dim Compute As CNumLib1                 'a Number Library Class for demo

'clears the print area for multiple runs
Private Sub cmdClear_Click()

    Cls

End Sub

'This procedure computes the pair energy using a call to the
'LJ_Energy function in the NumLib1 Dynamic Linked Library.

Private Sub Command1_Click()

    'local declarations
    'longs
    Dim Counter As Long         'iteration counter
    
    'singles
    Dim starttimer As Single    'ticks at start of loop
    Dim stoptimer As Single     'ticks at loop exit
    
    'doubles
    Dim Energy As Double        'function return value
    
    'start the timer
    starttimer = Timer
    
    'instantiates the compute object as NumLib1 class
    'Set Compute = New CNumLib1
    
    For Counter = 1 To cNumIts
    
        'call the LJ_Energy function
        'in the compute object, passing a value
        'Energy = Compute.LJ_Energy(1.234)
        Energy = vbLJEnergy(1.234, 0.5314)
    
    Next Counter
    
    'stop the timer
    stoptimer = Timer
    
    'display the results
    Print
    Print "Computed pair energy "; cNumIts; " times using"
    Print "dll library function call."
    Print "Energy = "; Energy; " in "; stoptimer - starttimer; " seconds."

End Sub

'This procedure computes the pair energy using inline code ... there
'is no function call, either to a module function or library.
'
'However, the object variable Compute is initialized so that the comparison
'is really that of the calling times....

Private Sub Command2_Click()

    'local declarations
    'longs
    Dim Counter As Long         'iteration counter
    
    'singles
    Dim starttimer As Single    'ticks at start of loop
    Dim stoptimer As Single     'ticks at loop exit
    
    'doubles
    Dim r6 As Double            'r^6
    Dim r12 As Double           'r^12
    Dim r As Double             'distance
    Dim sigma6 As Double        'sigma^6
    Dim sigma12 As Double       'sigma^12
    Dim Energy As Double        'computed energy
    
    'assign a value
    r = 1.234
    
    'assign sigma6
    sigma6 = 0.5314
    
    'start the timer
    starttimer = Timer
    
    'instantiates the object even though it is not used
    'since the real comparison of this demo is the actual
    'calling vs. inlining
    'Set Compute = New CNumLib1
    
    For Counter = 1 To cNumIts
    
        'compute r^6, see Chapter 8
        r6 = r * r * r * r * r * r
        
        'compute r^12, see Chapter 8
        r12 = r6 * r6
        
        'compute sigma^12
        sigma12 = sigma6 * sigma6
        
        'compute the LJ Energy
        Energy = sigma12 / r12 - sigma6 / r6
    
    Next Counter
    
    'stop the timer
    stoptimer = Timer
    
    'display the results
    Print
    Print "Computed pair energy "; cNumIts; " times using"
    Print "inline code."
    Print "Energy = "; Energy; " in "; stoptimer - starttimer; " seconds"


End Sub
