VERSION 5.00
Begin VB.Form frmFunction07 
   BorderStyle     =   1  'Fixed Single
   Caption         =   "Function07 - RND() vs. Hand Coded Number Generator"
   ClientHeight    =   2955
   ClientLeft      =   45
   ClientTop       =   330
   ClientWidth     =   5445
   Icon            =   "Function07.frx":0000
   LinkTopic       =   "Form1"
   MaxButton       =   0   'False
   MinButton       =   0   'False
   ScaleHeight     =   2955
   ScaleWidth      =   5445
   StartUpPosition =   2  'CenterScreen
   Begin VB.CommandButton cmdClear 
      Caption         =   "Clear Text"
      Height          =   615
      Left            =   3960
      TabIndex        =   2
      Top             =   1920
      Width           =   1335
   End
   Begin VB.CommandButton cmdGen 
      Caption         =   "Compute using inline Generator"
      Height          =   615
      Left            =   3960
      TabIndex        =   1
      Top             =   1080
      Width           =   1335
   End
   Begin VB.CommandButton cmdRnd 
      Caption         =   "Compute using RND"
      Height          =   615
      Left            =   3960
      TabIndex        =   0
      Top             =   240
      Width           =   1335
   End
End
Attribute VB_Name = "frmFunction07"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = False
'This program compares the use of an intrinsic function to
'inline, hand written, code.  The intrinsic function explored
'is the pseudo random number function RND().
'
'There are numerous recursive pseudo random number generation functions;
'the one chosen for this example is
'
'       X(k+1) = (a * Xk) mod m
'
'where a is a positive integer that defines the generator, the initial
'value X(0) is the "seed" and the period is given by m.
'
'If the value X(k+1) is divided by m, a value between 0 and 1 is obtained.
'
'On the test computer, the hand coded, inline, version ran
'approximately 3 times faster.

Option Explicit

'Global declarations
'Constants
Const cNumIts As Long = 50000000     'number of iterations

'Longs
Dim Xk As Long      'the last pseudo random number in the sequence
                    'could declare this local static rather than
                    'global; in any case, it needs to be retained
                    'so the numbers are generated in sequence
'Singles
Dim starttimer As Single        'system ticks as start of loop
Dim stoptimer As Single         'system ticks at loop exit

'clears the display area
Private Sub cmdClear_Click()

    Cls

End Sub

'This procedure uses the hand-coded pseudo random number
'generator to avoid repetitive calls to the RND() function.
'The seed value is 200 (specified in the Form_Load initialization
'procedure), a=25 and m=2^24.  Using m=2^24 gives a period equal
'to the intrinsic Visual Basic RND() function.

Private Sub cmdGen_Click()

    'Local declarations
    
    'Longs
    Dim ItCounter As Long       'iterative loop counter
    Dim Xk_1 As Long            'the NEXT value to get
    Dim a As Long               'defines the generator
    Dim m As Long               'period of the generator
    
    'Singles
    Dim r As Single             'actual pseudo random number 'returned'
                                'given by kk_1 / m.
                                
    'start the timer
    starttimer = Timer
    
    'include the initialization code to make a "fair comparison"
    'since these do not exist when RND() is called.
    a = 25
    m = 2 ^ 24
    
    'start the loop
    For ItCounter = 1 To cNumIts
    
        'compute the next pseudo random number in the sequence
        Xk_1 = (a * Xk) Mod m
        r = Xk_1 / m
    
        '"increment" the recursive index
        Xk = Xk_1
    
    Next ItCounter
    
    'stop the timer
    stoptimer = Timer
    
    'Display the results
    Print
    Print "Obtained "; cNumIts; " Random Numbers via the "
    Print "Inline Coded Generator in"
    Print stoptimer - starttimer; " seconds."

End Sub

'This procedure obtains the random numbers from the intrinsic
'RND() function.

Private Sub cmdRnd_Click()

    'Local declarations
    'Longs
    Dim ItCounter As Long       'iteration counter
    
    'Singles
    Dim r As Single             'actual random number returned
    
    'start the timer
    starttimer = Timer
    
    For ItCounter = 1 To cNumIts
    
        'get the next system pseudo random number
        'in the sequence
        r = Rnd
    
    Next ItCounter

    'stop the timer
    stoptimer = Timer
    
    'display the results
    Print
    Print "Obtained "; cNumIts; " Random Numbers via the "
    Print "Intrinsic RND() Function in"
    Print stoptimer - starttimer; " seconds."

End Sub

'The only initialization code needed is the 'seed' value
'for the inline generator....X(0).

Private Sub Form_Load()
    
    'seeding with X(0)=200, but any integer less than
    'm can be used.
    Xk = 200

End Sub
