VERSION 5.00
Begin VB.Form frmLoop05 
   BorderStyle     =   1  'Fixed Single
   Caption         =   "Loop05 - Time dependent heat flow demo"
   ClientHeight    =   2250
   ClientLeft      =   45
   ClientTop       =   330
   ClientWidth     =   5895
   Icon            =   "frmLoop05.frx":0000
   LinkTopic       =   "Form1"
   MaxButton       =   0   'False
   MinButton       =   0   'False
   ScaleHeight     =   2250
   ScaleWidth      =   5895
   StartUpPosition =   2  'CenterScreen
   Begin VB.CommandButton Command2 
      Caption         =   "Solve using unrolled Guass-Seidel"
      Height          =   615
      Left            =   4080
      TabIndex        =   1
      Top             =   1080
      Width           =   1695
   End
   Begin VB.CommandButton Command1 
      Caption         =   "Solve using rolled Gauss-Seidel"
      Height          =   615
      Left            =   4080
      TabIndex        =   0
      Top             =   240
      Width           =   1695
   End
End
Attribute VB_Name = "frmLoop05"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = False
'This program solves the time dependent heat
'equation by the Finite Volume method.
'Gauss-Seidel iteration is used to solve the
'coupled equations.  The time dependence modeled
'using the fully implicit (most computationally
'costly) method.

Option Explicit

'global declarations
'longs

'singles
Const tStep As Single = 0.001
Const tStop As Single = 1200
Const OptTol As Single = 0.00001
Dim A(5, 5) As Single
Dim b(5) As Single
Dim T0(5) As Single
Dim T(5) As Single

'This routine uses rolled loops for the Gauss-Seidel solver
Private Sub Command1_Click()

    'local declarations
    'longs
    Dim j As Long
    Dim k As Long
    Dim counter As Long
    Dim NumIts As Long
    
    'singles
    Dim delta_T As Single
    Dim max_delta_T As Single
    Dim Tdiff As Single
    Dim Sum1 As Single
    Dim Sum2 As Single
    Dim T_Old(5) As Single
    Dim TCounter As Single
    Dim starttimer As Single
    Dim stoptimer As Single
    Dim AP0 As Single
    
    'initializations
    T0(1) = 200
    T0(2) = 200
    T0(3) = 200
    T0(4) = 200
    T0(5) = 200

    T(1) = T0(1)
    T(2) = T0(2)
    T(3) = T0(3)
    T(4) = T0(4)
    T(5) = T0(5)

    A(1, 1) = 400025
    A(1, 2) = -25
    A(1, 3) = 0
    A(1, 4) = 0
    A(1, 5) = 0

    A(2, 1) = -25
    A(2, 2) = 400050
    A(2, 3) = -25
    A(2, 4) = 0
    A(2, 5) = 0

    A(3, 1) = 0
    A(3, 2) = -25
    A(3, 3) = 400050
    A(3, 4) = -25
    A(3, 5) = 0

    A(4, 1) = 0
    A(4, 2) = 0
    A(4, 3) = -25
    A(4, 4) = 400050
    A(4, 5) = -25

    A(5, 1) = 0
    A(5, 2) = 0
    A(5, 3) = 0
    A(5, 4) = -25
    A(5, 5) = 400075

    AP0 = 10000000# * (0.004 / tStep) * 0.01
    
    Cls
    
    'start the timer
    starttimer = Timer
    
    For TCounter = 2 To tStop Step tStep
    
    'set 'right side' of matrix equation
    For counter = 1 To 5
    
        b(counter) = AP0 * T0(counter)
        
    Next counter
    
    'solve the system for T(j)
    'using Gauss-Seidel procedure
    NumIts = 1
    delta_T = 1
    max_delta_T = 0
    
    Do While delta_T >= OptTol And NumIts <= 1000
    
        
        For counter = 1 To 5
    
            T_Old(counter) = T(counter)
    
        Next counter
    
        For j = 1 To 5

            Sum1 = 0
            For k = 1 To j - 1
        
                Sum1 = A(j, k) * T(k) + Sum1
            
            Next k
        
            Sum2 = 0
        
            For k = j + 1 To 5
        
                Sum2 = Sum2 + (A(j, k) * T_Old(k))

            Next k
        
            T(j) = Sum1 + Sum2 - b(j)
        
            T(j) = T(j) / -A(j, j)
            
            Tdiff = Abs(T(j) - T_Old(j))
            
            If Tdiff > max_delta_T Then
            
                max_delta_T = Tdiff
            
            End If
        
        Next j
    
        NumIts = NumIts + 1
        delta_T = max_delta_T
        max_delta_T = 0
    
    Loop
    
    
    
    For counter = 1 To 5
    
        T0(counter) = T(counter)
        
    Next counter

    Next TCounter
    
    'stop the timer
    stoptimer = Timer
    
    'display results
    Print "Via Rolled Loops, Node Temperatures:"
    Print "T(1)= "; T(1); "  T(2)= "; T(2)
    Print "T(3)= "; T(3); "  T(4)= "; T(4)
    Print "T(5)= "; T(5)
    Print "Solved in "; stoptimer - starttimer; " seconds"

End Sub

'this routine uses unrolled loops for the Gauss-Seidel
'solver
Private Sub Command2_Click()

    'local declarations
    'longs
    Dim j As Long
    Dim k As Long
    Dim counter As Long
    Dim NumIts As Long
    
    'singles
    Dim delta_T As Single
    Dim max_delta_T As Single
    Dim Tdiff As Single
    Dim Sum1 As Single
    Dim Sum2 As Single
    Dim T_Old(5) As Single
    Dim TCounter As Single
    Dim starttimer As Single
    Dim stoptimer As Single
    Dim AP0 As Single
    
    'initializations
    T0(1) = 200
    T0(2) = 200
    T0(3) = 200
    T0(4) = 200
    T0(5) = 200

    T(1) = T0(1)
    T(2) = T0(2)
    T(3) = T0(3)
    T(4) = T0(4)
    T(5) = T0(5)

    A(1, 1) = 400025
    A(1, 2) = -25
    A(1, 3) = 0
    A(1, 4) = 0
    A(1, 5) = 0

    A(2, 1) = -25
    A(2, 2) = 400050
    A(2, 3) = -25
    A(2, 4) = 0
    A(2, 5) = 0

    A(3, 1) = 0
    A(3, 2) = -25
    A(3, 3) = 400050
    A(3, 4) = -25
    A(3, 5) = 0

    A(4, 1) = 0
    A(4, 2) = 0
    A(4, 3) = -25
    A(4, 4) = 400050
    A(4, 5) = -25

    A(5, 1) = 0
    A(5, 2) = 0
    A(5, 3) = 0
    A(5, 4) = -25
    A(5, 5) = 400075

    
    AP0 = 10000000# * (0.004 / tStep) * 0.01
    
    Cls
    
    'start the timer
    starttimer = Timer

    For TCounter = 2 To tStop Step tStep
    
    'set 'right side' of matrix equation
    b(1) = AP0 * T0(1)
    b(2) = AP0 * T0(2)
    b(3) = AP0 * T0(3)
    b(4) = AP0 * T0(4)
    b(5) = AP0 * T0(5)
    
    'solve the system for T(j)
    'using Gauss-Seidel procedure
    NumIts = 1
    delta_T = 1
    max_delta_T = 0
    
    Do While delta_T >= OptTol And NumIts <= 1000
    
    
        T_Old(1) = T(1)
        T_Old(2) = T(2)
        T_Old(3) = T(3)
        T_Old(4) = T(4)
        T_Old(5) = T(5)
    
        'this begins the unrolled Gauss-Seidel procedure
        'j=1
        'k=2-5
        Sum2 = A(1, 2) * T_Old(2) + A(1, 3) * T_Old(3) + A(1, 4) * T_Old(4) + A(1, 5) * T_Old(5)
        
        T(1) = Sum2 - b(1)
        T(1) = T(1) / -A(1, 1)
        
        max_delta_T = Abs(T(1) - T_Old(1))
        
        'j=2
        'k=1
        Sum1 = A(2, 1) * T(1)
        
        'k=3-5
        Sum2 = A(2, 3) * T_Old(3) + A(2, 4) * T_Old(4) + A(2, 5) * T_Old(5)
        
        T(2) = Sum1 + Sum2 - b(2)
        T(2) = T(2) / -A(2, 2)
        
        Tdiff = Abs(T(2) - T_Old(2))
            
        If Tdiff > max_delta_T Then
            
            max_delta_T = Tdiff
            
        End If
        
        'j=3
        'k=1-2
        Sum1 = A(3, 1) * T(1) + A(3, 2) * T(2)
        
        'k=4-5
        Sum2 = A(3, 4) * T_Old(4) + A(3, 5) * T_Old(5)
        
        T(3) = Sum1 + Sum2 - b(3)
        T(3) = T(3) / -A(3, 3)
        
        Tdiff = Abs(T(3) - T_Old(3))
            
        If Tdiff > max_delta_T Then
            
            max_delta_T = Tdiff
            
        End If
        
        'j=4
        'k=1-3
        Sum1 = A(4, 1) * T(1) + A(4, 2) * T(2) + A(4, 3) * T(3)
        
        'k=5
        Sum2 = A(4, 5) * T_Old(5)
        
        T(4) = Sum1 + Sum2 - b(4)
        T(4) = T(4) / -A(4, 4)
        
        Tdiff = Abs(T(4) - T_Old(4))
            
        If Tdiff > max_delta_T Then
            
            max_delta_T = Tdiff
            
        End If
        
        'j=5
        'k=1-4
        Sum1 = A(5, 1) * T(1) + A(5, 2) * T(2) + A(5, 3) * T(3) + A(5, 4) * T(4)
        
        T(5) = Sum1 - b(5)
        T(5) = T(5) / -A(5, 5)
        
        Tdiff = Abs(T(5) - T_Old(5))
            
        If Tdiff > max_delta_T Then
            
            max_delta_T = Tdiff
            
        End If
    
        NumIts = NumIts + 1
        delta_T = max_delta_T
    
    Loop
    
    T0(1) = T(1)
    T0(2) = T(2)
    T0(3) = T(3)
    T0(4) = T(4)
    T0(5) = T(5)

    Next TCounter
    
    'stop the timer
    stoptimer = Timer
    
    'display results
    Print "Via Unrolled Loops, Node Temperatures:"
    Print "T(1)= "; T(1); "  T(2)= "; T(2)
    Print "T(3)= "; T(3); "  T(4)= "; T(4)
    Print "T(5)= "; T(5)
    Print "Solved in "; stoptimer - starttimer; " seconds"

End Sub

'initialization
Private Sub Form_Load()

T0(1) = 200
T0(2) = 200
T0(3) = 200
T0(4) = 200
T0(5) = 200

T(1) = T0(1)
T(2) = T0(2)
T(3) = T0(3)
T(4) = T0(4)
T(5) = T0(5)

A(1, 1) = 225
A(1, 2) = -25
A(1, 3) = 0
A(1, 4) = 0
A(1, 5) = 0

A(2, 1) = -25
A(2, 2) = 250
A(2, 3) = -25
A(2, 4) = 0
A(2, 5) = 0

A(3, 1) = 0
A(3, 2) = -25
A(3, 3) = 250
A(3, 4) = -25
A(3, 5) = 0

A(4, 1) = 0
A(4, 2) = 0
A(4, 3) = -25
A(4, 4) = 250
A(4, 5) = -25

A(5, 1) = 0
A(5, 2) = 0
A(5, 3) = 0
A(5, 4) = -25
A(5, 5) = 275


End Sub
