VERSION 5.00
Begin VB.Form frmRISC01 
   BorderStyle     =   1  'Fixed Single
   Caption         =   "RISC01 - CISC vs RISC LJ Energy Computation"
   ClientHeight    =   3060
   ClientLeft      =   45
   ClientTop       =   330
   ClientWidth     =   6345
   Icon            =   "frmRISC01.frx":0000
   LinkTopic       =   "Form1"
   MaxButton       =   0   'False
   MinButton       =   0   'False
   ScaleHeight     =   3060
   ScaleWidth      =   6345
   StartUpPosition =   2  'CenterScreen
   Begin VB.CommandButton cmdClear 
      Caption         =   "Clear Text"
      Height          =   615
      Left            =   4560
      TabIndex        =   2
      Top             =   1920
      Width           =   1455
   End
   Begin VB.CommandButton cmdCISCEn 
      Caption         =   "CISC"
      Height          =   615
      Left            =   4560
      TabIndex        =   1
      Top             =   240
      Width           =   1455
   End
   Begin VB.CommandButton cmdRISCEn 
      Caption         =   "RISC"
      Height          =   615
      Left            =   4560
      TabIndex        =   0
      Top             =   1080
      Width           =   1455
   End
End
Attribute VB_Name = "frmRISC01"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = False
' This program uses the Lennard Jones pair energy calculation
' to illustrate programming in the RISC Style.
'
' This program is performing a very simple floating point computation.  The
' more floating point work done within the interative loop, of course, the greater
' the benefit.  For example, similar gains will be observed with the computation
' of the actual distances to be used for the LJ calculation.
'
' The executable was compiled using VB 6.0 SP5 Pro with the following options:
'
'   (1) Native Code, Optimized for fast
'   (2) No Advanced Optimizations
'
' Execution times on the author's machine were:
'
'       CISC Style          9.7 seconds
'       (the Scientist Style as described in Chapter 1)
'
'       RISC Style          0.4 seconds
'
' which constitutes performance improvement by a factor of 25.
'

Option Explicit

'Global Declarations

'Constants
Const r As Double = 1.634       'just a value to use

'Longs
Dim counter As Long     'Iteration Counter

'Singles
Dim timestop As Single      'system ticks at start of loop
Dim timestart As Single     'stystem ticks at loop end

' This procedure computes the reduced LJ Energy in a RISC style.
' Though several RISC styles were tested,
' this "RISC" style was the fastest way to do the LJ energy calculation.
' Compilers don't like complicated fp functions like r^6 or r^12, plus
' we take advantage of a little caching the cpu.

Private Sub cmdRISCEn_Click()
    
    Dim denom1 As Double        'holder variable for r^6
    Dim denom2 As Double        'holder variable for r^12
    Dim E As Double             'computed energy
    
    timestart = Timer   'start the timer
    
    For counter = 1 To 10000000
    
        denom1 = r * r * r * r * r * r  'this allows compiler to create
                                        ' native cpu instruction friendly
                                        ' object code
        denom2 = denom1 * denom1
        
        E = 1 / denom2 - 1 / denom1
        
    Next counter
    
    timestop = Timer    ' stop the timer
    
    'display the results
    Print "Calculated E=(1/r^6) + (1/r^12)  10,000,000 times "
    Print "using optimized RISC style algorithm in  "
    Print Str(timestop - timestart) & "  seconds"
    Print "E = " & E
    Print

End Sub

' This procedure illustrates the "Scientific Programmer" style of
' programming mathematical functions.  Note the math looks like the
' Lennard Jones Equation (minus constants) would look on paper, and notice
' only one line of source code is used.  In contrast, the RISC style approach
' in Command1_Click() 'spreads it out' over several lines and codes the
' equation a little more 'baby-ish' looking.


Private Sub cmdCISCEn_Click()

    Dim E As Double
    
    timestart = Timer   'start the timer
    
    For counter = 1 To 10000000
        
        E = (1 / r) ^ 12 - (1 / r) ^ 6  ' this throws everything
                                        ' at the compiler in one shot;
                                        ' inefficient object code and complicated
                                        ' function emulators must be invoked.
        
    Next counter
    
    timestop = Timer    ' stop the timer
    
    Print "Calculated E=(1/r^6) + (1/r^12)  10,000,000 times "
    Print "using optimized Scientific CISC style algorithm in  "
    Print Str(timestop - timestart) & "  seconds"
    Print "E = " & E
    Print
    
    E = 0

End Sub

' Simply clears the screen so multiple runs can be checked.
' Due to os overhead, it is better to average several runs to get
' a clearer picture of execution times.

Private Sub cmdClear_Click()

    Cls

End Sub

