VERSION 5.00
Begin VB.Form frmLookUP01 
   BorderStyle     =   1  'Fixed Single
   Caption         =   "LookUP01 - Compare Look Up Table to Explicit Calculation"
   ClientHeight    =   2985
   ClientLeft      =   45
   ClientTop       =   330
   ClientWidth     =   5970
   Icon            =   "LookUp01.frx":0000
   LinkTopic       =   "Form1"
   MaxButton       =   0   'False
   MinButton       =   0   'False
   ScaleHeight     =   2985
   ScaleWidth      =   5970
   StartUpPosition =   2  'CenterScreen
   Begin VB.CommandButton cmdClear 
      Caption         =   "Clear Text"
      Enabled         =   0   'False
      Height          =   495
      Left            =   3960
      TabIndex        =   3
      Top             =   2280
      Width           =   1815
   End
   Begin VB.CommandButton cmdCalcEachEn 
      Caption         =   "Compute 10,000,000 Energies explicitly"
      Enabled         =   0   'False
      Height          =   495
      Left            =   3960
      TabIndex        =   2
      ToolTipText     =   "Actually compute the energy for each iteration"
      Top             =   840
      Width           =   1815
   End
   Begin VB.CommandButton cmdCalcEnsLU 
      Caption         =   """Compute"" 10,000,000 Energies by Look-Up"
      Enabled         =   0   'False
      Height          =   495
      Left            =   3960
      TabIndex        =   1
      ToolTipText     =   "Look Up precomputed energy for each iteration"
      Top             =   1560
      Width           =   1815
   End
   Begin VB.CommandButton cmdEnLookUp 
      Caption         =   "Compute Energy Look-Up"
      Height          =   495
      Left            =   3960
      TabIndex        =   0
      ToolTipText     =   "Compute the array to be used as Look Up table"
      Top             =   120
      Width           =   1815
   End
End
Attribute VB_Name = "frmLookUP01"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = False
'This program demonstrates the use of Look-Up Tables for the
'computation of a reduced Lennard Jones energy.  The Look-Up
'table in this demo is an array storing the values E(r).  Therefore,
'when an energy is needed, it can be 'looked up,' rather than
'explicitly computed.
'
'For this demo, r is in picometers*10; that is, r precision
'is 0.1 picometers.

Option Explicit

'globals declarations
'constants
Const cNumIts As Long = 10000000        'number of iteration
'just a value to use for r
Const r As Long = 1023    'distance, 1.023 angstoms = 102.3 pm

'longs
Dim Counter As Long     'iteration loop counter
Dim rIndex As Long      'r value as an array index

'singles
Dim starttimer As Single    'These allow primitive execution
Dim stoptimer As Single     'timing
    
'doubles
Dim Energy(28300) As Double  'The Look-Up Table for E(r)
Dim r1 As Double            'r^6 cached value in RISC algorithm
Dim r2 As Double            'r^12 cached value in RISC algorithm
    
'Perform floating point energy calculation each time
'through the loop
Private Sub cmdCalcEachEn_Click()

    Dim ThisEnergy As Double    'computed energy
    Dim InnerR As Double        'r in picometers for calculation

    'start the timer
    starttimer = Timer
    
    'compute InnerR in picometers
    InnerR = r * 0.1
    
    For Counter = 1 To cNumIts
    
        'compute r^6
        r1 = InnerR * InnerR * InnerR * InnerR * InnerR * InnerR 'calc r^6
        'compute r^12
        r2 = r1 * r1
        
        'compute Energy
        ThisEnergy = 1 / r2 - 1 / r1
        
    Next Counter
    
    'stop the timer
    stoptimer = Timer
    
    'display the results
    Print "Iterative energies explicitly computed each time in"
    Print stoptimer - starttimer; " seconds."
    Print "Energy = "; ThisEnergy; ""
    Print

End Sub

'Get energy via look-up table
Private Sub cmdCalcEnsLU_Click()

    Dim ThisEnergy As Double        'Looked up energy

    'start the timer
    starttimer = Timer
    
    For Counter = 1 To cNumIts
    
        'look-up the energy
        'r is already an index
        ThisEnergy = Energy(r)
    
    Next Counter
    
    'stop the timer
    stoptimer = Timer
    
    'display the result
    Print "Iterative energies calculated by Look-Up in"
    Print stoptimer - starttimer; " seconds."
    Print "Energy = "; ThisEnergy
    Print


End Sub

'Clear the screen
Private Sub cmdClear_Click()
    
    Cls

End Sub

'Initialize the Look-Up array Energy(r)
Private Sub cmdEnLookUp_Click()
    
    'local declarations
    'doubles
    Dim R_value As Double
    
    'start the timer
    starttimer = Timer
    
    'need 28,300 energies for this example
    For rIndex = 50 To 28300
    
        'convert to double precision
        R_value = CDbl(rIndex * 0.1)
        
        'compute rIndex^6
        r1 = R_value * R_value * R_value * R_value * R_value * R_value
        
        'compute rIndex^12
        r2 = r1 * r1
    
        'compute and store Energy array
        Energy(rIndex) = 1 / r2 - 1 / r1
            
    Next rIndex

    'stop the timer
    stoptimer = Timer
    
    'display the results
    Print "Look-Up Array initialized in"
    Print stoptimer - starttimer; " seconds."
    Print
    
    cmdCalcEachEn.Enabled = True
    cmdCalcEnsLU.Enabled = True
    cmdClear.Enabled = True

End Sub

