VERSION 5.00
Begin VB.Form frmLookUp04 
   BorderStyle     =   1  'Fixed Single
   Caption         =   "LookUp04 - Combined r and LJ Energy Compute vs. LUT"
   ClientHeight    =   3405
   ClientLeft      =   45
   ClientTop       =   330
   ClientWidth     =   5445
   Icon            =   "frmLookUp04.frx":0000
   LinkTopic       =   "Form1"
   MaxButton       =   0   'False
   MinButton       =   0   'False
   ScaleHeight     =   3405
   ScaleWidth      =   5445
   StartUpPosition =   2  'CenterScreen
   Begin VB.CommandButton Command4 
      Caption         =   "Clear Text"
      Height          =   615
      Left            =   3840
      TabIndex        =   3
      Top             =   2640
      Width           =   1455
   End
   Begin VB.CommandButton Command3 
      Caption         =   "Look-Up r and E"
      Height          =   615
      Left            =   3840
      TabIndex        =   2
      Top             =   1800
      Width           =   1455
   End
   Begin VB.CommandButton Command2 
      Caption         =   "Compute r and E using RISC"
      Height          =   615
      Left            =   3840
      TabIndex        =   1
      Top             =   960
      Width           =   1455
   End
   Begin VB.CommandButton Command1 
      Caption         =   "Compute Look-Up Tables for r and E"
      Height          =   615
      Left            =   3840
      TabIndex        =   0
      Top             =   120
      Width           =   1455
   End
End
Attribute VB_Name = "frmLookUp04"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = False
Option Explicit

'Global Declarations
'longs
Const cNumIts As Long = 100000000   'number of iterations
'just some values to use
Const x1 As Long = 103              'particle 1 x coordinate
Const x2 As Long = 373              'particle 2 x coordinate
Const y1 As Long = 271              'particle 1 y coordinate
Const y2 As Long = 144              'particle 2 y coordinate

'singles
Dim StartTimer As Single
Dim StopTimer As Single

'single arrays
Dim r(2830, 2830) As Single

'doubles
Const Depth As Double = 10.22             'well depth / Boltzmann's constant
Const Sigma As Double = 258               'E(sigma)=0, in pm
Const Sigma6 As Double = 294929510000000# 'sigma^6, in pm
Const Sigma12 As Double = 8.6983418E+28   'sigma^12, in pm

'double arrays
Dim E(2830) As Double


'routine to initialize the look up tables for r and E
'r is given in picometers
Private Sub Command1_Click()

    'local declarations
    'longs
    Dim Counter                 'counter to initialize E(r)
    Dim xCounter As Long        'counter for delta x to initialize r
    Dim yCounter As Long        'counter for delta y to initialize r
    
    'doubles
    Dim rvalue As Double        'actual r for E(r)
    Dim r6 As Double            'rvalue^6
    Dim r12 As Double           'rvalue^12
    Dim deltax As Double        'x2-x1, then (x2-x1)^2
    Dim deltay As Double        'y2-y1, then (y2-y1)^2
    
    'start the timer
    StartTimer = Timer
    
    'compute E(r) from 1 to 2380 pm (0.01 to 23.80 angstroms)
    For Counter = 1 To 2830

        'convert index to double
        rvalue = CDbl(Counter)
        
        'compute r^6
        r6 = rvalue * rvalue * rvalue * rvalue * rvalue * rvalue
        
        'compute r^12
        r12 = r6 * r6
        
        'compute E(r)
        E(Counter) = Sigma12 / r12 - Sigma6 / r6
        E(Counter) = E(Counter) * Depth
        
    Next Counter
    
    'compute r(deltax, deltay) for all 'possible' separations
    'within a 20x20 angstrom (2000x2000 picometer) box
    For xCounter = 0 To 2830
    
        'convert to double
        deltax = CDbl(xCounter)
        
        'to compute r, need deltax squared.  Note, however
        'that to avoid computing the square in the actual
        'computation, the index will be deltax directly
        deltax = deltax * deltax
    
        For yCounter = 0 To 2830
        
            'convert to double
            deltay = CDbl(yCounter)
            
            'to compute r, need deltay squared
            deltay = deltay * deltay
            
            'compute sum
            deltay = deltax + deltay
            
            'and take square root
            'only need integer part, since these will be
            'indices to use to look up energy in E(r)
            r(xCounter, yCounter) = CLng(Sqr(deltay))
        
        Next yCounter
        
    Next xCounter
    
    'stop the timer
    StopTimer = Timer
    
    'display the result
    Print "Time to initialize the r(dx, dy) and E(r)"
    Print "Look-Up Tables was "; StopTimer - StartTimer; " seconds."
    Print
    
End Sub

'This routine actually computes the energy for the two 'test particles'
'using the RISC style algorithm developed in Ch. 8
Private Sub Command2_Click()

    'local declarations
    'longs
    Dim ItCounter As Long           'iteration counter
    
    'doubles
    Dim delx As Double              'delta x
    Dim dely As Double              'delta y
    Dim r2 As Double                'r^2
    Dim r6 As Double                'r^6
    Dim r12 As Double               'r^12
    Dim Energy As Double            'Computed Energy
    
    'start the timer
    StartTimer = Timer
    
    For ItCounter = 1 To cNumIts
    
        'compute delta x and delta y from the coordinates
        delx = CDbl(x2 - x1)
        dely = CDbl(y2 - y1)
        
        'compute distance^2 using RISC algorithm from Ch.8
        delx = delx * delx
        dely = dely * dely
        
        r2 = delx + dely
        
        'compute Energy in RISC algorith from Ch.8
        'using distance^2
        
        r6 = r2 * r2 * r2
        r12 = r6 * r6
        
        Energy = Sigma12 / r12 - Sigma6 / r6
        
        Energy = Energy * Depth
    
    Next ItCounter
    
    'stop the timer
    StopTimer = Timer
    
    'display the result
    Print "Actually Computed E from two particle coordinates"
    Print cNumIts; " times in "; StopTimer - StartTimer; " seconds."
    Print "Energy = "; Energy
    Print

End Sub

'This routine 'computes' the pair energy for the two
'"test particles" using look up tables for both r and E.
Private Sub Command3_Click()

    'local declarations
    'longs
    Dim ItCounter As Long       'iteration counter
    Dim delx As Long            'delta x
    Dim dely As Long            'delta y
    Dim EIndex As Long          'looked up r value, used as
                                'index to E look up table
    
    'doubles
    Dim Energy As Double

    'start the timer
    StartTimer = Timer

    For ItCounter = 1 To cNumIts
    
        delx = Abs(x1 - x2)
        dely = Abs(y1 - y2)
    
        EIndex = r(delx, dely)
        
        Energy = E(EIndex)
        
    Next ItCounter
    
    'stop the timer
    StopTimer = Timer

    'display the result
    Print "Using LUTs, obtained E from two particle coordinates"
    Print cNumIts; " times in "; StopTimer - StartTimer; " seconds."
    Print "Energy = "; Energy
    Print

End Sub

'clear the display area for multiple runs
Private Sub Command4_Click()

    Cls

End Sub
