//Function07.c Implementation File
//
// This program compares the stdlib function rand() to an inline, handrolled 
// psuedo random number generator.
//
// The intrinsic function (in stdlib) is called from fnA()
// The inline generator is in fnB()

#define PLATFORM LINUX

//includes
#if PLATFORM == Windows
	#include <windows.h>
#elif PLATFORM == LINUX
	#include <sys/time.h>
#endif

#include <stdio.h>
#include <stdlib.h>

//defines
#define NumIts 50000000		//number of iterations

//functions
void fnIntrinsic();			//uses intrinsic rand()
void fnHandRolled();			//uses hand rolled generator

//global declarations
int xk;					//'last' random number generated

//computes random numbers from stdlib rand() function
void fnIntrinsic()
{

	//local declarations
	//integers
	int i;			//iteration counter
	int rnum;		//random number
	
	//variables for time execution time
#if PLATFORM == Windows

	//doubles
	double ElapsedTime;	//ticks during loop
	double t1;		//ticks at start of loop
	double t2;		//ticks at loop end	

#elif PLATFORM == Linux

	//floats
	float ElapsedTime;	//ticks during loop
	
	//doubles
	double t1;		//ticks at start of loop
	double t2;		//ticks at loop end
	
	//structures needed for basic timing functions
	static struct timeval _tstart, _tstop;
	static struct timezone tz;

#endif
	
	//display entry message
	printf("\nGetting Random Numbers with Intrinsic function...\n");
	
	//get start time
#if PLATFORM == Windows

	t1 = GetTickCount();
	
#elif

	gettimeofday(&_tstart, &tz);

#endif


	for(i=1; i<= NumIts; ++i){
	
		//get next random number in sequence
		rnum = rand();


	}
	
	//stop the timer
#if PLATFORM == Windows

	t2 = GetTickCount();

#elif PLATFORM == Linux

	gettimeofday(&_tstop, &tz);

	//get seconds from the structures
	t1 = (double)_tstart.tv_sec + (double)_tstart.tv_usec * 0.000001;
	t2 = (double)_tstop.tv_sec + (double)_tstop.tv_usec * 0.000001;

#endif

	//compute the elapsed time
#if PLATFORM == Windows
	
	ElapsedTime = (t2 - t1) * 0.001;

#elif PLATFORM == Linux

	ElapsedTime = (float)(t2 - t1);

#endif

	//display the result
	printf("%s%g%s", "\nElapsed Time = ", ElapsedTime, " seconds\n\n");
	
}//end fnIntrinsic()

void fnHandRolled()
{

	//local declarations
	//integers
	int i;			//iteration counter
	int rnum;		//a pseudo random number
	
	//variables for time execution time
#if PLATFORM == Windows

	//doubles
	double ElapsedTime;	//ticks during loop
	double t1;		//ticks at start of loop
	double t2;		//ticks at loop end	

#elif PLATFORM == Linux

	//floats
	float ElapsedTime;	//ticks during loop
	
	//doubles
	double t1;		//ticks at start of loop
	double t2;		//ticks at loop end
	
	//structures needed for basic timing functions
	static struct timeval _tstart, _tstop;
	static struct timezone tz;

#endif
	
	//seed the generator
	xk = 200;
	
	//display entry message
	printf("\nGetting Random Numbers with Hand Rolled Generator...\n");
	
	//get start time
#if PLATFORM == Windows

	t1 = GetTickCount();
	
#elif

	gettimeofday(&_tstart, &tz);

#endif

	for(i=1; i<= NumIts; ++i){
	
		//compute the next pseudo random number
		//in the sequence
		rnum = (25 * xk) % 16777216;
		
		//increment the recursive index
		xk = rnum;
		
	}
	
	//stop the timer
#if PLATFORM == Windows

	t2 = GetTickCount();

#elif PLATFORM == Linux

	gettimeofday(&_tstop, &tz);

	//get seconds from the structures
	t1 = (double)_tstart.tv_sec + (double)_tstart.tv_usec * 0.000001;
	t2 = (double)_tstop.tv_sec + (double)_tstop.tv_usec * 0.000001;

#endif

	//compute the elapsed time
#if PLATFORM == Windows
	
	ElapsedTime = (t2 - t1) * 0.001;

#elif PLATFORM == Linux

	ElapsedTime = (float)(t2 - t1);

#endif

	//display the result
	printf("%s%g%s", "\nElapsed Time = ", ElapsedTime, " seconds\n\n");
	
}//end fnHandRolled()

//entry function
int main()
{
	//local declarations
	//integers
	int bTERM;		//menu flag

	//character strings
	char inchar;		//input character
	

	//initialize bTERM to NOT TERMINATE
	bTERM = 0;
	
	//clear terminal
#if PLATFORM == Windows
	
	system("cls");

#elif PLATFORM == Linux

	system("clear");

#endif
	
	do {
		
		//present menu to user
		printf("Enter the Number of the routine to run, and press <Enter> \n");
		printf("1\tUse Intrinsic rand() function\n");
		printf("2\tUse Hand-Rolled Generator\n");
		printf("q\tEnd this program\n");
	
		//get user input
		scanf("%s", &inchar);

		//inchar = getchar();

		//call the user selected function
		switch	(inchar){

			case '1': fnIntrinsic();
				break;
	
			case '2': fnHandRolled();
				break;
		
			//select the terminate function	
			case 'q': printf("Ending This Program\n");
				
				//set flag to terminate
				bTERM = 1;
				break;

		}
		
	} while(!bTERM);

	return (0);

}
