// Data07Dlg.cpp : implementation file
//
// This program demonstrates the use of caching structure (or class)
// member data in simple variables for repeated access, vs accessing
// such data directly.  The class is declared globally.
// Some simple math is done for demo purposes.
//
// The function fnA() is the direct member access.
// The function fnC() uses simple cache variables.


#include <sys/time.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

//global declarations
const int NUMITS = 20000000;	//number of iterations

//define the class
class CAStructure
{

public:

	float x;
	float y;
	float z;

};

//instantiate the class
CAStructure	ThisStructure;


//functions
void fnA();
void fnB();						

//do some simple math with structure.member access
void fnA()
{	
	//local declarations
	int counter;		//iteration counter
	
	//float
	float a;		//results for some simple math
	float b;
	float c;
	
	//variables for time execution time
	//floats
	float ElapsedTime;	//ticks during loop
	
	//doubles
	double t1;		//ticks at start of loop
	double t2;		//ticks at loop end
	
	//structures needed for basic timing functions
	static struct timeval _tstart, _tstop;
	static struct timezone tz;
		
	//display entry message
	printf("\nComputing with Structure.Member Access...\n");

	//initialize structure values
	ThisStructure.x = (float)123.4;
	ThisStructure.y = (float)16.453;
	ThisStructure.z = (float)-0.00345;
	
	//get start time
	gettimeofday(&_tstart, &tz);

	for(counter=1; counter <=NUMITS; ++counter){
        
		//do some floating point math to force memory references
        	a = ThisStructure.x + ThisStructure.y + ThisStructure.z;
        	b = ThisStructure.x * ThisStructure.y * ThisStructure.z;
        	c = ThisStructure.x * ThisStructure.x + ThisStructure.y * 
			ThisStructure.y + ThisStructure.z * ThisStructure.z;
    
    }
    
	//stop the timer
	gettimeofday(&_tstop, &tz);

	//get seconds from the structures
	t1 = (double)_tstart.tv_sec + (double)_tstart.tv_usec * 0.000001;
	t2 = (double)_tstop.tv_sec + (double)_tstop.tv_usec * 0.000001;
	
	//compute the elapsed time
	ElapsedTime = (float)(t2 - t1);

	//display the result
	printf("%s%g%s", "\nElapsed Time = ", ElapsedTime, " seconds\n\n");
	
}//end fnA()

//do some simple math with structure data cached in simple variables
void fnB()
{	

	//local declarations
	int counter;		//iteration counter

	//float
	float a;		//results for some simple math
	float b;
	float c;
	float xcache;		//the cache variables
	float ycache;
	float zcache;

	//variables for time execution time
	//floats
	float ElapsedTime;	//ticks during loop
	
	//doubles
	double t1;		//ticks at start of loop
	double t2;		//ticks at loop end
	
	//structures needed for basic timing functions
	static struct timeval _tstart, _tstop;
	static struct timezone tz;
		
	//display entry message
	printf("\nComputing with structure data cached in simple variables...\n");

	//initialize structure values
	ThisStructure.x = (float)123.4;
	ThisStructure.y = (float)16.453;
	ThisStructure.z = (float)-0.00345;
	
	//get start time
	gettimeofday(&_tstart, &tz);

	//cache the member data in simple variables
	xcache = ThisStructure.x;
	ycache = ThisStructure.y;
	zcache = ThisStructure.z;

	for(counter=1; counter <=NUMITS; ++counter){
        
		//do some floating point math to force memory references
        	a = xcache + ycache + zcache;
        	b = xcache * ycache * zcache;
        	c = xcache * xcache + ycache * ycache + zcache * zcache;
    
    	}
    
	//stop the timer
	gettimeofday(&_tstop, &tz);

	//get seconds from the structures
	t1 = (double)_tstart.tv_sec + (double)_tstart.tv_usec * 0.000001;
	t2 = (double)_tstop.tv_sec + (double)_tstop.tv_usec * 0.000001;
	
	//compute the elapsed time
	ElapsedTime = (float)(t2 - t1);

	//display the result
	printf("%s%g%s", "\nElapsed Time = ", ElapsedTime, " seconds\n\n");
	
}//end fnB()


//entry function
int main()
{
	//local declarations
	//integers
	int bTERM;		//menu flag

	//character strings
	char inchar;		//input character
	

	//initialize bTERM to NOT TERMINATE
	bTERM = 0;
	
	//clear terminal
	system("clear");
	
	do {
		
		//present menu to user
		printf("Enter the Number of the routine to run, and press <Enter> \n");
		printf("1\tFloat computations using structure access\n");
		printf("2\tFloat computations using structure data cached in simple variables\n");
		printf("q\tEnd this program\n");
	
		//get user input
		scanf("%s", &inchar);

		//call the user selected function
		switch	(inchar){

			case '1': fnA();
				break;
				
			case '2': fnB();
				break;
	
			//select the terminate function	
			case 'q': printf("Ending This Program\n");
				
				//set flag to terminate
				bTERM = 1;
				break;

		}
		
	} while(!bTERM);

	return 0;

}
