
// Loop02.c : implementation file
//
// This program demonstrates loop unrolling for a simple vector dot
// product.
//
// The rolled routine is fnA().
// The unrolled routine is fnB().


#include <sys/time.h>
#include <stdio.h>
#include <math.h>
#include <stdlib.h>

//defines
#define NumIts 100000000		//number of iterations

//functions
void fnA();			
void fnB();			

//dot product via rolled loop
void fnA()
{

	//general local declarations
	//integers
	int i;					//iteration counter
	int dotcounter;			//dot product counter

	//singles
	float a[3];				//vector a
	float b[3];				//vector b
	float c;				//dot product result
	
	//variables for time execution time
	//floats
	float ElapsedTime;	//ticks during loop
	
	//doubles
	double t1;		//ticks at start of loop
	double t2;		//ticks at loop end
	
	//structures needed for basic timing functions
	static struct timeval _tstart, _tstop;
	static struct timezone tz;

	//initialize the two vectors with some made up values
	a[0] = (float)23.45;
	a[1] = (float)14.65;
	a[2] = (float)33.87;

	b[0] = (float)36.99;
	b[1] = (float)21.32;
	b[2] = (float)17.38;
	
	//display entry message
	printf("\nComputing Dot Product with rolled loop...\n");
	
	//get start time
	gettimeofday(&_tstart, &tz);
	
	for(i=1; i<= NumIts; ++i){
	
		c = (float)0;
		
		//dot product in rolled loop
		for(dotcounter=0; dotcounter < 3; ++dotcounter){

			c += a[dotcounter] * b[dotcounter];
        
		}

	}
	
	//stop the timer
	gettimeofday(&_tstop, &tz);

	//get seconds from the structures
	t1 = (double)_tstart.tv_sec + (double)_tstart.tv_usec * 0.000001;
	t2 = (double)_tstop.tv_sec + (double)_tstop.tv_usec * 0.000001;
	
	//compute the elapsed time
	ElapsedTime = (float)(t2 - t1);

	//display the result
	printf("%s%g%s", "\nElapsed Time = ", ElapsedTime, " seconds\n");
	printf("%s%g%s", "Dot Product = ", c, "\n");
	
}//end fnA()

void fnB()
{

	//general local declarations
	//integers
	int i;					//iteration counter

	//singles
	float a[3];				//vector a
	float b[3];				//vector b
	float c;				//dot product result
	
	//variables for time execution time
	//floats
	float ElapsedTime;	//ticks during loop
	
	//doubles
	double t1;		//ticks at start of loop
	double t2;		//ticks at loop end
	
	//structures needed for basic timing functions
	static struct timeval _tstart, _tstop;
	static struct timezone tz;

	//initialize the two vectors with some made up values
	a[0] = (float)23.45;
	a[1] = (float)14.65;
	a[2] = (float)33.87;

	b[0] = (float)36.99;
	b[1] = (float)21.32;
	b[2] = (float)17.38;
	
	//display entry message
	printf("\nComputing Dot Product with rolled loop...\n");
	
	//get start time
	gettimeofday(&_tstart, &tz);
	
	for(i=1; i<= NumIts; ++i){
	
		//note also eliminated a c=0 line!!
		
		//dot product in unrolled loop
		c = a[0] * b[0] + a[1] * b[1] + a[2] * b[2];

	}
	
	//stop the timer
	gettimeofday(&_tstop, &tz);

	//get seconds from the structures
	t1 = (double)_tstart.tv_sec + (double)_tstart.tv_usec * 0.000001;
	t2 = (double)_tstop.tv_sec + (double)_tstop.tv_usec * 0.000001;
	
	//compute the elapsed time
	ElapsedTime = (float)(t2 - t1);

	//display the result
	printf("%s%g%s", "\nElapsed Time = ", ElapsedTime, " seconds\n");
	printf("%s%g%s", "Dot Product = ", c, "\n");
	
}//end fnB()

//entry function
int main()
{
	//local declarations
	//integers
	int bTERM;		//menu flag

	//character strings
	char inchar;		//input character
	

	//initialize bTERM to NOT TERMINATE
	bTERM = 0;
	
	//clear terminal
	system("clear");
	
	do {
		
		//present menu to user
		printf("Enter the Number of the routine to run, and press <Enter> \n");
		printf("1\t3d Vector Dot Product with Rolled Loop\n");
		printf("2\t3d Vector Dot Product with Unrolled Loop\n");
		printf("q\tEnd this program\n");
	
		//get user input
		scanf("%s", &inchar);

		//call the user selected function
		switch	(inchar){

			case '1': fnA();
				break;
	
			case '2': fnB();
				break;
		
			//select the terminate function	
			case 'q': printf("Ending This Program\n");
				
				//set flag to terminate
				bTERM = 1;
				break;

		}
		
	} while(!bTERM);

	return 0;

}
