// LookUp02.c : implementation file
//
// This program compares the performance of explicit computations of
// the Boltzmann Factor to the use of a Look-Up Table.
//
// The Look-up Table assignment routine is in fnA().
// The Explicit calculation routine is in fnB().
// The Look-Up Table calculation routine is in fnC().


#include <sys/time.h>
#include <stdio.h>
#include <math.h>
#include <stdlib.h>

//defines
#define NumIts 10000000		//number of iterations

//global declarations
//integers
int bINIT;	 //flag = true if LUT has been initialized

//doubles
double k = 1.381E-23;	// Boltzmann constant
double Temp = 100;	// Temperature, just using 100
double Energy = 4e-21;  // Energy, just using 4e-21
double ExponLUT[10000];	// LUT for Boltzmann factor

//functions
void fnA();			
void fnB();			
void fnC();


//initialize the look up table
void fnA()
{

	//local declarations
	//integers
	int counter;			// iteration counter
	
	//doubles
	double arg1;			// argument for Boltzmann assignment
	
	//variables for time execution time
	//floats
	float ElapsedTime;	//ticks during loop
	
	//doubles
	double t1;		//ticks at start of loop
	double t2;		//ticks at loop end
	
	//structures needed for basic timing functions
	static struct timeval _tstart, _tstop;
	static struct timezone tz;
	
	//display entry message
	printf("\nInitializing Look-Up Table ...\n");
	
	//get start time
	gettimeofday(&_tstart, &tz);
	
	for(counter=1; counter <= 10000; ++counter){
    
		arg1 = (double)counter/(double)100;

		ExponLUT[counter] = exp(-arg1);  

        }
	
	//stop the timer
	gettimeofday(&_tstop, &tz);

	//get seconds from the structures
	t1 = (double)_tstart.tv_sec + (double)_tstart.tv_usec * 0.000001;
	t2 = (double)_tstop.tv_sec + (double)_tstop.tv_usec * 0.000001;
	
	//compute the elapsed time
	ElapsedTime = (float)(t2 - t1);

	//display the result
	printf("%s%g%s", "\nLook-Up Table Initialized in ", ElapsedTime, " seconds\n");
	
	//set flag to 'true'
	bINIT = 1;
	
}//end fnA()

// interative Boltzmann Factor by actual computation
void fnB()
{

	//general local declarations
	//integers
	int counter;			//iteration counter

	//doubles
	double Boltzmann;		//computed Boltzmann Factor
	
	//variables for time execution time
	//floats
	float ElapsedTime;	//ticks during loop
	
	//doubles
	double t1;		//ticks at start of loop
	double t2;		//ticks at loop end
	
	//structures needed for basic timing functions
	static struct timeval _tstart, _tstop;
	static struct timezone tz;
	
	//display entry message
	printf("\nStarting Iterative Explicit Boltzmann Factor calculation...\n");
	
	//get start time
	gettimeofday(&_tstart, &tz);
	
	for(counter=1; counter <= NumIts; ++counter){
    
		Boltzmann = exp(-Energy / (k * Temp));

        }
	
	//stop the timer
	gettimeofday(&_tstop, &tz);

	//get seconds from the structures
	t1 = (double)_tstart.tv_sec + (double)_tstart.tv_usec * 0.000001;
	t2 = (double)_tstop.tv_sec + (double)_tstop.tv_usec * 0.000001;
	
	//compute the elapsed time
	ElapsedTime = (float)(t2 - t1);

	//display the result
	printf("%s%g%s", "\nComputed Boltzmann Factor in ", ElapsedTime, " seconds\n");
	
}//end fnB()

// interative Boltzmann Factor by Look-Up Table
void fnC()
{

	//general local declarations
	//integers
	int counter;			//iteration counter
	int ExIndex;			//entry value for Boltzmann LUT


	//doubles
    	double Boltzmann;              //computed Boltzmann Factor
	
	//variables for time execution time
	//floats
	float ElapsedTime;	//ticks during loop
	
	//doubles
	double t1;		//ticks at start of loop
	double t2;		//ticks at loop end
	
	//structures needed for basic timing functions
	static struct timeval _tstart, _tstop;
	static struct timezone tz;

	
	//display entry message
	printf("\nStarting Iterative Boltzmann Factor via LUT ...\n");
	
	//get start time
	gettimeofday(&_tstart, &tz);
	
	for(counter=1; counter <= NumIts; ++counter){
    
		//compute 'index'
		ExIndex = (int)((Energy/(k * Temp)) * 100);

		//look up value
		Boltzmann = ExponLUT[ExIndex];
        }
	
	//stop the timer
	gettimeofday(&_tstop, &tz);

	//get seconds from the structures
	t1 = (double)_tstart.tv_sec + (double)_tstart.tv_usec * 0.000001;
	t2 = (double)_tstop.tv_sec + (double)_tstop.tv_usec * 0.000001;
	
	//compute the elapsed time
	ElapsedTime = (float)(t2 - t1);

	//display the result
	printf("%s%g%s", "\nLooked Up Boltzmann Factor in ", ElapsedTime, " seconds\n");
	
}//end fnC()

//entry function
int main()
{
	//local declarations
	//integers
	int bTERM;		//menu flag

	//character strings
	char inchar;		//input character
	

	//initialize bTERM to NOT TERMINATE
	bTERM = 0;
	
	//initialize bINIT to NOT INITIALIZED
	bINIT = 0;
	
	//clear terminal
	system("clear");
	
	do {
		
		//present menu to user
		printf("Enter the Number of the routine to run, and press <Enter> \n");
		printf("1\tInitialize the Look-Up Table\n");
		
		if(bINIT){
			printf("2\tIterative Explicit Boltzmann Factor\n");
			printf("3\tIterative Boltzmann Factor via Look-Up Table\n");
			printf("q\tEnd this program\n");
		}
	
		//get user input
		scanf("%s", &inchar);

		//call the user selected function
		switch	(inchar){

			case '1': fnA();
				break;
	
			case '2': fnB();
				break;
				
			case '3': fnC();
				break;
		
			//select the terminate function	
			case 'q': printf("Ending This Program\n");
				
				//set flag to terminate
				bTERM = 1;
				break;

		}
		
	} while(!bTERM);

	return 0;

} // End Main
