// Multi01.c : implementation file
//

// This program compares the performance of calling a function with
// using inline code.  The functions are empty and do no work.

// fnA() is a worker function (called by fnB or fnC)
// The single thread routine is in fnC().
// The multiple threaded routine is in fnB().


#include <sys/time.h>
#include <stdio.h>
#include <math.h>
#include <stdlib.h>

//defines
#define DEC		( void *(*)(void*) )
#define loopend		700000000

//functions
void fnA();			
void fnB();
void fnC();			

//Worker Thread
void fnA()
{

	//general local declarations
	//integers
	int counter;	  // iteration counter
	
	//display entry message
	printf("\nEntering Worker Thread ...\n");
	
	for(counter=1; counter <= loopend; ++counter);

	printf("\nDone with Worker Thread\n");
	
}//end fnA()

// Main Thread of two Thread Version
void fnB()
{

	//general local declarations
	//integers
	int counter;			//iteration counter
	int th1;			//Thread ID
	
	//variables for time execution time
	//floats
	float ElapsedTime;	//ticks during loop
	
	//doubles
	double t1;		//ticks at start of loop
	double t2;		//ticks at loop end
	
	//structures needed for basic timing functions
	static struct timeval _tstart, _tstop;
	static struct timezone tz;
	
	//get start time
	gettimeofday(&_tstart, &tz);
	
	//create the 'worker' thread
	if(pthread_create(&th1, NULL, DEC fnA, NULL)){
		printf("Thread creation failed");
	} 
	
	printf("Starting User Interface Work in Main Thread\n");
	
	for(counter=1; counter <= 100; ++counter){
    
		printf("0");
		system("sleep 0.025");
        
    	}
	
	pthread_join();
	
	//stop the timer
	gettimeofday(&_tstop, &tz);

	//get seconds from the structures
	t1 = (double)_tstart.tv_sec + (double)_tstart.tv_usec * 0.000001;
	t2 = (double)_tstop.tv_sec + (double)_tstop.tv_usec * 0.000001;
	
	//compute the elapsed time
	ElapsedTime = (float)(t2 - t1);

	//display the result
	printf("%s%g%s", "\nElapsed Time = ", ElapsedTime, " seconds\n");
	
}//end fnB()

// Function Single Thread Version
void fnC()
{

	//general local declarations
	//integers
	int counter;			//iteration counter
	
	//variables for time execution time
	//floats
	float ElapsedTime;	//ticks during loop
	
	//doubles
	double t1;		//ticks at start of loop
	double t2;		//ticks at loop end
	
	//structures needed for basic timing functions
	static struct timeval _tstart, _tstop;
	static struct timezone tz;
	
	//display entry message
	printf("\nSingle Thread Function...\n");
	
	//get start time
	gettimeofday(&_tstart, &tz);
	
	printf("Starting Worker Function\n");
	
	for(counter=1; counter <= loopend; ++counter);
	
	printf("Done with Worker Function\n"); 
	
	printf("Starting User Interface Function\n");
	
	for(counter=1; counter <= 100; ++counter){
    
		printf("0");
		system("sleep 0.025");
        
    	}
	
	
	//stop the timer
	gettimeofday(&_tstop, &tz);

	//get seconds from the structures
	t1 = (double)_tstart.tv_sec + (double)_tstart.tv_usec * 0.000001;
	t2 = (double)_tstop.tv_sec + (double)_tstop.tv_usec * 0.000001;
	
	//compute the elapsed time
	ElapsedTime = (float)(t2 - t1);

	//display the result
	printf("%s%g%s", "\nElapsed Time = ", ElapsedTime, " seconds\n");
	
}//end fnC()

//entry function
int main()
{
	//local declarations
	//integers
	int bTERM;		//menu flag

	//character strings
	char inchar;		//input character
	

	//initialize bTERM to NOT TERMINATE
	bTERM = 0;
	
	//clear terminal
	system("clear");
	
	do {
		
		//present menu to user
		printf("Enter the Number of the routine to run, and press <Enter> \n");
		printf("1\tSingle Threaded Application\n");
		printf("2\tMultiThreaded Application\n");
		printf("q\tEnd this program\n");
	
		//get user input
		scanf("%s", &inchar);

		//call the user selected function
		switch	(inchar){

			case '1': fnC();
				break;
	
			case '2': fnB();
				break;
		
			//select the terminate function	
			case 'q': printf("Ending This Program\n");
				
				//set flag to terminate
				bTERM = 1;
				break;

		}
		
	} while(!bTERM);

	return 0;

} // End Main


