// OS04.cpp : Defines the entry point for the console application.
//
// This program demonstrates a Console Application in Windows.  No
// graphical user interface is used.  The source code is fairly 
// portable to Linux, except for the headers used, the main() function 
// declaration and the Windows API call to GetTickCount().
//
// To perform some computational work, the program selects a random
// distance between 1 and 7 angstroms, and computes the Lennard
// Jones pair energy.  Helium LJ parameters are used.

#include "stdafx.h"
#include "windows.h"
#include "windowsx.h"
#include "stdio.h"

//Defines
#define NumIts 100000000		//number of iterations
#define sigma  (float)258		//lennard jones sigma parameter
#define depth  (float)10.22		//lennard jones depth parameter

int main(int argc, char* argv[])
{
	//local declarations
	//integers
	int i;		//iteration counter
	int rnum;	//next random number in sequence
	int xk;		//last random number in sequence

	//floats
	float r;			//distance between particles
	float r_6;			//distance^6
	float r_12;			//distance^12
	float sigma_6;		//sigma^6
	float sigma_12;		//sigma^12
	float energy;		//computed energy

	//doubles
	double starttime;	//ticks at start of loop
	double stoptime;	//ticks at end of loop
	double elapsedtime;	//seconds during loop

	//character strings
	char ch;			//input string to end

	//seed the random number generator
	xk = 200;

	//diplay 'computing' message
	printf("Computing iterative Lennard Jones Energies ...\n\n");

	//start the timer
	starttime = GetTickCount();
	
	//compute sigma^6, per Chapter 8
	sigma_6 = sigma * sigma * sigma * sigma * sigma * sigma;

	//compute sigma^12, per Chapter 8
	sigma_12 = sigma_6 * sigma_6;

	//start iteration
	for(i=0; i<NumIts; ++i){
		
		//get a random r between 0 and 1
		rnum = (25 * xk) % 16777216;
		r = (float)rnum / (float)16777216;

		//increment last random number
		xk = rnum;

		//get r between 1 and 7 angstroms
		r = r * 7 + 1;

		//compute r^6, per Chapter 8
		r_6 = r * r * r * r * r * r;
	
		//compute r^12, per Chapter 8
		r_12 = r_6 * r_6;

		//compute energy
		energy = sigma_12/r_12 - sigma_6/r_6;
		energy *= depth;

	}

	//stop the timer
	stoptime = GetTickCount();

	//compute the elapsed time
	elapsedtime = (stoptime - starttime) * 0.001;

	//display result
	printf("%s%g%s", "Elapsed time for Windows Console App: ",elapsedtime," seconds.\n\n");

	//display closing message and wait for <enter> before exiting
	printf("Press <Enter> to end");
	scanf("%c", &ch);

	//exit
	return 0;
}
