// Data02.c : implementation file
//
// This program explores the data typing of loop counters.  In general,
// it is assumed that loop counters should be 32 bit integers (on
// 32 bit machines with 32 bit OS's). 
//
// To illustrate this, y=sin(x) is computed at 0.01 degree increments.
// Since the sin() function takes a double arguement in radians.
//
// There are four demos in this program:
//
//		The loop is done in degrees with a double counter in fnA()
//		The loop is done in degrees with an integer counter in fnB()
//		The loop is done in radians with a double counter in fnC()
//		The loop is done in radians with an integer counter in fnD()


#include <windows.h>
#include <stdio.h>
#include <math.h>
#include <stdlib.h>

//defines
#define NumIts 1000		//number of iterations

//functions
void fnA();			
void fnB();

// global declarations
double cDeg2Rad  = 0.017453292;		//Pi/180 to convert degrees to radians
double ccDeg2Rad = 0.00017453292;   //0.01 degrees in radians
double c2Pi      = 6.283185307;		//2 * Pi
double cDegStep  = 0.01;			//Degree step size, the 'precision' we wish
									//to compute sin(angle)
double cRadStep  = 0.00001;			//Converts the radian LONG to Double in Meth4			

// computes y=sin(x) using int counter loop in degrees
void fnA()
{

	//general local declarations
	//integers
	int counter;			//counter for variable 'x'
	int ItCounter;			//iteration counters
	
	//doubles
	double dummy;			//return value for sin(x), not needed for demo

	//variables for time execution time
	//doubles
	double ElapsedTime;	//ticks during loop
	double t1;		//ticks at start of loop
	double t2;		//ticks at loop end
	
	//display entry message
	printf("\nComputing sin(x) in degrees with INTEGER loop counter...\n");
	
	//get start time
	t1 = GetTickCount();
	
	for(ItCounter=1; ItCounter <=NumIts; ++ItCounter){
        
		//using integer loop counter
		//compute y=sin(x), 0-360 degrees in 0.01 degree increments
        for(counter=0; counter <=36000; ++counter){ 
		
            //sin() gets double arg
            dummy = sin((double)counter * ccDeg2Rad);				 
        
        }
    
    }
	
	//stop the timer
	t2 = GetTickCount();
	
	//compute the elapsed time
	ElapsedTime = (t2 - t1) * 0.001;

	//display the result
	printf("%s%g%s", "\nElapsed Time = ", ElapsedTime, " seconds\n");
	
}//end fnA()

//computes y=sin(x) using double loop counter in degrees
void fnB()
{

	//general local declarations
	//integers
	int ItCounter;				//iteration counter
	
	//doubles
	double x;					//value of x, in degrees
	double dummy;				//return value for sin(x), not needed for demo

	//variables for time execution time
	//doubles
	double ElapsedTime;	//ticks during loop
	double t1;		//ticks at start of loop
	double t2;		//ticks at loop end

	//display entry message
	printf("\nComputing sin(x) in degrees with DOUBLE loop counter...\n");
	
	//get start time
	t1 = GetTickCount();
	
	for(ItCounter=1; ItCounter <= NumIts; ++ItCounter){
        
        //using double loop counter
		//compute y=sin(x), 0-360 degrees in 0.01 degree increments
		for(x=0; x <=360; x = x + 0.01){ 
		
			//sin() gets double arg
            dummy = sin(x * cDeg2Rad);		
        
        }
    
    }
	
	//stop the timer
	t2 = GetTickCount();

	//compute the elapsed time
	ElapsedTime = (t2 - t1) * 0.001;

	//display the result
	printf("%s%g%s", "\nElapsed Time = ", ElapsedTime, " seconds\n");
	
}//end fnB()

//computes y=sin(x) using double loop counter in radians
void fnC()
{

	//general local declarations
	//integers
	int ItCounter;				//iteration counter
	
	//doubles
	double arg1;				//argument to sin(x), in radians
	double dummy;				//return value for sin(x), not needed for demo

	//variables for time execution time
	//doubles
	double ElapsedTime;	//ticks during loop
	double t1;		//ticks at start of loop
	double t2;		//ticks at loop end

	//display entry message
	printf("\nComputing sin(x) in radians with DOUBLE loop counter...\n");
	
	//get start time
	t1 = GetTickCount();
	
	for(ItCounter=1; ItCounter <= NumIts; ++ItCounter){

        //using double loop counter
		//compute y=sin(x), 0-360 degrees in 0.01 degree increments
		//the argument counter is in radians
        for(arg1=0; arg1 <= c2Pi; arg1 = arg1 + ccDeg2Rad){ 
		
			//sin() gets double arg
            dummy = sin(arg1);				 
        
        }
    
    }
	
	//stop the timer
	t2 = GetTickCount();

	//compute the elapsed time
	ElapsedTime = (t2 - t1) * 0.001;

	//display the result
	printf("%s%g%s", "\nElapsed Time = ", ElapsedTime, " seconds\n");
	
}//end fnC()

//computes y=sin(x) using int loop counter in radians
void fnD()
{

	//general local declarations
	//integers
	int counter;			//counter for variable 'x'
	int ItCounter;			//iteration counters
	
	//doubles
	double dummy;			//return value for sin(x), not needed for demo

	//variables for time execution time
	//doubles
	double ElapsedTime;	//ticks during loop
	double t1;		//ticks at start of loop
	double t2;		//ticks at loop end
	
	//display entry message
	printf("\nComputing sin(x) in radians with INTEGER loop counter...\n");
	
	//get start time
	t1 = GetTickCount();
	
	for(ItCounter=1; ItCounter <=NumIts; ++ItCounter){
		
		//using integer loop counter
		//compute y=sin(x), 0-360 degrees in 0.01 degree increments
		//the argument counter is in radians
		for(counter=0; counter <= 62831853; counter=counter+1745){ 
		
            //sin() gets double arg
            dummy = sin((double)counter * ccDeg2Rad);				 
        
        }
    
    }
	
	//stop the timer
	t2 = GetTickCount();

	//compute the elapsed time
	ElapsedTime = (t2 - t1) * 0.001;

	//display the result
	printf("%s%g%s", "\nElapsed Time = ", ElapsedTime, " seconds\n");
	
}//end fnD()

//entry function
int main()
{
	//local declarations
	//integers
	int bTERM;		//menu flag

	//character strings
	char inchar;		//input character
	

	//initialize bTERM to NOT TERMINATE
	bTERM = 0;
	
	//clear terminal
	system("cls");
	
	do {
		
		//present menu to user
		printf("Enter the Number of the routine to run, and press <Enter> \n");
		printf("1\tsin(x) in degrees with INTEGER loop counter\n");
		printf("2\tsin(x) in degrees with DOUBLE loop counter\n");
		printf("3\tsin(x) in radians with DOUBLE loop counter\n");
		printf("4\tsin(x) in radians with INTEGER loop counter\n");
		printf("q\tEnd this program\n");
	
		//get user input
		scanf("%s", &inchar);

		//call the user selected function
		switch	(inchar){

			case '1': fnA();
				break;
	
			case '2': fnB();
				break;

			case '3': fnC();
				break;

			case '4': fnD();
				break;
		
			//select the terminate function	
			case 'q': printf("Ending This Program\n");
				
				//set flag to terminate
				bTERM = 1;
				break;

		}
		
	} while(!bTERM);

	return 0;

}
