// Function02.c : implementation file
//

// This program compares the performance of calling a function with
// using inline code.  The functions compute a reduced LJ pair Energy.
// Parameter passing by value and by reference are both used.
//
// The function call passing parameter by value is in fnA().
// The function call passing parameter by reference is in fnB().
// The inline routine is in fnC().


#include <windows.h>
#include <stdio.h>
#include <math.h>
#include <stdlib.h>

//defines
#define NumIts 10000000		//number of iterations

//functions
void fnA();			
void fnB();			

double calc_ljenergyRef(double *Rptr);
double calc_ljenergyVal(double R);


//perform iterative LJ Energy function call by passing distance value, r
void fnA()
{

	//general local declarations
	//integers
	int counter;			  // iteration counter

	//doubles
	double r;                 //distance, a double to pass to function
    double energy;            //energy, holder for function return value
	
	//variables for time execution time
	//doubles
	double ElapsedTime;	//ticks during loop
	double t1;		//ticks at start of loop
	double t2;		//ticks at loop end
	
	//just a value to use
	r = 1.234;
	
	//display entry message
	printf("\nCalling Function by Value ...\n");
	
	//get start time
	t1 = GetTickCount();
	
	for(counter=1; counter <= NumIts; ++counter){
    
		//call the function, passing a double and
        //returning a double
        energy = calc_ljenergyVal(r);   

    }
	
	//stop the timer
	t2 = GetTickCount();

	//compute the elapsed time
	ElapsedTime = (t2 - t1) * 0.001;

	//display the result
	printf("%s%g%s", "\nElapsed Time = ", ElapsedTime, " seconds\n");
	printf("%s%g%s", "Computed Reduced Energy = ", energy, "\n");

}//end fnA()

//perform iterative LJ Energy function call by passing a reference to r 
void fnB()
{
	//general local declarations
	//integers
	int counter;			  // iteration counter

	//doubles
	double r;                 //distance, a double to pass to function
    double energy;            //energy, holder for function return value
	
	//variables for time execution time
	//doubles
	double ElapsedTime;	//ticks during loop
	double t1;		//ticks at start of loop
	double t2;		//ticks at loop end

	//just a value to use
	r = 1.234;
	
	//display entry message
	printf("\nCalling Function by Reference ...\n");
	
	//get start time
	t1 = GetTickCount();
	
	for(counter=1; counter <= NumIts; ++counter){
    
		//call the function, passing a double and
        //returning a double
        energy = calc_ljenergyRef(&r);   

    }
	
	//stop the timer
	t2 = GetTickCount();

	//compute the elapsed time
	ElapsedTime = (t2 - t1) * 0.001;

	//display the result
	printf("%s%g%s", "\nElapsed Time = ", ElapsedTime, " seconds\n");
	printf("%s%g%s", "Computed Reduced Energy = ", energy, "\n");

}  // End fnB()

// interative inline "function"
void fnC()
{

	//general local declarations
	//integers
	int counter;			//iteration counter

	//doubles
	double r;                 //distance, a double to pass to function
	double r1;				  //holder variable, would be used in energy calc
	double r2;				  //holder variable, would be used in energy calc
    double energy;            //energy, holder for function return value
	
	//variables for time execution time
	//doubles
	double ElapsedTime;	//ticks during loop
	double t1;		//ticks at start of loop
	double t2;		//ticks at loop end

	//just a value to use
	r = 1.234;
	
	//display entry message
	printf("\nComputing iterative LJ Energy with Inline Code...\n");
	
	//get start time
	t1 = GetTickCount();
	
	for(counter=1; counter <= NumIts; ++counter){
    
		//compute r^6
        r1 = r * r * r * r * r * r;
		
		//compute r^12
		r2 = r1 * r1;

		//compute reduced energy
		energy = 1/r2 - 1/r1;

    }
	
	//stop the timer
	t2 = GetTickCount();

	//compute the elapsed time
	ElapsedTime = (t2 - t1) * 0.001;

	//display the result
	printf("%s%g%s", "\nElapsed Time = ", ElapsedTime, " seconds\n");
	printf("%s%g%s", "Computed Reduced Energy = ", energy, "\n");
	
}//end fnC()

//entry function
int main()
{
	//local declarations
	//integers
	int bTERM;		//menu flag

	//character strings
	char inchar;		//input character
	

	//initialize bTERM to NOT TERMINATE
	bTERM = 0;
	
	//clear terminal
	system("cls");
	
	do {
		
		//present menu to user
		printf("Enter the Number of the routine to run, and press <Enter> \n");
		printf("1\tIterative Function Call passing 1 parameter by value\n");
		printf("2\tIterative Function Call passing 1 parameter by reference\n");
		printf("3\tIterative Inline Function\n");
		printf("q\tEnd this program\n");
	
		//get user input
		scanf("%s", &inchar);

		//call the user selected function
		switch	(inchar){

			case '1': fnA();
				break;
	
			case '2': fnB();
				break;

			case '3': fnC();
				break;
		
			//select the terminate function	
			case 'q': printf("Ending This Program\n");
				
				//set flag to terminate
				bTERM = 1;
				break;

		}
		
	} while(!bTERM);

	return 0;

} // End Main

double calc_ljenergyVal (double R)
{

	double R1;		//used internally in the 
	double R2;		//optimized lj energy computation
	double En;		//Energy

	R1 = R * R * R * R * R * R;
	R2 = R1 * R1;

	En = 1/R2 - 1/R1;

	return (En);

}

double calc_ljenergyRef (double *Rptr)
{

	double R1;		//used internally in the 
	double R2;		//optimized lj energy computation
	double En;

	R1 = *Rptr;
	R1 = R1 * R1 * R1 * R1 * R1 * R1;
	R2 = R1 * R1;

	En = 1/R2 - 1/R1;

	return (En);

}
