// Other03.c : implementation file
//
//This program demonstrates the use of binary flag encoding
//as an alternative to multiple If decision  statements.
//
//The particular 'application' is to select an interaction energy
//type for the pair energy in a system with three types of particles.
//In this system, there are six possible interactions arising from
//the combinations of three particles.
//
//It is assumed that the energy for 'particle 1' is being computed.
//
//The first method to select this interaction energy type, one that
//is commonly used, is to use multiple If/Then (or, as in this example,
//If/ElseIf) statements.  See the function fnB().
//
//An alternative method, borrowed from a technique often used in
//system programming, is to encode the particle type in a binary
//flag structure (in this simple case, only a single int is used).
//With this data structure, no If/Then code structure is needed;
//the energy type is selected by simple integer addition or via
//an OR statement (as illustrated in function fnC().

#include <windows.h>
#include <stdio.h>
#include <math.h>

#define NUMITS 50000000

//global declarations
//integers
int bINIT;			//initialization flag
int r[50000001];		//random 1's,2's and 4's

//functions
void fnA();			
void fnB();
void fnC();			

double calc_ljenergy(double R);		// the called function

//initialize the array of random particle types
void fnA()
{

	//local declarations
	//integers
	int rcounter;				//counter used to initialize random array
	int randomval;


	//initialize the random number array
	for(rcounter = 1; rcounter <= NUMITS; ++rcounter){

		//rand() returns an integer < about 32000
		randomval = rand();
		
		//want random 1's, 2's, 4's for this demo
		if(randomval <= 10666) r[rcounter] = 1;
		if(randomval >= 21332) r[rcounter] = 4;
		if((randomval < 21332) & (randomval > 10666)) r[rcounter] = 2;
		
	}


        bINIT = 1;
	
}//end fnA()

//selection via if decision structure
void fnB()
{

	//local declarations
	//integers
	int counter;		//iteration counter
	int typeval;		//determined type

	//values needed for timing functions
	double ElapsedTime;	//ticks during loop
	double t1;		//ticks at start of loop
	double t2;		//ticks at loop end
		
	//display entry message
	printf("\nDecision via If statements...\n");
	
	//get start time
	t1 = GetTickCount();
	
	//select via if statement
	for(counter = 2; counter <= NUMITS; ++counter){

		//decision to select energy type
		if((r[counter] == 1) && (r[1] == 1)) typeval = 1;
		if((r[counter] == 2) && (r[1] == 1)) typeval = 3;
		if((r[counter] == 4) && (r[1] == 1)) typeval = 5;
		if((r[counter] == 1) && (r[1] == 2)) typeval = 3;
		if((r[counter] == 2) && (r[1] == 2)) typeval = 2;
		if((r[counter] == 4) && (r[1] == 2)) typeval = 6;
		if((r[counter] == 1) && (r[1] == 4)) typeval = 5;
		if((r[counter] == 2) && (r[1] == 4)) typeval = 6;
		if((r[counter] == 4) && (r[1] == 4)) typeval = 4;

	}
	
	//stop the timer
	t2 = GetTickCount();

	//compute the elapsed time
	ElapsedTime = (t2 - t1) * 0.001;

	//display the result
	printf("%s%g%s", "\nElapsed Time = ", ElapsedTime, " seconds\n");
	
}//end fnB()

//assignment of type via binary flag
void fnC(){

	//local declarations
	//integers
	int counter;		//iteration counter
	int typeval;		//determined type
 
	//values needed for timing functions
	double ElapsedTime;	//ticks during loop
	double t1;		//ticks at start of loop
	double t2;		//ticks at loop end
		
	//display entry message
	printf("\nIterative Full Energy Calculation...\n");
	
	//get start time
	t1 = GetTickCount();
	
	//select via if statement
	for(counter = 2; counter <= NUMITS; ++counter){

		//decision to select energy type
		typeval = (r[counter] | r[1]);

	}

	//stop the timer
	t2 = GetTickCount();

	//compute the elapsed time
	ElapsedTime = (t2 - t1) * 0.001;

	//display the result
	printf("%s%g%s", "\nElapsed Time = ", ElapsedTime, " seconds\n");

}//end fnC()

//entry function
int main()
{
	//local declarations
	//integers
	int bTERM;		//menu flag

	//character strings
	char inchar;		//input character
	

	//initialize bINIT to NOT INITIATED
	bINIT = 0;

	//initialize bTERM to NOT TERMINATE
	bTERM = 0;
	
	//clear terminal
	system("clear");
	
	do {
		
		//present menu to user
		printf("Enter the Number of the routine to run, and press <Enter> \n");
		printf("1\tInitializations\n");

		if(bINIT){
		
			printf("2\t50,000,000 If Decisions\n");
			printf("3\t50,000,000 Decisions via bit encoding\n");
		
		}
		
		printf("q\tEnd this program\n");
	
		//get user input
		scanf("%s", &inchar);
		
		if((bINIT==0)&&((inchar=='2')||(inchar=='3'))){
		   
		   inchar=0;
		   
		}

		//call the user selected function
		switch	(inchar){

			case '1': fnA();
				break;
	
			case '2': fnB();
				break;
				
			case '3': fnC();
				break;
		
			//select the terminate function	
			case 'q': printf("Ending This Program\n");
				
				//set flag to terminate
				bTERM = 1;
				break;

		}
		
	} while(!bTERM);

	return 0;

} // End Main

double calc_ljenergy (double R)
{

	double R1;		//these would be used internally in the 
	double R2;		//optimized lj energy computation

	R1 = 0;
	R2 = 0;

	return(0);

}
