// Loop05Dlg.cpp : implementation file
//
// This program solves the time dependent heat equation numerically
// using the Finite Volume Method.  Gauss-Seidel iteration is used to
// solve the coupled equations.  The time dependence is modeled using
// the fully implicit (and more computationally costly) method.
//
// The rolled version of the solution is in CLoop05Dlg::OnButton1()
// The unrolled version is CLoop05Dlg::OnButton2()

#include "stdafx.h"
#include "Loop05.h"
#include "Loop05Dlg.h"
#include "math.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//defines
#define tStep	0.001		//time step, in seconds
#define tStop	1200		//ending time, in seconds
#define OptTol	0.00001		//tolerance for convergence test

/////////////////////////////////////////////////////////////////////////////
// CLoop05Dlg dialog

CLoop05Dlg::CLoop05Dlg(CWnd* pParent /*=NULL*/)
	: CDialog(CLoop05Dlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CLoop05Dlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CLoop05Dlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CLoop05Dlg)
		// NOTE: the ClassWizard will add DDX and DDV calls here
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CLoop05Dlg, CDialog)
	//{{AFX_MSG_MAP(CLoop05Dlg)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_BUTTON1, OnButton1)
	ON_BN_CLICKED(IDC_BUTTON2, OnButton2)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CLoop05Dlg message handlers

BOOL CLoop05Dlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
	
	// TODO: Add extra initialization here
	
	return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CLoop05Dlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CLoop05Dlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}


//This routine uses rolled up loops for the Gauss Seidel Solver
void CLoop05Dlg::OnButton1() 
{
	//local declarations
	//integers
	int counter;			//handy counter for small loops
	int i;					//row counter in Gauss-Seidel solver
	int k;					//column counter in Gauss-Seidel solver
	int NumIts;				//number of iterations in Gauss-Seidel solver
							//this is used to prevent infinite loop in case
							//of poor or non-convergence in the solver
	int j;					//output string pointer

	//floats
	float delta_T;			//max change in Temperature; used to test for convergence
	float max_delta_T;		//used to determine the delta_T
	float sum1;				//a temporary summation holder
	float sum2;				//a temporary summation holder
	float T_old[5];			//the set of temperatures at the last iteration
	float tCounter;			//time counter
	float Tdiff;			//individual Temperature difference; used
							//to compute max_delta_T

	float a[5][5];			//the coefficient matrix in the linear system
	float b[5];				//'right hand side' of linear system
	float T0[5];			//node temperatures at start of given time step
							//these are updated at the end of each time step
	float T[5];				//node temperatures at five spacial locations

	//doubles
	double starttime;		//system ticks at start procedure
	double stoptime;		//system ticks at end of procedure
	double ElapsedTime;		//system ticks during procedure
	

	//character strings
	char buffer[200];		//output string

	//initialize the linear system using data from Ch. 7
	//initial node temperatures
	for (counter=0; counter <= 4; ++counter){

		T0[counter] = 200;
		T[counter] = 200;

	}

	//coefficient matrix
	a[0][0] = 400025;
	a[0][1] = -25;
	a[0][2] = 0;
	a[0][3] = 0;
	a[0][4] = 0;

	a[1][0] = -25;
	a[1][1] = 400050;
	a[1][2] = -25;
	a[1][3] = 0;
	a[1][4] = 0;

    a[2][0] = 0;
	a[2][1] = -25;
	a[2][2] = 400050;
	a[2][3] = -25;
	a[2][4] = 0;

	a[3][0] = 0;
	a[3][1] = 0;
	a[3][2] = -25;
	a[3][3] = 400050;
	a[3][4] = -25;

	a[4][0] = 0;
	a[4][1] = 0;
	a[4][2] = 0;
	a[4][3] = -25;
	a[4][4] = 400075;

    
	//start the timer
    starttime = GetTickCount();		

	for (tCounter = (float)0.001; tCounter <= (float)tStop; tCounter += (float)tStep){

		//set right side of the system
		for(counter = 0; counter <=4; ++counter){

			b[counter] = 400000 * T0[counter];

		}

		//solve the system for T(j) using Gauss-Seidel procedure

		NumIts = 1;
		delta_T = 1;
		max_delta_T = 0;

		do{

			//save the old T(j)'s
			for(counter = 0; counter <= 4; ++counter){

				T_old[counter] = T[counter];

			}

			for(i=0; i<= 4; ++i){

				sum1 = 0;

				for(k=0; k <= i-1; ++k){

					sum1 += a[i][k]*T[k];

				}

				sum2 = 0;

				for(k = i+1; k<=4; ++k){

					sum2 += a[i][k]*T_old[k];

				}

				T[i] = sum1 + sum2 - b[i];
				T[i] /= -a[i][i];

				Tdiff = (float)fabs(T[i] - T_old[i]);

				if(Tdiff > max_delta_T){

					max_delta_T = Tdiff;

				}

			}

			++NumIts;
			delta_T = max_delta_T;
			max_delta_T = 0;

		}while ((delta_T >= OptTol) & (NumIts <= 1000));

		//update the 'initial' temperatures for the beginning
		//of the next time step

		for(counter = 0; counter <=4; ++counter){

			T0[counter] = T[counter];

		}

	}

    
	//stop the timer
    stoptime = GetTickCount();		
    
	//compute the elapsed time, in seconds
    ElapsedTime = (stoptime - starttime) * .001;

	//display the result in a message box
	j=sprintf(buffer, "Using Rolled Loop Solver, solved time dependent problem in %g", ElapsedTime);
	j += sprintf(buffer + j, "%s", " seconds.\n");

	MessageBox(buffer, "Done with Rolled-Up Solution",
				MB_OK | MB_ICONEXCLAMATION);
		
}


//this routine uses unrolled loops for the Gauss-Seidel solver
void CLoop05Dlg::OnButton2() 
{
	//local declarations
	//integers
	int counter;			//handy counter for small loops
	int NumIts;				//number of iterations in Gauss-Seidel solver
							//this is used to prevent infinite loop in case
							//of poor or non-convergence in the solver
	int j;					//output string pointer

	//floats
	float delta_T;			//max change in Temperature; used to test for convergence
	float max_delta_T;		//used to determine the delta_T
	float sum1;				//a temporary summation holder
	float sum2;				//a temporary summation holder
	float T_old[5];			//the set of temperatures at the last iteration
	float tCounter;			//time counter
	float Tdiff;			//individual Temperature difference; used
							//to compute max_delta_T

	float a[5][5];			//the coefficient matrix in the linear system
	float b[5];				//'right hand side' of linear system
	float T0[5];			//node temperatures at start of given time step
							//these are updated at the end of each time step
	float T[5];				//node temperatures at five spacial locations

	//doubles
	double starttime;		//system ticks at start procedure
	double stoptime;		//system ticks at end of procedure
	double ElapsedTime;		//system ticks during procedure
	

	//character strings
	char buffer[200];		//output string

	//initialize the linear system using data from Ch. 7
	//initial node temperatures
	for (counter=0; counter <= 4; ++counter){

		T0[counter] = 200;
		T[counter] = 200;

	}

	//coefficient matrix
	a[0][0] = 400025;
	a[0][1] = -25;
	a[0][2] = 0;
	a[0][3] = 0;
	a[0][4] = 0;

	a[1][0] = -25;
	a[1][1] = 400050;
	a[1][2] = -25;
	a[1][3] = 0;
	a[1][4] = 0;

    a[2][0] = 0;
	a[2][1] = -25;
	a[2][2] = 400050;
	a[2][3] = -25;
	a[2][4] = 0;

	a[3][0] = 0;
	a[3][1] = 0;
	a[3][2] = -25;
	a[3][3] = 400050;
	a[3][4] = -25;

	a[4][0] = 0;
	a[4][1] = 0;
	a[4][2] = 0;
	a[4][3] = -25;
	a[4][4] = 400075;

    
	//start the timer
    starttime = GetTickCount();		

	for (tCounter = (float)0.001; tCounter <= (float)tStop; tCounter += (float)tStep){

		//set right side of the system
		for(counter = 0; counter <=4; ++counter){

			b[counter] = 400000 * T0[counter];

		}

		//solve the system for T(j) using Gauss-Seidel procedure

		NumIts = 1;
		delta_T = 1;
		max_delta_T = 0;

		do{

			//save the old T(j)'s

			T_old[0] = T[0];
			T_old[1] = T[1];
			T_old[2] = T[2];
			T_old[3] = T[3];
			T_old[4] = T[4];

			//this begins the unrolled Gauss-Seidel procedure
			
			//i=0
			//k=1-4
			sum2 = a[0][1]*T_old[1] + a[0][2]*T_old[2] + a[0][3]*T_old[3] + a[0][4]*T_old[4];

			T[0] = sum2 - b[0];
			T[0] /= -a[0][0];

			max_delta_T = (float)fabs(T[0] - T_old[0]);

			//i=1
			//k=0

			//k=2-4
			sum2 = a[1][2]*T_old[2] + a[1][3]*T_old[3] + a[1][4]*T_old[4];
			
			T[1] = a[1][0]*T[0] + sum2 - b[1];
			T[1] /= -a[1][1];

			Tdiff = (float)fabs(T[1] - T_old[1]);
				
			if(Tdiff > max_delta_T){

				max_delta_T = Tdiff;

			}

			//i=2
			//k=0-1
			sum1 = a[2][0]*T[0] + a[2][1]*T[1];

			//k=3-4
			sum2 = a[2][3]*T_old[3] + a[2][4]*T_old[4];

			T[2] = sum1 + sum2 - b[2];
			T[2] /= -a[2][2];

			Tdiff = (float)fabs(T[2] - T_old[2]);
				
			if(Tdiff > max_delta_T){

				max_delta_T = Tdiff;

			}

			//i=3
			//k=0-2
			sum1 = a[3][0]*T[0] + a[3][1]*T[1] + a[3][2]*T[2];

			T[3] = sum1 + a[3][4]*T[4] - b[3];
			T[3] /= -a[3][3];

			Tdiff = (float)fabs(T[3] - T_old[3]);
				
			if(Tdiff > max_delta_T){

				max_delta_T = Tdiff;
			
			}
				
			//i=4
			//k=0-3
			sum1 = a[4][0]*T[0] + a[4][1]*T[1] + a[4][2]*T[2] + a[4][3]*T[3];

			T[4] = sum1 - b[4];
			T[4] /= -a[4][4];

			Tdiff = (float)fabs(T[4] - T_old[4]);
				
			if(Tdiff > max_delta_T){

				max_delta_T = Tdiff;

			}

			++NumIts;
			delta_T = max_delta_T;

		}while ((delta_T >= OptTol) & (NumIts <= 1000));

		//update the 'initial' temperatures for the beginning
		//of the next time step

		T0[0] = T[0];
		T0[1] = T[1];
		T0[2] = T[2];
		T0[3] = T[3];
		T0[4] = T[4];

	}

    
	//stop the timer
    stoptime = GetTickCount();		
    
	//compute the elapsed time, in seconds
    ElapsedTime = (stoptime - starttime) * .001;

	//display the result in a message box
	j=sprintf(buffer, "Using Unrolled Loop Solver, solved time dependent problem in %g", ElapsedTime);
	j += sprintf(buffer + j, "%s", " seconds.\n");

	MessageBox(buffer, "Done with Unrolled Solution",
				MB_OK | MB_ICONEXCLAMATION);
	
	
}
