// RISC01Dlg.cpp : implementation file
//
// This program uses the Lennard Jones pair energy calculation
// to illustrate programming in the RISC Style.
//
// This program is performing a very simple floating point 
// computation.  The more floating point work done within the
// interative loop, of course, the greater the benefit.  For example, 
// similar gains will be observed with the computation of the actual 
// distances to be used for the LJ calculation (see RISC02 and RISC03).

#include "stdafx.h"
#include "RISC01.h"
#include "RISC01Dlg.h"
#include "math.h"
#include "stdio.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define NUMITS 10000000

/////////////////////////////////////////////////////////////////////////////
// CRISC01Dlg dialog

CRISC01Dlg::CRISC01Dlg(CWnd* pParent /*=NULL*/)
	: CDialog(CRISC01Dlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CRISC01Dlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CRISC01Dlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CRISC01Dlg)
		// NOTE: the ClassWizard will add DDX and DDV calls here
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CRISC01Dlg, CDialog)
	//{{AFX_MSG_MAP(CRISC01Dlg)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_BUTTON2, OnButton2)
	ON_BN_CLICKED(IDC_BUTTON1, OnButton1)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CRISC01Dlg message handlers

BOOL CRISC01Dlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
	
	// TODO: Add extra initialization here
	
	return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CRISC01Dlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CRISC01Dlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}

//computes a reduced LJ pair energy in a RISC algorithm
void CRISC01Dlg::OnButton2() 
{
	
	//local declarations
	//integers
	int j;					//output char pointer
	int RiscCounter;		//iteration counter

	//doubles
	double RiscR;			//entry value distance
	double denom1;			//RiscR^6
	double denom2;			//RiscR^12
	double RiscEnergy;		//computed Energy
	double starttime;		//system ticks at start of loop
	double stoptime;		//system ticks at loop exit
	double ElapsedTime;		//system ticks during loop

	//character strings
	char buffer[200];		//output string

	//assign a value to use
	RiscR = 1.673;

	//start the timer
	starttime = GetTickCount();

	for(RiscCounter=1; RiscCounter <=NUMITS; ++RiscCounter){

		//compute RiscR^6
		denom1 = RiscR * RiscR * RiscR * RiscR * RiscR * RiscR;
		
		//compute RiscR^12
		denom2 = denom1 * denom1;

		//compute LJ pair energy
		RiscEnergy = 1/denom2 - 1/denom1;

	}

	//stop the timer
	stoptime = GetTickCount();

	//compute the elapsed time, in seconds
	ElapsedTime = (stoptime - starttime) * 0.001;
	
	//build output string and display in a message box
	j = sprintf(buffer,"Computed Energy 10,000,000 times by RISC style in %g ",ElapsedTime);
	j += sprintf(buffer + j, "%s"," seconds.");

	MessageBox(buffer,
					 "Done with RISC",
					 MB_OK | MB_ICONEXCLAMATION);

}

// computes a reduced LJ Energy CISC style
void CRISC01Dlg::OnButton1() 
{

	//local declarations
	//integers
	int j;					//output string pointer
	int counter;			//iteration counter

	//doubles
	double Energy;			//computed energy		
	double r;				//entry value distance
	double starttime;		//system ticks at start of loop
	double stoptime;		//system ticks at loop exit
	double ElapsedTime;		//system ticks during loop

	//character strings 
	char buffer[200];		//output string

	//assign a value to use
	r=1.673;

	//start the timer
	starttime = GetTickCount();

	for(counter = 1; counter<= NUMITS; ++counter){

		//compute LJ Energy in CISC style
		//equation 'appears' as math on paper
		Energy = 1/pow(r,12) - 1/pow(r,6);
		
	}

	//stop the timer
	stoptime = GetTickCount();

	//compute the elapsed time
	ElapsedTime = (stoptime - starttime) * 0.001;
	
	//build the output string and display in a message box
	j = sprintf(buffer,"Computed Energy NUMITS times by CISC style in %g ",ElapsedTime);
	j += sprintf(buffer + j, "%s"," seconds.");


	MessageBox(buffer,
			   "Done with CISC Computation",
			   MB_OK | MB_ICONEXCLAMATION);

}

void CRISC01Dlg::OnOK() 
{
	// TODO: Add extra cleanup here
	
	CDialog::OnOK();
}

void CRISC01Dlg::OnCancel() 
{
	// TODO: Add extra cleanup here
	
	CDialog::OnCancel();
}

