// RISC04Dlg.cpp : implementation file
//
// This program demonstrates the use of the RISC style for the Boltzmann
// factor computation.  In this case, RISC style code is not an
// optimization.

#include "stdafx.h"
#include "RISC04.h"
#include "RISC04Dlg.h"
#include "stdio.h"
#include "math.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//global declarations
//integers
int cNumIts = 10000000;

//doubles
double Temp = 89.45;
double kBoltz = 1.381E-23;
double Energy = 4.52E-20;

/////////////////////////////////////////////////////////////////////////////
// CRISC04Dlg dialog

CRISC04Dlg::CRISC04Dlg(CWnd* pParent /*=NULL*/)
	: CDialog(CRISC04Dlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CRISC04Dlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CRISC04Dlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CRISC04Dlg)
		// NOTE: the ClassWizard will add DDX and DDV calls here
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CRISC04Dlg, CDialog)
	//{{AFX_MSG_MAP(CRISC04Dlg)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDCISC, OnCisc)
	ON_BN_CLICKED(IDRISC, OnRisc)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CRISC04Dlg message handlers

BOOL CRISC04Dlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
	
	// TODO: Add extra initialization here
	
	return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CRISC04Dlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CRISC04Dlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}

// Event handlers for user click on CISC button
void CRISC04Dlg::OnCisc() 
{

	int Counter;		// iteration counter
	int j;				// for output to message box

	char buffer[200];	// for output to message box
    
    double BoltzFactor;	// result of exp(arg) calculation
	double starttime;	// system ticks at start of loop
	double stoptime;	// system ticks at end of loop
	double ElapsedTime;	// stoptime - starttime
    
    //start the timer
    starttime = GetTickCount();		
    
    for(Counter=1; Counter <= cNumIts; ++Counter){

		// CISC Boltzman calc
        BoltzFactor = exp(-Energy / (kBoltz * Temp));	
    
    }
    
	//stop the timer
    stoptime = GetTickCount();		

	// compute elapsed time
	ElapsedTime = (stoptime - starttime) * 0.001;	
	

	//display result to message box
	j = sprintf(buffer,"Computed Boltzman Factor 10,000,000 times by CISC style in %g ",ElapsedTime);
	j += sprintf(buffer + j, "%s"," seconds.");

	MessageBox(buffer,
					 "Done with CISC exp(arg)",
					 MB_OK | MB_ICONEXCLAMATION);



}

// Event handler for user click on RISC button
void CRISC04Dlg::OnRisc() 
{
	//local declarations
	//integers
	int Counter;		// iteration counter
	int j;				// for output to message box

	//character strings
	char buffer[200];	// for output to message box
    
	//doubles
    double BoltzFactor;	// result of exp(arg) calc
	double Temp1;		// intermediate variable
	double arg1;		// actual argument passed to exp() function
	double starttime;	// system ticks at start of loop
	double stoptime;	// system ticks at end of loop
	double ElapsedTime;	// stoptime - starttime
    
    //start the timer
    starttime = GetTickCount();		
    
    for(Counter=1; Counter <= cNumIts; ++Counter){

		//compute factor in RISC style
		Temp1 = kBoltz * Temp;		
		arg1 = -Energy / Temp1;		

        BoltzFactor = exp(-arg1);
    
    }
    
	// stop the timer
    stoptime = GetTickCount();		

	// compute elapsed time during loop
	ElapsedTime = (stoptime - starttime) * 0.001;	
	
	//display the result in a message box
	j = sprintf(buffer,"Computed Boltzman Factor 10,000,000 times by RISC style in %g ",ElapsedTime);
	j += sprintf(buffer + j, "%s"," seconds.");

	MessageBox(buffer,
					 "Done with RISC exp(arg)",
					 MB_OK | MB_ICONEXCLAMATION);


}
