// RISC05Dlg.cpp : implementation file
//
// This program demonstrates the elimination of wasted computation
// in an algorithm.  The 'application' is the combined distance
// and LJ energy calculation.  In the routine shown in RISC03, the
// distance is computed using a square root, then the distance is
// 'squared' to obtain r^6.
//
// The optimization shown in this demo eliminates the squaring of the
// square root; in a sense, r^2 is used in place of r, and takes
// advantage of r^6 = (r^2)^3.

#include "stdafx.h"
#include "RISC05.h"
#include "RISC05Dlg.h"
#include "math.h"
#include "stdio.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CRISC05Dlg dialog

CRISC05Dlg::CRISC05Dlg(CWnd* pParent /*=NULL*/)
	: CDialog(CRISC05Dlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CRISC05Dlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CRISC05Dlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CRISC05Dlg)
		// NOTE: the ClassWizard will add DDX and DDV calls here
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CRISC05Dlg, CDialog)
	//{{AFX_MSG_MAP(CRISC05Dlg)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDwithsqr, Onwithsqr)
	ON_BN_CLICKED(IDwithoutsqr, Onwithoutsqr)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CRISC05Dlg message handlers

BOOL CRISC05Dlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
	
	// TODO: Add extra initialization here
	
	return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CRISC05Dlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CRISC05Dlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}

// Compute r and E in RISC style algorithm, with squaring the square
//root
void CRISC05Dlg::Onwithsqr() 
{

	// local declarations
	// integers
	int j;							//output string point
	int counter;					//iteration counter

	// doubles
	double x1 = 1.234;				//entry value coordinates
	double x2 = 2.675;				//with some values to use
	double y1 = 0.045;
	double y2 = 2.073;
	double tempx;					//deltax, then deltax^2
	double tempy;					//deltay, then deltay^2
	double r;						//computed distance, using square
									//root
	double denom1;					//r^6
	double denom2;					//r^12
	double Energy;					//computed LJ energy
	double starttime;				//system ticks at start of loop
	double stoptime;				//system ticks at loop exit
	double ElapsedTime;				//system ticks during loop

	//character strings
	char buffer[200];				//output string

	//start the timer
	starttime = GetTickCount();

	for(counter = 1; counter <= 10000000; ++counter){

		//compute distance as in RISC02, using RISC style
		tempx = x1 - x2;
		tempx = tempx * tempx;
		tempy = y1 - y2;
		tempy = tempy * tempy;

		//actual distance is computed
		r = sqrt(tempx + tempy);

		//compute the LJ Energy in RISC style using computed r
		//compute r^6
		denom1 = r * r * r * r * r * r;
		
		//compute r^12 = (r^6)^2
		denom2 = denom1 * denom1;

		//compute the reduced LJ Energy
		Energy = 1/denom2 - 1/denom1;

	}

	//stop the timer
	stoptime = GetTickCount();

	//compute the elapsed time
	ElapsedTime = (stoptime - starttime) * 0.001;

	//build the output string and display in a message box
	j = sprintf(buffer, "Computed r and E 10,000,000 times in RISC style in ");
	j += sprintf(buffer + j, "%g ", ElapsedTime);
	j += sprintf(buffer + j, "%s", " seconds.");

	MessageBox(buffer, "Done with RISC Computation",
				MB_OK | MB_ICONEXCLAMATION);
	


}

// Compute r and E via RISC style algorithm, eliminating the
// superfluous sqrt followed by squaring
void CRISC05Dlg::Onwithoutsqr() 
{
	// local declarations
	// integers
	int j;							//output string pointer
	int counter;					//iteration counter

	// doubles
	double x1 = 1.234;				//entry value coordinates
	double x2 = 2.675;				//with some values to use
	double y1 = 0.045;
	double y2 = 2.073;
	double tempx;					//deltax, then deltax^2
	double tempy;					//deltay, then deltay^2
	double r;						//distance^2, since actual
									//distance is not needed in this
									//calculation
	double denom1;					//distance^6
	double denom2;					//distance^12
	double Energy;					//computed energy
	double starttime;				//system ticks at start of loop
	double stoptime;				//system ticks at loop exit
	double ElapsedTime;				//system ticks during loop

	//character strings
	char buffer[200];				//output string

	//start the timer
	starttime = GetTickCount();

	for(counter = 1; counter <= 10000000; ++counter){

		//compute distance^2 in RISC algorithm
		tempx = x1 - x2;
		tempx = tempx * tempx;
		tempy = y1 - y2;
		tempy = tempy * tempy;

		//no sqrt() since actual r is not needed
		r = tempx + tempy;

		//compute distance^6, which is (distance^2)^3
		denom1 = r * r * r;

		//compute distance^12
		denom2 = denom1 * denom1;

		//compute reduced LJ energy
		Energy = 1/denom2 - 1/denom1;

	}

	//stop the timer
	stoptime = GetTickCount();

	//compute the elapsed time
	ElapsedTime = (stoptime - starttime) * 0.001;

	//build output string and display result in a message box
	j = sprintf(buffer, "Computed r and E 10,000,000 times in RISC style in ");
	j += sprintf(buffer + j, "%g ", ElapsedTime);
	j += sprintf(buffer + j, "%s", " seconds.");

	MessageBox(buffer, "Done with RISC Computation",
				MB_OK | MB_ICONEXCLAMATION);
	


}
