// LookUp01Dlg.cpp : implementation file
//
// This program demonstrates the Look Up Table 
// technique for the 'reduced' Lennard Jones
// pair energy.
//
// The Look Up Table is initialized in CLookUp01Dlg::OnInitialize()  
// The explicit calculation is in CLookUp01Dlg::OnCompute() 
// The Look Up Table calculation is in CLookUp01Dlg::OnLookUp()

#include "stdafx.h"
#include "LookUp01.h"
#include "LookUp01Dlg.h"
#include "math.h"
#include "stdio.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//global declarations

//integers
int cNumIts = 10000000;
    
int r = 1023;    //distance * 1000, 1.023 angstoms


//doubles
double Energy[5700];  //The Look-Up Table

  

/////////////////////////////////////////////////////////////////////////////
// CLookUp01Dlg dialog

CLookUp01Dlg::CLookUp01Dlg(CWnd* pParent /*=NULL*/)
	: CDialog(CLookUp01Dlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CLookUp01Dlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CLookUp01Dlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CLookUp01Dlg)
		// NOTE: the ClassWizard will add DDX and DDV calls here
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CLookUp01Dlg, CDialog)
	//{{AFX_MSG_MAP(CLookUp01Dlg)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDCompute, OnCompute)
	ON_BN_CLICKED(IDInitialize, OnInitialize)
	ON_BN_CLICKED(IDLookUp, OnLookUp)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CLookUp01Dlg message handlers

BOOL CLookUp01Dlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
	
	// TODO: Add extra initialization here
	
	return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CLookUp01Dlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CLookUp01Dlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}

// explicit computation of the LJ energy
void CLookUp01Dlg::OnCompute() 
{

	//local declarations
	//integers
	int j;				// output string pointer
	int RiscCounter;	// iteration counter
	
	//doubles
	double RiscR;		// distance 
	double R6;			// distance^6
	double R12;			// distance^12
	double RiscEnergy;	// computed 'energy'
	double starttime;	// ticks at start of loop
	double stoptime;	// ticks at loop exit
	double ElapsedTime;	// ticks during loop
	
	//character strings
	char buffer[200];	// output string

	//just a value to use for distance
	RiscR = 1.023;

	//start the timer
	starttime = GetTickCount();

	for(RiscCounter=1; RiscCounter <=10000000; ++RiscCounter){

		//compute distance^6
		R6 = RiscR * RiscR * RiscR * RiscR * RiscR * RiscR;
		
		//compute distance^12
		R12 = R6 * R6;

		//compute 'energy'
		RiscEnergy = 1/R12 - 1/R6;

	}

	//stop the timer
	stoptime = GetTickCount();

	//compute loop time in seconds
	ElapsedTime = (stoptime - starttime) * 0.001;
	
	//build the output string and display in message box
	j = sprintf(buffer,"Computed Energy 10,000,000 times by RISC style in %g ",ElapsedTime);
	j += sprintf(buffer + j, "%s"," seconds. ");
	j += sprintf(buffer + j, "\nEnergy = %.15g ", RiscEnergy);

	MessageBox(buffer,
					 "Explicit Risc Energy Calculation",
					 MB_OK | MB_ICONEXCLAMATION);


}

// function to initialize the look up table
void CLookUp01Dlg::OnInitialize() 
{

	//local declarations
	int j;				// output string pointer
	int Counter;		// assignment index
	
	//doubles
	double RiscR;		// distance
	double R6;			// distance^6
	double R12;			// distance^12
	double starttime;	// ticks at start of loop
	double stoptime;	// ticks at loop exit
	double ElapsedTime;	// ticks during loop
	
	//character strings
	char buffer[200];	// output string

	//start the timer
	starttime = GetTickCount();

	for(Counter=50; Counter <=5657; ++Counter){

		//compute distance from index
		RiscR = (double)Counter * 0.001;

		//compute distance^6
		R6 = RiscR * RiscR * RiscR * RiscR * RiscR * RiscR;

		//compute distance^12
		R12 = R6 * R6;

		//compute reduced energy for this index
		Energy[Counter] = 1/R12 - 1/R6;

	}

	//stop the timer
	stoptime = GetTickCount();

	//compute the loop time in seconds
	ElapsedTime = (stoptime - starttime) * 0.001;

	//build the output string and display in message box
	j = sprintf(buffer,"Computed Energy Look Up Table in %.3g ",ElapsedTime);
	j += sprintf(buffer + j, "%s"," seconds. ");

	MessageBox(buffer,
					 "Look Up Table Initialization",
					 MB_OK | MB_ICONEXCLAMATION);
	

}

// function to 'compute' energy using look up table
void CLookUp01Dlg::OnLookUp() 
{

	//local declarations
	//integers
	int j;				// output string pointer
	int RiscCounter;	// iteration counter
	
	//doubles
	double ThisEnergy;	// 'computed' energy
	double starttime;	// ticks at start of loop
	double stoptime;	// ticks at loop exit
	double ElapsedTime;	// loop time
	
	//character string
	char buffer[200];	// output string

	//start the timer
	starttime = GetTickCount();

	for(RiscCounter=1; RiscCounter <=10000000; ++RiscCounter){

		//look up the energy
		ThisEnergy = Energy[r];

	}

	//stop the timer
	stoptime = GetTickCount();

	//compute the loop time in seconds
	ElapsedTime = (stoptime - starttime) * 0.001;
	
	//build the output string and display in message box
	j = sprintf(buffer,"Looked Up Energy 10,000,000 times in %g ",ElapsedTime);
	j += sprintf(buffer + j, "%s"," seconds. ");
	j += sprintf(buffer + j, "\nEnergy = %.15g ", ThisEnergy);

	MessageBox(buffer,
					 "Look Up Energies",
					 MB_OK | MB_ICONEXCLAMATION);

}
