// LookUp03Dlg.cpp : implementation file
//
// This program demonstrates the Look Up Table technique for several
// common math operations
//
// Working Functions are:
//
//		CLookUp03Dlg::OnInitialize()  initializes the look-up arrays
//		CLookUp03Dlg::OnExplicitSin() performs sin(x) by explicit calc.
//		CLookUp03Dlg::OnExplicitln()  performs ln(x) by explicit calc.
//		CLookUp03Dlg::OnExplicitpwr() performs x^y by explicit calc.
//		CLookUp03Dlg::OnLUTsin()      performs sin(x) using LUT
//		CLookUp03Dlg::OnLUTln()       performs ln(x) using LUT
//		CLookUp03Dlg::OnLUTpwr()      performs x^y using LUT
//

#include "stdafx.h"
#include "LookUp03.h"
#include "LookUp03Dlg.h"
#include "math.h"
#include "stdio.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//global declarations

//integers
int cNumIts = 50000000;			// number of times to perform the calcs 

//doubles
double starttime;			// system ticks at start of loop
double stoptime;			// system ticks at end of loop
double Angle = 1.31266;		// an angle at which to get sine(angle)
double Base = 6.345;		// test values for x^y:  6.345^4.78
double Expon = 4.78;
double LogArg = 5.32;		// a value at which to get ln(LogArg)

// The look up tables
double SinLU[70001];		//look up table for sin(x)
double LogLU[70001];		//look up table for ln(x)
double exponLU[70001];		//look up table for x^y

/////////////////////////////////////////////////////////////////////////////
// CLookUp03Dlg dialog

CLookUp03Dlg::CLookUp03Dlg(CWnd* pParent /*=NULL*/)
	: CDialog(CLookUp03Dlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CLookUp03Dlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CLookUp03Dlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CLookUp03Dlg)
		// NOTE: the ClassWizard will add DDX and DDV calls here
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CLookUp03Dlg, CDialog)
	//{{AFX_MSG_MAP(CLookUp03Dlg)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_Initialize, OnInitialize)
	ON_BN_CLICKED(IDC_ExplicitSin, OnExplicitSin)
	ON_BN_CLICKED(IDC_Explicitln, OnExplicitln)
	ON_BN_CLICKED(IDC_Explicitpwr, OnExplicitpwr)
	ON_BN_CLICKED(IDC_LUTsin, OnLUTsin)
	ON_BN_CLICKED(IDC_LUTln, OnLUTln)
	ON_BN_CLICKED(IDC_LUTpwr, OnLUTpwr)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CLookUp03Dlg message handlers

BOOL CLookUp03Dlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
	
	// TODO: Add extra initialization here
	
	return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CLookUp03Dlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CLookUp03Dlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}

//Initialize the look-up tables
void CLookUp03Dlg::OnInitialize() 
{

	//local declarations
	//integers
	int InitCounter;		//counter for initialization of array elements

	//doubles
	double luAngle;			//angle for sin(angle)
	double luExpon;			//exponent for x^y function
	double luLogArg;		//argument for ln(x) function

	//initialize 70,000 elements for each array
	for(InitCounter=1; InitCounter <= 70000; ++InitCounter){

		//compute angle from counter, and assign sin() array
		luAngle = (double)InitCounter / (double)10000;
        SinLU[InitCounter] = sin(luAngle);
        
		//compute exponent from counter, and assign x^y array
        luExpon = (double)InitCounter / (double)10000;
        exponLU[InitCounter] = pow(Base, luExpon);
    
		//compute argument from counter, and assign ln(x) array
        luLogArg = (double)InitCounter / (double)10000;
        LogLU[InitCounter] = log(luLogArg);


	}
	
	//display that initialization is done.
	MessageBox("LUT Initialization Complete",
					 "Look-Up Table Initialization",
					 MB_OK | MB_ICONEXCLAMATION);

}

//iterative sin(x) by explicit calculation
void CLookUp03Dlg::OnExplicitSin() 
{

	//local declarations
	//integers
	int j;				//output string pointer
	int Counter;		//iteration counter
	
	//doubles
	double Value;		//return value for sin(x) function
	double ElapsedTime;	//elapsed time during loop
	
	//character strings
	char buffer[200];	//output string

	//start the timer
	starttime = GetTickCount();

	for(Counter=1; Counter <= cNumIts; ++Counter){

		//explicit computation of sin(x)
		Value = sin(Angle);

	}

	//stop the timer
	stoptime = GetTickCount();

	//compute elapsed time
	ElapsedTime = (stoptime - starttime) * 0.001;
	
	//build output string and display result in a message box
	j = sprintf(buffer,"Computed sin(x) 50,000,000 times in %g ",ElapsedTime);
	j += sprintf(buffer + j, "%s"," seconds. ");
	j += sprintf(buffer + j, "\nsin(1.31266) = %.15g ", Value);

	MessageBox(buffer,
					 "Explicit Sine Calculation",
					 MB_OK | MB_ICONEXCLAMATION);

	
	
} //end CLookUp03Dlg::OnExplicitSin()

//iterative ln(x) by explicit calculation
void CLookUp03Dlg::OnExplicitln() 
{

	//local declarations
	//integers		
	int j;					//output string pointer	
	int Counter;			//iteration counter
	
	//doubles
	double Value;			//return value for ln(x) function
	double ElapsedTime;		//elapsed time during loop
	
	//character strings
	char buffer[200];		//output string

	//start the timer
	starttime = GetTickCount();

	for(Counter=1; Counter <= cNumIts; ++Counter){

		//explicitly compute the ln(x)
		Value = log(LogArg);

	}

	//stop the timer
	stoptime = GetTickCount();

	//compute the elapsed time
	ElapsedTime = (stoptime - starttime) * 0.001;
	
	//build the output string and display the result in a message box
	j = sprintf(buffer,"Computed ln(x) 50,000,000 times in %g ",ElapsedTime);
	j += sprintf(buffer + j, "%s"," seconds. ");
	j += sprintf(buffer + j, "\nln(5.32) = %.15g ", Value);

	MessageBox(buffer,
					 "Explicit ln(x) Calculation",
					 MB_OK | MB_ICONEXCLAMATION);
	
	
} // end CLookUp03Dlg::OnExplicitln()

// iterative x^y by explicit calculation
void CLookUp03Dlg::OnExplicitpwr() 
{

	//local declarations
	//integers
	int j;					//output string pointer
	int Counter;			//iteration counter
	
	//doubles
	double Value;			//return value for pow(x,y) function
	double ElapsedTime;		//elapsed time during loop
	
	//character strings
	char buffer[200];		//output string

	//start the timer
	starttime = GetTickCount();

	for(Counter=1; Counter <= cNumIts; ++Counter){

		//explicitly compute x^y
		Value = pow(Base,Expon);

	}

	//stop the timer
	stoptime = GetTickCount();

	//compute the elapsed time
	ElapsedTime = (stoptime - starttime) * 0.001;
	
	//build the output string and display the result in a message box
	j = sprintf(buffer,"Computed x^y 50,000,000 times in %g ",ElapsedTime);
	j += sprintf(buffer + j, "%s"," seconds. ");
	j += sprintf(buffer + j, "\n6.345^4.78 = %.15g ", Value);

	MessageBox(buffer,
					 "Explicit x^y Calculation",
					 MB_OK | MB_ICONEXCLAMATION);

	
} // end CLookUp03Dlg::OnExplicitpwr()

// iterative sin(x) using look-up table
void CLookUp03Dlg::OnLUTsin() 
{

	//local declarations
	//integers
	int j;						//output string pointer
	int Counter;				//iteration counter
	int AngleIndex;				//lookup table index
	
	//doubles
	double Value;				//return value for lookup sin(x)
	double ElapsedTime;			//elapsed time during loop
	
	//character strings
	char buffer[200];			//output string

	//start the timer
	starttime = GetTickCount();

	for(Counter=1; Counter <= cNumIts; ++Counter){

		//compute the index from desired angle
		AngleIndex = (int)(Angle * (double)10000);

		//look-up sin(x)
		Value = SinLU[AngleIndex];

	}

	//stop the timer
	stoptime = GetTickCount();

	//compute the elapsed time
	ElapsedTime = (stoptime - starttime) * 0.001;
	
	//build the output string and display the result in a message box
	j = sprintf(buffer,"Looked Up sin(x) 50,000,000 times in %g ",ElapsedTime);
	j += sprintf(buffer + j, "%s"," seconds. ");
	j += sprintf(buffer + j, "\nsin(1.31266) = %.15g ", Value);

	MessageBox(buffer,
					 "Sin(x) Look Up",
					 MB_OK | MB_ICONEXCLAMATION);

	
} // end CLookUp03Dlg::OnLUTsin()

// iterative ln(x) using look-up table
void CLookUp03Dlg::OnLUTln() 
{
	//local declarations
	//integers
	int j;				//output string pointer
	int Counter;		//iteration counter
	int LogIndex;		//lookup table index
	
	//doubles
	double Value;		//return value for look up ln(x)
	double ElapsedTime;	//elapsed time during loop
	
	//character strings
	char buffer[200];	//output string

	//start the timer
	starttime = GetTickCount();

	for(Counter=1; Counter <= cNumIts; ++Counter){

		//compute the index from desired argument
		LogIndex = (int)(LogArg * (double)10000);

		//look up ln(x)
		Value = LogLU[LogIndex];

	}

	//stop the timer
	stoptime = GetTickCount();

	//compute the elapsed time
	ElapsedTime = (stoptime - starttime) * 0.001;
	
	//build the output string and display the result in a message box
	j = sprintf(buffer,"Looked Up ln(x) 50,000,000 times in %g ",ElapsedTime);
	j += sprintf(buffer + j, "%s"," seconds. ");
	j += sprintf(buffer + j, "\nln(5.32) = %.15g ", Value);

	MessageBox(buffer,
					 "ln(x) Look Up",
					 MB_OK | MB_ICONEXCLAMATION);
	
}

// iterative x^y using look-up table
void CLookUp03Dlg::OnLUTpwr() 
{
	//local declarations
	//integers
	int j;					//output string pointer
	int Counter;			//iteration counter
	int PwrIndex;			//lookup table index
	
	//doubles
	double Value;			//return value for lookup x^y		
	double ElapsedTime;		//elapsed time during loop
	
	//character strings
	char buffer[200];		//output string

	//start the timer
	starttime = GetTickCount();

	for(Counter=1; Counter <= cNumIts; ++Counter){

		//compute index from desired exponent
		PwrIndex = (int)(Expon * (double)10000);

		//look up pow(x,y)
		Value = exponLU[PwrIndex];

	}

	//stop the timer
	stoptime = GetTickCount();

	//compute the elapsed time
	ElapsedTime = (stoptime - starttime) * 0.001;
	
	//build the output string and display the result in a message box
	j = sprintf(buffer,"Looked Up x^y 50,000,000 times in %g ",ElapsedTime);
	j += sprintf(buffer + j, "%s"," seconds. ");
	j += sprintf(buffer + j, "\nsin(1.31266) = %.15g ", Value);

	MessageBox(buffer,
					 "x^y Look Up",
					 MB_OK | MB_ICONEXCLAMATION);
		
}
