// Other03Dlg.cpp : implementation file
//
//This program demonstrates the use of binary flag encoding
//as an alternative to multiple If decision  statements.
//
//The particular 'application' is to select an interaction energy
//type for the pair energy in a system with three types of particles.
//In this system, there are six possible interactions arising from
//the combinations of three particles.
//
//It is assumed that the energy for 'particle 1' is being computed.
//
//The first method to select this interaction energy type, one that
//is commonly used, is to use multiple If/Then (or, as in this example,
//If/ElseIf) statements.  See the function COther03Dlg::OnButton1().
//
//An alternative method, borrowed from a technique often used in
//system programming, is to encode the particle type in a binary
//flag structure (in this simple case, only a single int is used).
//With this data structure, no If/Then code structure is needed;
//the energy type is selected by simple integer addition or via
//an OR statement (as illustrated in function
// COther03Dlg::OnButton3()).


#include "stdafx.h"
#include "Other03.h"
#include "Other03Dlg.h"
#include "stdio.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define NUMITS 50000000

//global declarations

//integers
int r[50000001];		//random 1's,2's and 4's

/////////////////////////////////////////////////////////////////////////////
// CAboutDlg dialog used for App About

class CAboutDlg : public CDialog
{
public:
	CAboutDlg();

// Dialog Data
	//{{AFX_DATA(CAboutDlg)
	enum { IDD = IDD_ABOUTBOX };
	//}}AFX_DATA

	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CAboutDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:
	//{{AFX_MSG(CAboutDlg)
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
	//{{AFX_DATA_INIT(CAboutDlg)
	//}}AFX_DATA_INIT
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAboutDlg)
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
	//{{AFX_MSG_MAP(CAboutDlg)
		// No message handlers
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// COther03Dlg dialog

COther03Dlg::COther03Dlg(CWnd* pParent /*=NULL*/)
	: CDialog(COther03Dlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(COther03Dlg)
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void COther03Dlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(COther03Dlg)
	DDX_Control(pDX, IDC_BUTTON3, m_Button3);
	DDX_Control(pDX, IDC_BUTTON1, m_Button1);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(COther03Dlg, CDialog)
	//{{AFX_MSG_MAP(COther03Dlg)
	ON_WM_SYSCOMMAND()
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_BUTTON4, OnButton4)
	ON_BN_CLICKED(IDC_BUTTON1, OnButton1)
	ON_BN_CLICKED(IDC_BUTTON3, OnButton3)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// COther03Dlg message handlers

BOOL COther03Dlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Add "About..." menu item to system menu.

	// IDM_ABOUTBOX must be in the system command range.
	ASSERT((IDM_ABOUTBOX & 0xFFF0) == IDM_ABOUTBOX);
	ASSERT(IDM_ABOUTBOX < 0xF000);

	CMenu* pSysMenu = GetSystemMenu(FALSE);
	if (pSysMenu != NULL)
	{
		CString strAboutMenu;
		strAboutMenu.LoadString(IDS_ABOUTBOX);
		if (!strAboutMenu.IsEmpty())
		{
			pSysMenu->AppendMenu(MF_SEPARATOR);
			pSysMenu->AppendMenu(MF_STRING, IDM_ABOUTBOX, strAboutMenu);
		}
	}

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
	
	// TODO: Add extra initialization here
	
	return TRUE;  // return TRUE  unless you set the focus to a control
}

void COther03Dlg::OnSysCommand(UINT nID, LPARAM lParam)
{
	if ((nID & 0xFFF0) == IDM_ABOUTBOX)
	{
		CAboutDlg dlgAbout;
		dlgAbout.DoModal();
	}
	else
	{
		CDialog::OnSysCommand(nID, lParam);
	}
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void COther03Dlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR COther03Dlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}

//initialize the array of random particle types
void COther03Dlg::OnButton4() 
{

	//local declarations
	//integers
	int rcounter;				//counter used to initialize random array
	int randomval;


	//initialize the random number array
	for(rcounter = 1; rcounter <= NUMITS; ++rcounter){

		//rand() returns an integer < about 32000
		randomval = rand();
		
		//want random 1's, 2's, 4's for this demo
		if(randomval <= 10666) r[rcounter] = 1;
		if(randomval >= 21332) r[rcounter] = 4;
		if((randomval < 21332) & (randomval > 10666)) r[rcounter] = 2;
		
	}

	// Enable button controls;
	m_Button1.EnableWindow();  
	m_Button3.EnableWindow();	

}

//selection via if decision structure
void COther03Dlg::OnButton1() 
{

	//local declarations
	//integers
	int counter;		//iteration counter
	int j;				//character pointer
	int typeval;		//determined type

	//doubles
	double starttime;		//system ticks at start of loop
	double stoptime;		//system ticks at end of loop
	double ElapsedTime;		//system ticks during loop

	//character strings
	char buffer[200];		//output string

	//start the timer
	starttime = GetTickCount();		
	
	//select via if statement
	for(counter = 2; counter <= NUMITS; ++counter){

		//decision to select energy type
		if((r[counter] == 1) && (r[1] == 1)) typeval = 1;
		if((r[counter] == 2) && (r[1] == 1)) typeval = 3;
		if((r[counter] == 4) && (r[1] == 1)) typeval = 5;
		if((r[counter] == 1) && (r[1] == 2)) typeval = 3;
		if((r[counter] == 2) && (r[1] == 2)) typeval = 2;
		if((r[counter] == 4) && (r[1] == 2)) typeval = 6;
		if((r[counter] == 1) && (r[1] == 4)) typeval = 5;
		if((r[counter] == 2) && (r[1] == 4)) typeval = 6;
		if((r[counter] == 4) && (r[1] == 4)) typeval = 4;

	}

	//stop the timer
	stoptime = GetTickCount();		
    
	//compute the elapsed time, in seconds
    ElapsedTime = (stoptime - starttime) * .001;

	//display the result in a message box
	j=sprintf(buffer, "50,000,000 decisions in %g", ElapsedTime);
	j += sprintf(buffer + j, "%s", " seconds.\n");

	MessageBox(buffer, "Done If Decisions",
				MB_OK | MB_ICONEXCLAMATION);
	
}


//assignment of type via binary flag 
void COther03Dlg::OnButton3() 
{

	//local declarations
	//integers
	int counter;		//iteration counter
	int j;				//character pointer
	int typeval;		//determined type

	//doubles
	double starttime;		//system ticks at start of loop
	double stoptime;		//system ticks at end of loop
	double ElapsedTime;		//system ticks during loop

	//character strings
	char buffer[200];		//output string

	//start the timer
	starttime = GetTickCount();		
	
	//select via if statement
	for(counter = 2; counter <= NUMITS; ++counter){

		//decision to select energy type
		typeval = (r[counter] | r[1]);

	}

	//stop the timer
	stoptime = GetTickCount();		
    
	//compute the elapsed time, in seconds
    ElapsedTime = (stoptime - starttime) * .001;

	//display the result in a message box
	j=sprintf(buffer, "50,000,000 decisions in %g", ElapsedTime);
	j += sprintf(buffer + j, "%s", " seconds.\n");

	MessageBox(buffer, "Done If Decisions",
				MB_OK | MB_ICONEXCLAMATION);
		
}
