// Multi01Dlg.cpp : implementation file
//
// This program demonstrates the use of multi-threading.  A problem
// involving two distinct tasks is simulated: a purely background
// computational part and a graphical display part.  The tasks are 
// independent.
//
// The performance issues discussed here concern only a SINGLE PROCESSOR
// machine!  Additional benefit may occur for SMP machines when multithreading
// applications.
//
// The single thread version is in CMulti01Dlg::OnButton1().  In a single
// thread, the two tasks must be done sequentially.  For this demo, the 
// 'compute' part is done first, so there is a noticable delay before the
// graphics are updated.  The user perceives this as a 'hit.'
//
// The two thread version is in CMulti01Dlg::OnButton2().  Here, the graphical
// updating begins immediately, concurrent with the 'compute' part.  The use
// sees this as much faster, since there is no delay before seeing a result on
// the screen.  Also, since the graphics output takes time, the compute
// part continues in the background, so actual total time is less as well.

#include "stdafx.h"
#include "Multi01.h"
#include "Multi01Dlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//defines
#define loopend 700000000		//end of loop
#define sleeptime 25			//milliseconds to sleep during loop

//PROTOTYPES
DWORD WINAPI Loop_Thread(LPVOID data);		//worker thread function

/////////////////////////////////////////////////////////////////////////////
// CMulti01Dlg dialog

CMulti01Dlg::CMulti01Dlg(CWnd* pParent /*=NULL*/)
	: CDialog(CMulti01Dlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CMulti01Dlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CMulti01Dlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CMulti01Dlg)
	DDX_Control(pDX, IDC_PROGRESS1, m_Progress1);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CMulti01Dlg, CDialog)
	//{{AFX_MSG_MAP(CMulti01Dlg)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_BUTTON1, OnButton1)
	ON_BN_CLICKED(IDC_BUTTON2, OnButton2)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMulti01Dlg message handlers

BOOL CMulti01Dlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
	
	// TODO: Add extra initialization here
	
	return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CMulti01Dlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CMulti01Dlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}

//update progress bar and do a loop in a single thread
void CMulti01Dlg::OnButton1() 
{
	//local declarations
	//integers
	int oldpos;			//previous progress bar position; returned from SetPos()
	int b;				//output string pointer

	//doubles
	double starttime;	//ticks at start of process
	double stoptime;	//ticks at end of process
	double ElapsedTime;	//ticks during process
	
	//character strings
	char buffer[200];	//output string

	//start the timer
	starttime = GetTickCount();

	//do the 'worker' loop
	for (int j=1; j <= loopend;++j);

	//do the 'display' work
	for (int i=1; i <= 100; ++i){

		//advance the progress bar
		oldpos = m_Progress1.SetPos(i);

		//brief pause to simulate work
		Sleep(sleeptime);

	}

	//stop the timer
	stoptime = GetTickCount();

	//compute the elapsed time
	ElapsedTime = (stoptime - starttime) * 0.001;
	
	//build the output string and display in an message box
	b = sprintf(buffer, "%s%g","Done with both calculations in a single thread in ",ElapsedTime);
	b += sprintf(buffer + b, "%s"," seconds. ");

	MessageBox(buffer, "Single Thread Demo",
				MB_OK | MB_ICONEXCLAMATION);

	
	//reset progress bar to "0"
	oldpos = m_Progress1.SetPos(0);

	
}

//update progres bar and do a loop in two different threads
void CMulti01Dlg::OnButton2() 
{

	int oldpos;			//old progress bar position; returned from SetPos()
	int b;				//output string pointer

	//doubles
	double starttime;	//ticks at start of process
	double stoptime;	//ticks at end of process
	double ElapsedTime;	//ticks during process
	
	//character strings
	char buffer[200];	//output string

	//others
	HANDLE thread_handle;	//handle to created thread
	DWORD  thread_id;		//thread id; returned from CreateThread()

	//start the timer
	starttime = GetTickCount();

	//create the worker thread with API call
	//this thread uses the function Loop_Thread
	thread_handle = CreateThread(NULL,
					0,
					Loop_Thread,
					(LPVOID)0,
					0,
					&thread_id);
	
	//do the 'display' loop in main thread
	for (int i=1; i <= 100; ++i){

		//advance the progress bar
		oldpos = m_Progress1.SetPos(i);

		//brief pause to simulate work
		Sleep(sleeptime);

	}
	
	//main thread should wait for child to die naturally before
	//releasing handle to OS
	WaitForSingleObject(thread_handle, INFINITE);
	
	//release handle to OS
	CloseHandle(thread_handle);

	//stop the timer
	stoptime = GetTickCount();

	//compute the elapsed time
	ElapsedTime = (stoptime - starttime) * 0.001;

	//build the output string and display in an message box
	b = sprintf(buffer, "%s%g","Done with calculation in two threads in ",ElapsedTime);
	b += sprintf(buffer + b, "%s"," seconds. ");

	MessageBox(buffer, "Two Thread Demo",
				MB_OK | MB_ICONEXCLAMATION);

	
	//reset progress bar to "0"
	oldpos = m_Progress1.SetPos(0);
	
}

//Worker Thread Function
DWORD WINAPI Loop_Thread(LPVOID data)
{

	//do the 'worker' loop
	for (int j=1; j <= loopend;++j);

	//return the sent data
	return(0);

}