// Par01Dlg.cpp : implementation file
//
// Dot product demo comparing x87 to 3dNow! and SSE SIMD methods.


#include "stdafx.h"
#include "Par01.h"
#include "Par01Dlg.h"
#include "stdio.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// global declarations
static int cNumIts = 100000000;		//number of iterations

/////////////////////////////////////////////////////////////////////////////
// CPar01Dlg dialog

CPar01Dlg::CPar01Dlg(CWnd* pParent /*=NULL*/)
	: CDialog(CPar01Dlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CPar01Dlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CPar01Dlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CPar01Dlg)
		// NOTE: the ClassWizard will add DDX and DDV calls here
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CPar01Dlg, CDialog)
	//{{AFX_MSG_MAP(CPar01Dlg)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_BUTTON1, OnButton1)
	ON_BN_CLICKED(IDC_BUTTON2, OnButton2)
	ON_BN_CLICKED(IDC_BUTTON3, OnButton3)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CPar01Dlg message handlers

BOOL CPar01Dlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
	
	// TODO: Add extra initialization here
	
	return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CPar01Dlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CPar01Dlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}

//computes vector dot product via unrolled, but purely scalar, code
void CPar01Dlg::OnButton1() 
{
	//local declarations
	//integers
	int counter;			//iteration counter
	int j;					//buffer 'pointer'

	//singles
	float a[3];				//vector a
	float b[3];				//vector b
	float c;				//dot product result

	//doubles
	double starttime;		//system ticks at start of loop
	double stoptime;		//system ticks at loop exit
	double ElapsedTime;		//system ticks during loop

	//characters
	char buffer[200];		//output string

	//initialize the two vectors with some made up values
	a[0] = (float)23.45;
	a[1] = (float)14.65;
	a[2] = (float)33.87;
	a[3] = (float)0;

	b[0] = (float)36.99;
	b[1] = (float)21.32;
	b[2] = (float)17.38;
	b[3] = (float)0;

	//start the timer
	starttime = GetTickCount();		

	for(counter=1; counter <= cNumIts; ++counter){

		//note also eliminated a c=0 line!!
		
		//dot product in unrolled loop
		c = a[0] * b[0] + a[1] * b[1] + a[2] * b[2];


    }
    
	//stop the timer
    stoptime = GetTickCount();		
    
	//compute the elapsed time, in seconds
    ElapsedTime = (stoptime - starttime) * .001;

	//display the result in a message box
	j=sprintf(buffer, "Using Unrolled Loop, performed dot product 100,000,000 times %g", ElapsedTime);
	j += sprintf(buffer + j, "%s", " seconds.\n");
	j += sprintf(buffer + j, "%s%g", "Result = ",c);

	MessageBox(buffer, "Done with Unrolled Dot Product",
				MB_OK | MB_ICONEXCLAMATION);

	
}

// 3dNow! vector dot product, 3-d vectors
void CPar01Dlg::OnButton2() 
{
	//local declarations
	//integers
	int counter;			//iteration counter
	int j;					//buffer 'pointer'

	//singles
	float a[3];				//vector a, 4-d
	float b[3];				//vector b, 4-d
	float c;				//dot product result

	//doubles
	double starttime;		//system ticks at start of loop
	double stoptime;		//system ticks at loop exit
	double ElapsedTime;		//system ticks during loop

	//characters
	char buffer[200];		//output string

	//boolean
	bool bAMDOK;			//cpu supports 3DNow! instructions

	//test for 3DNow! capability in cpu
	//this is adapted from code in AMD 3DNow! Tech. Manual
	_asm {

				push	eax
				push	edx
				xor		eax, eax		; CPUid function Vendor ID
				CPUid
				test	eax, eax
				jz		$No3dNow		
				mov		eax, 80000000h	; CPUid function Largest Ext. Val
				CPUid
				cmp		eax, 80000001h
				jl		$No3dNow
				mov		eax, 80000001h	; CPUid function Sig. + features
				CPUid
				test	edx, 80000000h
				jnz		$Yes3dNow
				jmp		$No3dNow
$Yes3dNow:		mov		bAMDOK, 1		; bAMDOK = true
				jmp		$EndCpuTest
$No3dNow:		mov		bAMDOK, 0		; bAMDOK = false
$EndCpuTest:	pop		edx
				pop		eax

	}

	if(bAMDOK){

		//initialize the two vectors with some made up values
		a[0] = (float)23.45;
		a[1] = (float)14.65;
		a[2] = (float)33.87;
		a[3] = (float)0;

		b[0] = (float)36.99;
		b[1] = (float)21.32;
		b[2] = (float)17.38;
		b[3] = (float)0;

		//start the timer
		starttime = GetTickCount(); 	

		//switch registers to mmx mode
		_asm	femms

		for(counter=1; counter <= cNumIts; ++counter){

			
			//3DNow! dot product, have to use assembly
			//technically, we are doing 4-dimension vectors
			//here to avoid the movd instruction
			//also recall that [index] in ASSEMBLY refers to
			//actual byte offsets.
			__asm {

				movq		mm0, a[0]		; mm(0) = a(0) | a(1)
				movq		mm3, b[0]		; mm(1) = b(0) | b(1)
				pfmul		mm0, mm3		; mm(0) = a(0)*b(0) | a(1) * b(1)
				movq		mm2, a[8]		; don't use movd
				movq		mm1, b[8]
				pfacc		mm0, mm0		
				pfmul		mm1, mm2
				pfadd		mm0, mm1		
				movd		c, mm0			; c = mm(0)lo
			
			
			}

		}

		//switch registers back to x87 mode
		_asm	femms
	
		//stop the timer
		stoptime = GetTickCount();		
	
		//compute the elapsed time, in seconds
		ElapsedTime = (stoptime - starttime) * .001;

		//display the result in a message box
		j=sprintf(buffer, "Using 3DNow!, performed dot product 100,000,000 times %g", ElapsedTime);
		j += sprintf(buffer + j, "%s", " seconds.\n");
		j += sprintf(buffer + j, "%s%g", "Result = ",c);

		MessageBox(buffer, "Done with 3DNow! Dot Product",
				MB_OK | MB_ICONEXCLAMATION);

	}
		
	else {

		MessageBox("This Processor does not support 3DNow! Instructions","No 3DNow!",
				MB_OK | MB_ICONEXCLAMATION);
	
	}
	
	
}

//dot product using SSE SIMD
void CPar01Dlg::OnButton3() 
{
	//general local declarations
	//integers
	int counter;		//iteration counter
	int bSSEOK;			//cpu supports SSE instructions
	int j;				//string position pointer

	//floats
	float a[4];			//vector a, 4-d
	float b[4];			//vector b, 4-d
	float c;			//dot product result
	
	//variables for time execution time
	//doubles
	double starttime;		//system ticks at start of loop
	double stoptime;		//system ticks at loop exit
	double ElapsedTime;		//system ticks during loop

	//character strings
	char buffer[200];			//string to display in message box

	//initialize SSE flag to zero
	bSSEOK = 0;

	//test for SSE capability in cpu
	_asm {

			push	eax
			push	edx
			xor		eax, eax		; CPUid function Vendor ID
			CPUid
			test	eax, eax
			jz		$NoSSE		
			mov		eax, 00000001h	; CPUid function Largest Ext. Val
			CPUid
			test	eax, 02000000h
			jl		$YesSSE
			jmp		$NoSSE
$YesSSE:	mov		bSSEOK, 1		; bSSEOK = true
			jmp		$EndCpuTest
$NoSSE:		mov		bSSEOK, 0		; bSSEOK = false
$EndCpuTest:		pop		edx
			pop		eax

	}

	if(bSSEOK){

		//initialize the two vectors with some made up values
		a[0] = (float)23.45;
		a[1] = (float)14.65;
		a[2] = (float)33.87;
		a[3] = (float)0;

		b[0] = (float)36.99;
		b[1] = (float)21.32;
		b[2] = (float)17.38;
		b[3] = (float)0;

		//start the timer
		starttime = GetTickCount(); 	

		//switch registers to mmx mode
		_asm	femms

		for(counter=1; counter <= cNumIts; ++counter){

			
			//SSE dot product, have to use assembly
			//technically, we are doing 4-dimension vectors
			//to make full use of 128 bit SSE registers
			//in essence, we get the 4th d for free
			__asm {

				movaps		xmm0, a[0]
				movaps		xmm1, b[0]
				mulps		xmm0, xmm1
				movaps		xmm1, xmm0
				shufps		xmm1, xmm0, 0xee
				addps		xmm1, xmm0
				movaps		xmm0, xmm1
				shufps		xmm1, xmm1, 0xe5
				addps		xmm1, xmm0
				movss		xmm0, c
			
			
			}

		}

		//switch registers back to x87 mode
		_asm	femms
	
		//stop the timer
		stoptime = GetTickCount();		
	
		//compute the elapsed time, in seconds
		ElapsedTime = (stoptime - starttime) * .001;

		//display the result in a message box
		j =  sprintf(buffer, "Using SSE, performed dot product 100,000,000 times %g", ElapsedTime);
		j += sprintf(buffer + j, "%s", " seconds.\n");
		j += sprintf(buffer + j, "%s%g", "Result = ",c);

		MessageBox(buffer, "Done with SSE Dot Product",
				MB_OK | MB_ICONEXCLAMATION);

		
	}
		
	else {

		MessageBox("This Processor does not support SSE Instructions","No SSE",
				MB_OK | MB_ICONEXCLAMATION);
			
	}	
}
